/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/


/*
 * AppShutdown
 * 
 * This class provides an hook to exit the application. Currently
 * it waits for the updater to finish, because data saving and syncing
 * is managed by the EnvironmentUpdater and DataSynchronizer threads,
 * run respectively when no request are to be done and every X seconds
 * (default: 10 seconds)
 */
package application;

import java.io.IOException;

import server.MultithreadedServer;
import sessions.SessionManager;
import debug.Debuginfo;

/**
 * Application shutdown hook. This thread is started when the Java
 * virtual machine is about to exit (due to a termination signal)
 * 
 * @author mriya3
 *
 */
public class AppShutdown extends Thread {
	private MultithreadedServer server;
	
	SessionManager smanager;
	
	public AppShutdown(SessionManager sm, MultithreadedServer server) {
		this.server = server;
		this.smanager = sm;
	}
	
	/**
	 * On wait for the updater to finish his work
	 * Meanwhile show a text message, then exit cleanly
	 */
	public void run() {
		new Debuginfo(1, "Requesting forced shut down");
		try { server.stopServing();
		} catch (IOException e1) {}
		if(!(smanager.getRootEnvironment().getSyncedStatus()) || (smanager.getRootEnvironment().isModified())) {
			new Debuginfo(11, "Waiting for data to be synced on disk...");
			while(!(smanager.getRootEnvironment().getSyncedStatus()) && (smanager.getRootEnvironment().isModified())) {
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
				}
				System.out.print(".");
			}
			System.out.print("done!\n");
		}
		new Debuginfo(1,"XML data flushed to disk");
		
	}
}
