/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Configuration management
 * 
 * To store internal configuration we use an instance of this class. Upon start of
 * the application, an instance of this class is created. A configuration file stored
 * on disk is read and used to load the properties object. Next some values may be 
 * overridden by using the command line switches
 */
package application;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.InvalidPropertiesFormatException;
import java.util.Properties;

import debug.Debuginfo;

/**
 * Configuration registry
 * It is used to store key,value pairs and its loaded from disk at startup
 * 
 * @author mriya3
 *  
 */
public class ConfigurationRegistry {
	
	/* Name of the file storing the configuration */
	private String filename = null;
	
	/* The properties object */
	protected Properties props = new Properties();
	
	/**
	 * Initializes a new Configuration registry
	 * 
	 * @param filename
	 */
	public ConfigurationRegistry(String filename) {
		new Debuginfo(1, "Configuration file (not verified) <" + filename + ">");
		this.filename = filename;
		loadConfiguration(null);
	}
	
	/**
	 * Returns a boolean value
	 * 
	 * @param key
	 * @param defvalue
	 * @return
	 */
	public boolean getBooleanValue(String key, boolean defvalue) {
		if (!(props.getProperty(key) == null))
			return new Boolean(props.getProperty(key).toString())
			.booleanValue();
		else
			return defvalue;
	}
	
	/**
	 * Returns a double value
	 * 
	 * @param key
	 * @param defvalue
	 * @return
	 */
	public double getDoubleValue(String key, double defvalue) {
		if (!(props.getProperty(key) == null))
			return new Integer(props.getProperty(key).toString()).doubleValue();
		else
			return defvalue;
	}
	
	/**
	 * Returns an integer value of defvalue
	 * 
	 * @param key
	 * @param defvalue
	 * @return
	 */
	public int getIntValue(String key, int defvalue) {
		if (!(props.getProperty(key) == null))
			return new Integer(props.getProperty(key).toString()).intValue();
		else
			return defvalue;
	}
	
	/**
	 * Returns a long value
	 * 
	 * @param key
	 * @param defvalue
	 * @return
	 */
	public long getLongValue(String key, long defvalue) {
		if (!(props.getProperty(key) == null))
			return new Integer(props.getProperty(key).toString()).longValue();
		else
			return defvalue;
	}
	
	/**
	 * Returns a string value from the properties or defvalue (if specified) if
	 * there is no key.
	 * 
	 * @param key
	 * @param defvalue
	 * @return
	 */
	public String getStringValue(String key, String defvalue) {
		if (!(props.getProperty(key) == null))
			return props.getProperty(key);
		else
			return defvalue;
	}
	
	/**
	 * Loads configuration parameters from an XML file
	 * 
	 * @param configfile
	 */
	public void loadConfiguration(String configfile) {
		if (configfile == null)
			configfile = this.filename;
		FileInputStream propsfile = null;
		try {
			propsfile = new FileInputStream(this.filename);
			try {
				this.props.loadFromXML(propsfile);
			} catch (InvalidPropertiesFormatException e1) {
				this.filename = null;
				new Debuginfo(2, "Invalid configuration file");
			} catch (IOException e1) {
				this.filename = null;
				new Debuginfo(2, "Error reading configuration file");
			}
		} catch (FileNotFoundException e) {
			this.filename = null;
			new Debuginfo(2, "Cannot open configuration file <" + configfile
					+ ">");
		}
	}
	
	/**
	 * Saves configuration parameters to an XML file
	 *  
	 */
	public void saveConfiguration() {
		FileOutputStream propsfile = null;
		try {
			propsfile = new FileOutputStream(this.filename);
			new Debuginfo(1, "Configuration saved to <" + this.filename + ">");
		} catch (FileNotFoundException e) {
			new Debuginfo(2, "Cannot save configuration to file (file not found)");
			return;
		} catch (Exception e) {
			new Debuginfo(2, "Cannot save configuration to file");
			return;
		}
		try {
			this.props.storeToXML(propsfile, "RoXanneConfiguration");
			propsfile.close();
		} catch (Exception e1) {
			new Debuginfo(2, "Error saving configuration file");
		}
	}
	
	/**
	 * Sets a boolean value
	 * 
	 * @param key
	 * @param value
	 */
	public void setBooleanValue(String key, boolean value) {
		props.setProperty(key, String.valueOf(value));
	}
	
	/**
	 * Sets a double value
	 * 
	 * @param key
	 * @param value
	 */
	public void setDoubleValue(String key, Double value) {
		props.setProperty(key, String.valueOf(value));
	}
	
	/**
	 * Sets a new integer value
	 * 
	 * @param key
	 * @param value
	 */
	public void setIntValue(String key, int value) {
		props.setProperty(key, String.valueOf(value));
	}
	
	/**
	 * Sets a long value
	 * 
	 * @param key
	 * @param value
	 */
	public void setLongValue(String key, long value) {
		props.setProperty(key, String.valueOf(value));
	}
	
	/**
	 * Sets a string value in the configuration
	 * 
	 * @param key
	 * @param value
	 */
	public void setStringValue(String key, String value) {
		this.props.setProperty(key, value);
	}
	
	/**
	 * Reloads the configuration from the original file
	 */
	public void reloadConfig() {
		loadConfiguration(null);
	}
	
}
