/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Init
 * 
 * This is the main object that creates the needed
 * objects to manage the database. Upon init a root environment
 * is created from the base XML data read from the disk.
 */
package application;

import jargs.gnu.CmdLineParser;

import java.io.File;
import java.io.IOException;
import java.security.Security;

import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import server.MultithreadedServer;
import sessions.SessionManager;
import debug.Debuginfo;
import environments.RootEnvironment;

/**
 * Init class
 * This class defines the main method to setup components and start the
 * complete application.
 * 
 * @author mriya3
 *
 */
public class Init {
	
	static final int BUILD = 1316;
	
	static final String CODENAME = "Quahog";
	
	static final boolean DEVELOPMENTVERSION = true;
	
	static final int MAJOR = 0;
	
	static final int MINOR = 8;
	
	static final int RELEASE = 16;
	
	public static void main(String args[]) {
		/* Set default debug level (not maximum) */
		Debuginfo.setDebugLevel(1);
		new Debuginfo(1, "roXanne Framework Init, x.core " + MAJOR + "."
				+ MINOR + "." + RELEASE + "." + BUILD + "-" + CODENAME);
		new Debuginfo(1,
				"Copyright (C)2004-2005 Amos Brocco <amos.brocco@unifr.ch>");
		new Debuginfo(1,
				"This program is free software; you can redistribute it and/or");
		new Debuginfo(1,
				"modify it under the terms of the GNU General Public License");
		new Debuginfo(1,
				"as published by the Free Software Foundation; either version 2");
		new Debuginfo(1,
				"of the License, or (at your option) any later version.");
		new Debuginfo(1,
				"This program is distributed in the hope that it will be useful,");
		new Debuginfo(1,
				"but WITHOUT ANY WARRANTY; without even the implied warranty of");
		new Debuginfo(1,
				"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the");
		new Debuginfo(1, "GNU General Public License for more details.");
		new Debuginfo(1,
				"You should have received a copy of the GNU General Public License");
		new Debuginfo(1,
				"along with this program; if not, write to the Free Software");
		new Debuginfo(
				1,
				"Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.");
		new Debuginfo(
				1,
				"This application uses and contains parts (either modified under the terms of the");
		new Debuginfo(1,
				"respective licenses or unmodified) of the following software:");
		new Debuginfo(1,
				"	Bouncy Castle Crypto APIs (C) 2000 - 2004 The Legion Of The Bouncy Castle ");
		new Debuginfo(1,
				"	JDOM Copyright (C) 2004 Jason Hunter & Brett McLaughlin.");
		new Debuginfo(1,
		"	JAXEN  Copyright 2003 (C) The Werken Company. All Rights Reserved.");
		new Debuginfo(1, "	JAXUP  Copyright 2002-2003 Erwin Bolwidt.");
		new Debuginfo(1, "	Base64 Tool (C) Robert Harder");
		new Debuginfo(1, "	JArgs (C) 2001-2003 Steve Purcell (C) 2002 Vidar Holen (C) 2002 Michal Ceresna (C) 2005 Ewan Mellor.");
		new Debuginfo(
				1,
		"and others. Please refer to documentation or source code for complete copyright info");
		new Debuginfo(1, "Available memory " + Runtime.getRuntime().maxMemory()
				+ " bytes, used " + Runtime.getRuntime().freeMemory()
				+ " bytes");
		new Debuginfo(1, "JVM: " + System.getProperty("java.version")
				+ " from " + System.getProperty("java.vendor") + " running on "
				+ System.getProperty("os.name") + "/"
				+ System.getProperty("os.arch"));
		/*
		 * Parse command line options and update configuration/configuration
		 * file
		 */
		CmdLineParser cmdline = new CmdLineParser();
		/* Set command line options available */
		CmdLineParser.Option configfile = cmdline.addStringOption('c',
		"configfile");
		CmdLineParser.Option xmlfile = cmdline.addStringOption('x', "xmlfile");
		CmdLineParser.Option debug = cmdline.addIntegerOption('d', "debug");
		CmdLineParser.Option help = cmdline.addBooleanOption('h', "help");
		/* Try parsing command line */
		try {
			cmdline.parse(args);
		} catch (CmdLineParser.OptionException e) {
			System.err.println(e.getMessage());
			showHelpUsage(2);
		}
		/* Need help ? */
		if((Boolean)cmdline.getOptionValue(help, Boolean.FALSE))
			showHelpUsage(0);
		
		/* Load configuration from file */
		ConfigurationRegistry configurator = new ConfigurationRegistry(
				(String) cmdline.getOptionValue(configfile,"roxanne.conf"));
		
		/* Set debug level */
		Integer debuglevel = (Integer) cmdline.getOptionValue(debug);
		if (!(debuglevel == null) && !(debuglevel.intValue() < 0)
				&& !(debuglevel.intValue() > 8)) {
			Debuginfo.setDebugLevel(debuglevel.intValue());
		}
		/* Get current directory */
		String currDir = "";
		try {
			currDir = new File(".").getCanonicalPath();
		} catch (IOException e3) {
			new Debuginfo(1, "Cannot get current directory");
		}
		
		/* Get root XML document filename */
		String xmlrootfile = (String) cmdline.getOptionValue(xmlfile);
		if (xmlrootfile == null)
			xmlrootfile = configurator.getStringValue("XMLdbfile", "root.xml");
		/* Create the root environment */
		RootEnvironment renv = new RootEnvironment();
		Document doc = null;
		/* Build root environment */
		SAXBuilder sb = new SAXBuilder();
		try {
			/* Load root XML file from disk */
			File basefile = new File(currDir + "/" + xmlrootfile);
			new Debuginfo(1, "Root XML file path is " + basefile + " ("
					+ basefile.length() + " bytes)");
			doc = sb.build(basefile);
		} catch (JDOMException e1) {
			new Debuginfo(3, "Cannot build root environment from " + currDir
					+ " <" + xmlrootfile + "> :\n\t"+e1.getMessage());
		} catch (IOException e1) {
			new Debuginfo(3, "Failed to read base XML in file <" + currDir
					+ "/" + xmlrootfile+">");
		} catch (NullPointerException e) {
			new Debuginfo(3, "Cannot open root XML file " + currDir + " <"
					+ xmlrootfile + "> :\n\t"+e.getMessage());
		} catch (Exception e) {
			new Debuginfo(3, "Cannot build root environment from " + currDir
					+ " <" + xmlrootfile + "> :\n\t"+e.getMessage());
		}
		/*
		 * If the base XML document was empty or not valid, set a default
		 * template
		 */
		if (doc == null) {
			doc = new Document();
			doc.addContent(new Element(configurator.getStringValue(
					"defaultRootNodeName", "roxannedb")));
		}
		renv.setBaseNode(null);
		renv.setRootDocument(doc);
		/* Set the root node & the environment link */
		doc.setEnvironment(renv);
		renv.setRootNode(doc.getRootElement());
		renv.recursiveSetEnvironmentLink(renv.getRootNode(), renv);
		renv.setUnmodified();
		/* Create a new SessionManager */
		SessionManager mainSessionManager = new SessionManager(configurator,
				renv);
		
		/* Initialize the server */
		MultithreadedServer server = new MultithreadedServer(mainSessionManager);
		mainSessionManager.setServer(server);
		
		/* Add bouncycastle provider */
		Security.addProvider(new BouncyCastleProvider());
		
		/* Configure server */
		configurator.setIntValue("ServerRequestTimeout", 1500);
		server.loadConfig(configurator);
		configurator.saveConfiguration();
		
		/* Install signal handler */
		AppShutdown asd = new AppShutdown(mainSessionManager, server);
		Runtime.getRuntime().addShutdownHook(asd);
		
		try {
			/* Now enter busy wait cycle for connections */
			Debuginfo.setDebugLevel(1);
			new Debuginfo(1, "Entering server wait for connection cycle...");
			server.startServing();
		} catch (Exception e2) {
			new Debuginfo(6, "Cannot start server. Aborting");
			System.exit(0);
		}
		// On exit terminate cleanly. Save XML to disk
	}
	
	/**
	 * Shows a little help
	 */
	private static void showHelpUsage(int exitstatus) {
		System.out
		.println("Usage: ./xcore.sh [-help] [-debug <debuglevel>] [-configfile <configfile>] [-xmlfile <xmlconfiguration>]\n");
		System.out.println("\t\t--help | -h\t\tShow this help message");
		System.out.println("\t\t--debug | -d\t\tSet debug level (default: 1)");
		System.out.println("\t\t--configfile | -c\tSpecify configuration file (default: roxanne.conf)");
		System.out.println("\t\t--xmlfile | -x\t\tSpecify root XML file");
		System.exit(exitstatus);
	}
	
}