/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Environment management
 * 
 * Environments are used to provide transparent decryption and encrypted data
 * access to the user. As the user access an encrypteddata node, the system
 * tries to perform transparent decryption. If that works, a new environment
 * is created containing the decoded XML data structure. Now, instead of returning
 * the real node data, the root of the environment is used.
 */
package environments;

import java.io.IOException;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.output.XMLOutputter;

import debug.Debuginfo;

import security.EncryptionException;
import security.KeyNotFoundException;
import security.XMLDecrypt;
import security.XMLEncrypt;
import sessions.Session;

/**
 * EnvironmentManager
 * 
 * This class tracks environment creation and mantains an mapping between base
 * nodes and environments
 * 
 * @author mriya3
 *  
 */
public class EnvironmentManager {
	private boolean disableNewEnvironmentCreation;

	private boolean disableNodeVisit;

	private Vector<Environment> envlist;

	private Hashtable<Element, Environment> nodeEnvironmentMapping;

	private boolean notifyModify;

	private boolean notifyRootUnsynced;

	private RootEnvironment rootEnvironment;

	private Session session;

	boolean writeCheck = false;

	public EnvironmentManager(Session session) {
		this.session = session;
		this.nodeEnvironmentMapping = new Hashtable<Element, Environment>();
		this.envlist = new Vector<Environment>();
		this.rootEnvironment = (RootEnvironment) session.getRootEnvironment();
	}

	public boolean checkWrites() {
		return this.writeCheck;
	}

	/**
	 * Closes all created environments (normally performed at the end of
	 * transactions). This method uses the unsafe closing method because we
	 * close environment in reverse order of creation, ensuring that 'bottom'
	 * environments always get closed before 'upper' ones.
	 */
	public void closeAllEnvironments() {
		updateAllEnvironments();
		envlist.clear();
		nodeEnvironmentMapping.clear();
	}

	/**
	 * Safe closing of the environment: also closes all environments below
	 * 
	 * @param env
	 */
	public void closeEnvironment(Environment env) {
		/*
		 * First, find all environments that have this environment as parent and
		 * close them
		 */
		Iterator<Environment> enviter = envlist.iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if (tmpenv.getParent().equals(env))
				closeEnvironment(tmpenv);
		}
		unsafeCloseEnvironment(env);
	}

	/**
	 * Closes all sub environments of the provided environment
	 * 
	 * @param env
	 */
	public void closeSubEnvironments(Environment env) {
		/*
		 * Find all environments that have this environment as parent and close
		 * them
		 */
		Iterator<Environment> enviter = ((Vector) envlist.clone()).iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if (tmpenv.getParent().equals(env))
				closeEnvironment(tmpenv);
		}
	}

	/**
	 * Deletes all dead (unlinked) environments
	 *  
	 */
	public void deleteAllDeadEnvironments() {
		if (envlist.size() > 0) {
			Vector origEnvList = (Vector) envlist.clone();
			for (int i = origEnvList.size() - 1; i >= 0; i--) {
				Environment env = (Environment) origEnvList.get(i);
				if (env.isDead()) {
					envlist.remove(env);
					nodeEnvironmentMapping.remove(env.getBaseNode());
					this.notifyRootUnsynced = true;
					this.rootEnvironment.setUnSynced();
				}

			}
		}
	}

	/**
	 * Removes the given environment; this is a safe method because it first
	 * calls a remove of all "child" environments recursively, ensuring that all
	 * sub-environments get removed from the list.
	 * 
	 * @param env
	 */
	public void deleteEnvironment(Environment env) {
		/*
		 * First, find all environments that have this environment as parent and
		 * close them
		 */
		Iterator<Environment> enviter = envlist.iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if (tmpenv.getParent().equals(env))
				deleteEnvironment(tmpenv);
		}
		unsafeDeleteEnvironment(env);
	}

	/**
	 * Disables environment creation; used when we need to get, for example, the
	 * XML code for encryption and we don't want that sub nodes get decrypted
	 */
	public void disableEnvCreation() {
		this.disableNewEnvironmentCreation = true;
	}

	public void disableModificationCheck() {
		this.writeCheck = false;
	}

	public void disableModificationNotify() {
		this.notifyModify = false;
	}

	/**
	 * Disables node visiting; this means that encrypted nodes are returned 'as
	 * they are', without transparent decryption.
	 */
	public synchronized void disableNodeVisiting() {
		this.disableNodeVisit = true;
	}

	/**
	 * @return
	 */
	public boolean enabledModifyNotify() {
		return this.notifyModify;
	}

	/**
	 * (Re-)enables environment creation
	 */
	public void enableEnvCreation() {
		this.disableNewEnvironmentCreation = false;
	}

	/**
	 * Enables checking if modification is possible or not
	 *
	 */
	public void enableModificationCheck() {
		this.writeCheck = true;
	}

	/**
	 * Enables notification of data alteration
	 *
	 */
	public void enableModificationNotify() {
		this.notifyModify = true;
	}

	/**
	 * (Re-)enables node visiting (i.e. transparent node decryption)
	 */
	public synchronized void enableNodeVisiting() {
		this.disableNodeVisit = false;
	}

	/**
	 * Returns the environment associated with a node; note that this method
	 * returns null if the environment does not exist.
	 */
	public Environment getEnvironment(Element basenode) {
		return (Environment) this.nodeEnvironmentMapping.get(basenode);
	}

	/**
	 * Returns the list of current environments
	 * 
	 * @return
	 */
	public Vector<Environment> getEnvironmentList() {
		setDeadOrAlive();
		deleteAllDeadEnvironments();
		return this.envlist;
	}

	/**
	 * Returns the root environment
	 * 
	 * @return
	 */
	public RootEnvironment getRootEnvironment() {
		return this.rootEnvironment;
	}

	/**
	 * Returns true if the content as an associated environment
	 * 
	 * @param content
	 * @return
	 */
	public boolean hasEnvironment(Element content) {
		return nodeEnvironmentMapping.containsKey(content);
	}

	/**
	 * Returns if an enviroment is yet linked to the tree
	 * 
	 * @param env
	 * @return
	 */
	public boolean isAlive(Environment env) {
		if (env.isAlive())
			return true;
		else if (env.isDead())
			return false;
		else {
			boolean value = isLinkedNode(env.getBaseNode());
			if (value)
				env.setAlive();
			else
				env.setDead();
			return env.isAlive();
		}
	}

	/**
	 * Returns if a node is not linked to the tree
	 * 
	 * @param e
	 * @return
	 */
	public boolean isLinkedNode(Element e) {
		if (e.getParentElement() == null)
			return false;
		else if (e.getParentElement() == rootEnvironment.getRootNode())
			return true;
		else
			return isLinkedNode(e.getParentElement());
	}

	/**
	 * Returns if the visit node feature is enabled
	 * 
	 * @return
	 */
	public synchronized boolean isVisitNodeEnabled() {
		return !(this.disableNodeVisit);
	}

	/**
	 * Returns true if the node is a base node for an environment;
	 * 
	 * @param node
	 * @return
	 */
	public boolean nodeHasEnvironment(Element node) {
		return this.nodeEnvironmentMapping.containsKey(node);
	}

	/**
	 * If we remove the read key we should update and close all environments
	 * using it
	 * 
	 * @param keyID
	 */
	public void onReadKeyRemoved(String keyID, String method) {
		for (int i = envlist.size() - 1; i >= 0; i--) {
			Environment env = envlist.get(i);
			if (!env.equals(null) && env.getKeyName().equals(keyID)
					&& env.getAlgorithm().equals(method))
				closeEnvironment(env);
		}

	}

	/**
	 * When we add a write key we make all environments depending on that key as
	 * writable unless the parent environment is not readonly
	 * 
	 * @param keyID
	 */
	public void onWriteKeyAdded(String keyID, String method) {
		Iterator<Environment> enviter = envlist.iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if (!(tmpenv.getParent() == null)
					&& !(tmpenv.getParent().isReadonly())
					&& tmpenv.getKeyName().equals(keyID)
					&& tmpenv.getAlgorithm().equals(method))
				tmpenv.setWritable();
		}
	}
	
	
	/**
	 * Set environments that use a certain key as modified
	 * 
	 * @param keyID
	 */
	public void forceUpdateEnvByKey(String keyID, String method) {
		Iterator<Environment> enviter = envlist.iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if (!(tmpenv.getParent() == null)
					&& tmpenv.getKeyName().equals(keyID)
					&& tmpenv.getAlgorithm().equals(method))
				tmpenv.setModified();
		}
	}

	/**
	 * If we remove a write key we should update the environment using it and
	 * set it readonly as well as set read only all sub environments.
	 * 
	 * @param keyID
	 */
	public void onWriteKeyRemoved(String keyID, String method) {
		for (int i = envlist.size() - 1; i >= 0; i--) {
			Environment env = envlist.get(i);
			if (!env.equals(null) && env.getKeyName().equals(keyID)
					&& env.getAlgorithm().equals(method)) {
				updateEnvironment(env);
				env.setReadonly();
				setSubEnvironmentsAsReadonly(env);
			}
		}

	}

	/**
	 * Evaluates if the root environments is actually modified or not
	 * 
	 * @return
	 */
	public boolean rootIsUpToDate() {
		if (notifyRootUnsynced)
			return false;
		if (envlist.size() > 0) {
			for (int i = envlist.size() - 1; i >= 0; i--) {
				if (envlist.get(i).isModified())
					return false;
			}
		}
		return true;
	}

	/**
	 * Set all environments as unknown (nor dead nor alive)
	 *  
	 */
	public void setAllEnvironmentsAsUnknown() {
		if (envlist.size() > 0) {
			for (int i = envlist.size() - 1; i >= 0; i--) {
				Environment env = (Environment) envlist.get(i);
				env.setUnknown();
			}
		}
	}

	/**
	 * Sets all environments as dead or alive
	 *  
	 */
	public void setDeadOrAlive() {
		if (envlist.size() > 0) {
			for (int i = envlist.size() - 1; i >= 0; i--) {
				Environment env = (Environment) envlist.get(i);
				if (!isAlive(env))
					setSubEnvironmentsAsDead(env);
			}
		}
	}

	/**
	 * Creates an environment based on the provided node and returns the root
	 * node of the environment. If the environment already exist then no new
	 * environment will be created. If the base node is not encrypted then it is
	 * returned as it is.
	 */
	public Element setEnvironment(Element basenode) {
		/*
		 * Fist we check if there is already an environment for this node If
		 * there is already an environment simply return it
		 */
		if (this.nodeHasEnvironment(basenode)) {
			return this.getEnvironment(basenode).getRootNode();
		}
		//if (!(this.getEnvironment(basenode) == null))
		//	return this.getEnvironment(basenode).getRootNode();
		if (this.disableNewEnvironmentCreation) {
			return basenode;
		} else {
			/* Second we check that the node is an encrypted one */
			if (!(basenode.isEncryptedNode())) {
				return basenode;
			}
			/*
			 * Then we should look at Node contents and get the encrypted XML
			 * (XML-Sec standard) to get the encryption algorithm and encryption
			 * key.
			 */
			disableNodeVisiting();
			Element encdataelement = (Element) basenode.clone();
			enableNodeVisiting();
			Element root = null;
			XMLDecrypt xdec;
			xdec = new XMLDecrypt(encdataelement, this.session.getKeyRing());
			try {
				Document decdoc = xdec.getDecryptedData();
				if (decdoc == null)
					return basenode;
				root = decdoc.detachRootElement();
				// Sets the parent link
				root.setParentLink(basenode.getParent());
			} catch (Exception e) {
				new Debuginfo(2, "Cannot decrypt node");
				return basenode;
			}
			/* Create a new environment object */
			Environment newenv = new Environment(this);
			/* Sets the used keyid and algorithm name in the environment */
			newenv.setKeyName(xdec.getKeyName());
			newenv.setAlgorithm(xdec.getAlgorithmName());
			/* Set parent environment link */
			Environment parent = basenode.getEnvironment();
			newenv.setParent(parent);
			/* Attach the document root to the environment */
			newenv.setRootNode(root);
			newenv.setBaseNode(basenode);
			/* Set the environment manager link to each node */
			newenv.recursiveSetEnvironmentLink(root, newenv);
			/* This (virtually) also sets the base source node link to each node */
			/*
			 * Was the environment responsible for the base node, read only? If
			 * yes, set also this new environment as read-only.
			 */
			if (basenode.getEnvironment().isReadonly()) {
				newenv.setReadonly();
			}
			/*
			 * If the user has no private key for the given keyID then set the
			 * environment as readonly
			 */
			if (!(newenv.getKeyName() == null))
				if (!this.session.getKeyRing().hasEncryptionKey(
						newenv.getKeyName(), newenv.getAlgorithm()))
					newenv.setReadonly();
			/* Register the environment object in the hashtable */
			this.nodeEnvironmentMapping.put(basenode, newenv);
			/*
			 * Add the environment also in the envlist so that it will be
			 * possible to track creation order
			 */
			this.envlist.add(newenv);
			/* Finished ! Return the root of the environment */
			return newenv.getRootNode();
		}
	}

	/**
	 * Sets all sub environments as unknown
	 *  
	 */
	public void setOnlySubEnvironmentsAsUnknown(Environment env) {
		/*
		 * First, find all environments that have this environment as parent and
		 * set them
		 */
		Iterator<Environment> enviter = envlist.iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if ((tmpenv.getParent().equals(env)))
				setSubEnvironmentsAsUnknown(tmpenv);
		}
	}

	/**
	 * Sets the root environment; this method is normally called by a Session
	 * object. The root environment should not be changed, and is never
	 * explicitly closed.
	 * 
	 * @param env
	 */
	public void setRootEnvironment(RootEnvironment env) {
		this.rootEnvironment = env;
	}

	/**
	 * Mark the root environment as in-sync with other environments
	 *  
	 */
	public void setRootSynced() {
		notifyRootUnsynced = false;
		this.rootEnvironment.setSynced();
	}

	/**
	 * Mark the root enviroment as out-of-sync with other environments
	 *  
	 */
	public void setRootUnSynced() {
		notifyRootUnsynced = true;
		this.rootEnvironment.setUnSynced();
	}

	/**
	 * Sets all sub environments as dead
	 *  
	 */
	public void setSubEnvironmentsAsDead(Environment env) {
		/*
		 * First, find all environments that have this environment as parent and
		 * set them
		 */
		Iterator<Environment> enviter = envlist.iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if ((tmpenv.getParent().equals(env)))
				setSubEnvironmentsAsDead(tmpenv);
		}
		env.setDead();
	}

	/**
	 * Sets the current environment and all sub environments as readonly
	 * 
	 * @param env
	 */
	private void setSubEnvironmentsAsReadonly(Environment env) {
		/*
		 * First, find all environments that have this environment as parent and
		 * set them readonly
		 */
		Iterator<Environment> enviter = envlist.iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if (tmpenv.getParent().equals(env))
				setSubEnvironmentsAsReadonly(tmpenv);
		}
		env.setReadonly();

	}

	/**
	 * Sets all sub and current environment as unknown
	 *  
	 */
	public void setSubEnvironmentsAsUnknown(Environment env) {
		/*
		 * First, find all environments that have this environment as parent and
		 * set them
		 */
		Iterator<Environment> enviter = envlist.iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if ((tmpenv.getParent().equals(env)))
				setSubEnvironmentsAsUnknown(tmpenv);
		}
		env.setUnknown();
	}

	/**
	 * Closes an environment, updating the basenode data. It's important that
	 * environmets get closed in the reverse order.
	 * 
	 * @param env
	 */
	public void unsafeCloseEnvironment(Environment env) {
		/* Call update Environment */
		unsafeUpdateEnvironment(env);
		/* Remove environment from the list */
		unsafeDeleteEnvironment(env);
		/* Garbage collection will do the rest */
	}

	/**
	 * Removes an environment (without closing it)
	 * 
	 * @param content
	 */
	public void unsafeDeleteEnvironment(Environment env) {
		Element key = env.getBaseNode();
		envlist.removeElement(env);
		nodeEnvironmentMapping.remove(key);
	}

	/**
	 * Updates environment data Warning! To ensure that no encryption is lost,
	 * always update all environments below the updated one before updating the
	 * current environment or use updateEnvironment instead
	 * 
	 * @param env
	 */
	public synchronized void unsafeUpdateEnvironment(Environment env) {
		if (env.isModified()) {
			disableNodeVisiting();
			/* Get the key ID that was stored in the environment */
			String keyID = env.getKeyName();
			String algo = env.getAlgorithm();
			if (!(keyID == null) && !(algo == null)) {
				/* Get the root node */
				Element envroot = env.getRootNode();
				/* Get XML */
				XMLOutputter op = new XMLOutputter();
				String xmldata = op.outputString(envroot);
				/* Encrypt XML */
				XMLEncrypt xe = null;
				try {
					xe = new XMLEncrypt(xmldata, this.session.getKeyRing());
				} catch (JDOMException e) {
					new Debuginfo(4, "Unable to encrypt data, JDOM Exception");
				} catch (IOException e) {
					new Debuginfo(4, "Unable to encrypt data, IO Exception");
				}
				try {
					if (!(xe == null)) {
						xe.setAlgorithmName(algo);
						xe.setKeyName(keyID);
						Document encdoc = xe.getEncryptedData();
						/* Replace the basenode contents */
						Element baseNode = env.getBaseNode();
						Element encdocroot = encdoc.detachRootElement();
						/* Update the base node */
						Collection clist = encdocroot.removeContent();
						baseNode.setContent(clist);
						baseNode.getEnvironment().recursiveSetEnvironmentLink(
								baseNode, baseNode.getEnvironment());
						/* Set environment as non modified */
					}
					env.setUnmodified();
				} catch (KeyNotFoundException e) {
					new Debuginfo(4,
							"Unable to encrypt because key was not found");
				} catch (EncryptionException e) {
					new Debuginfo(4, "Unable to encrypt data");
				}
			}
			enableNodeVisiting();
		}
	}

	/**
	 * Updates all environments
	 */
	public void updateAllEnvironments() {
		setDeadOrAlive();
		deleteAllDeadEnvironments();
		if (envlist.size() > 0) {
			for (int i = envlist.size() - 1; i >= 0; i--) {
				unsafeUpdateEnvironment(envlist.get(i));
			}
		}
	}

	/**
	 * Updates the given environment; this is a safe method because it first
	 * calls an update of all "child" environments recursively, ensuring that
	 * all encrypted data is then up-to-date in the upper environment.
	 * 
	 * @param env
	 */
	public void updateEnvironment(Environment env) {
		/*
		 * First, find all environments that have this environment as parent and
		 * close them
		 */
		Iterator<Environment> enviter = envlist.iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if (tmpenv.getParent().equals(env))
				updateEnvironment(tmpenv);
		}
		unsafeUpdateEnvironment(env);
	}

	/**
	 * Updates all sub environments of the provided environment
	 * 
	 * @param env
	 */
	public void updateSubEnvironments(Environment env) {
		/*
		 * Find all environments that have this environment as parent and update
		 * them
		 */
		Iterator<Environment> enviter = ((Vector) envlist.clone()).iterator();
		while (enviter.hasNext()) {
			Environment tmpenv = enviter.next();
			if (tmpenv.getParent().equals(env))
				updateEnvironment(tmpenv);
		}
	}

	/**
	 * Visits a node; visiting means that if the node is encrypted and we own a
	 * decryption key then encrypted XML data is parsed and gets accessible.
	 * Visiting a node can be disabled, meaning that the encrypted node
	 * structure can be forced.
	 * 
	 * @param node
	 */
	public Element visitNode(Element node) {
		if (disableNodeVisit || !(node.isEncryptedNode()))
			return node;
		else
			return this.setEnvironment(node);
	}

	/**
	 * Removes all environments without updating
	 *  
	 */
	public void zapAllEnvironments() {
		this.envlist.clear();
		this.nodeEnvironmentMapping.clear();
	}

}
