/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * AES128 Encryption method
 * 
 * This is the recommended encryption method: fast and pretty secure.
 * It is faster than DESede but provides analogue security.
 */
package security.AES128;

import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SecureRandom;
import java.util.Hashtable;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;

import org.bouncycastle.crypto.InvalidCipherTextException;

import security.Base64;
import security.DecryptionException;
import security.EncryptionException;
import security.EncryptionMethod;
import security.KeyGenerationException;

/**
 * AES 128 Encryption / Decryption class
 */
public class AES128 implements EncryptionMethod {

	private String seed;

	private byte[] iv;

	public AES128() {
	}

	public String encryptData(String data, Object encKey)
			throws IllegalBlockSizeException, BadPaddingException,
			NoSuchAlgorithmException, NoSuchProviderException,
			NoSuchPaddingException, EncryptionException,
			InvalidCipherTextException, InvalidKeyException,
			InvalidAlgorithmParameterException {
		/* Generate the IV */
		byte[] iv = new byte[16];
		SecureRandom random = new SecureRandom();
		random.nextBytes(iv);
		IvParameterSpec ivSpec = new IvParameterSpec(iv);
		/* Encrypt */
		Cipher aes = Cipher.getInstance("AES/CBC/PKCS5Padding");
		aes.init(Cipher.ENCRYPT_MODE, (SecretKey) ((MyAES128Key) encKey)
				.getKey(), ivSpec);
		byte[] cipherdata = aes.doFinal(data.getBytes());
		int resultlength = iv.length + cipherdata.length;
		byte[] result = new byte[resultlength];
		/* Construct the output byte array, first append the IV */
		for (int i = 0; i <= 15; i++) {
			result[i] = iv[i];
		}
		/* The append the cipher data */
		for (int i = 16; i < result.length; i++) {
			result[i] = cipherdata[i - 16];
		}
		/* Encode to Base64 and return */
		return Base64.encodeBytes(result);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see security.EncryptionMethod#decryptData(java.lang.String,
	 *      java.lang.Object)
	 */
	public byte[] decryptData(String data, Object decKey)
			throws NoSuchAlgorithmException, NoSuchProviderException,
			NoSuchPaddingException, IllegalBlockSizeException,
			BadPaddingException, DecryptionException,
			InvalidCipherTextException, InvalidKeyException,
			InvalidAlgorithmParameterException {
		byte[] input = Base64.decode(data);
		byte[] iv = new byte[16];
		byte[] cipherdata = new byte[input.length - 16];
		/* Get the IV */
		for (int i = 0; i <= 15; i++) {
			iv[i] = input[i];
		}
		/* Get the cipher data */
		for (int i = 16; i < input.length; i++) {
			cipherdata[i - 16] = input[i];
		}
		IvParameterSpec ivSpec = new IvParameterSpec(iv);
		Cipher aes = Cipher.getInstance("AES/CBC/PKCS5Padding");
		aes.init(Cipher.DECRYPT_MODE, (SecretKey) ((MyAES128Key) decKey)
				.getKey(), ivSpec);
		/* Get the gold! */
		byte[] plaindata = aes.doFinal(cipherdata);
		return plaindata;
	}

	public int getAlgorithmType() {
		return 0;
	}

	public String getAlgorithmName() {
		return "AES_128";
	}

	public void generateKeyPair(Hashtable parameters)
			throws KeyGenerationException {
	}

	public void generateKey(Hashtable parameters) throws KeyGenerationException {
		String pass = (String) parameters.get("password");
		if (pass == null)
			throw new KeyGenerationException();
		this.seed = pass;
	}

	public String getPrivateKey() {
		return null;
	}

	public String getPublicKey() {
		return null;
	}

	public String getKey() {
		return new MyAES128Key(this.seed).encode();
	}

}
