/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * DESede Decryption
 */
package security.DESede;

import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.Security;
import java.security.spec.AlgorithmParameterSpec;
import java.util.Hashtable;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.PBEParameterSpec;

import org.bouncycastle.crypto.BufferedBlockCipher;
import org.bouncycastle.crypto.engines.DESEngine;
import org.bouncycastle.crypto.modes.CBCBlockCipher;
import org.bouncycastle.crypto.modes.PaddedBlockCipher;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import security.Base64;
import security.DecryptionException;
import security.EncryptionException;
import security.EncryptionMethod;
import security.KeyGenerationException;

/**
 * DESede Algorithm
 */
public class DESede implements EncryptionMethod {

	private SecretKey key;

	private String seed;

	private BufferedBlockCipher cipher;

	private byte[] salt = { (byte) 0xA9, (byte) 0x9B, (byte) 0xC8, (byte) 0x32,
			(byte) 0x56, (byte) 0x35, (byte) 0xE3, (byte) 0x03 };

	private int iterationCount = 19;

	public DESede() {
		Security.addProvider(new BouncyCastleProvider());
		cipher = new PaddedBlockCipher(new CBCBlockCipher(new DESEngine()));
	}

	public String encryptData(String data, Object encKey)
			throws IllegalBlockSizeException, BadPaddingException,
			NoSuchAlgorithmException, NoSuchProviderException,
			NoSuchPaddingException, EncryptionException, InvalidKeyException,
			InvalidAlgorithmParameterException {
		if (data == null || data.length() == 0) {
			return "";
		}
		SecretKey key = (SecretKey) ((MyDESedeKey) encKey).getKey();
		Cipher ecipher = Cipher.getInstance(key.getAlgorithm());

		AlgorithmParameterSpec paramSpec = new PBEParameterSpec(salt,
				iterationCount);

		ecipher.init(Cipher.ENCRYPT_MODE, key, paramSpec);

		try {
			return Base64.encodeBytes(ecipher.doFinal(data.getBytes("UTF8")));
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return null;
	}

	public byte[] decryptData(String data, Object decKey)
			throws NoSuchAlgorithmException, NoSuchProviderException,
			NoSuchPaddingException, IllegalBlockSizeException,
			BadPaddingException, DecryptionException, InvalidKeyException,
			InvalidAlgorithmParameterException {

		SecretKey key = (SecretKey) ((MyDESedeKey) decKey).getKey();

		AlgorithmParameterSpec paramSpec = new PBEParameterSpec(salt,
				iterationCount);

		Cipher dcipher = Cipher.getInstance(key.getAlgorithm());
		dcipher.init(Cipher.DECRYPT_MODE, key, paramSpec);
		return dcipher.doFinal(Base64.decode(data));
	}

	public int getAlgorithmType() {
		return 1;
	}

	public String getAlgorithmName() {
		return "DESede";
	}

	public void generateKeyPair(Hashtable parameters)
			throws KeyGenerationException {
		// N/A since it's a symmetric algorithm
	}

	public void generateKey(Hashtable parameters) throws KeyGenerationException {
		String pass = (String) parameters.get("password");
		if (pass == null)
			throw new KeyGenerationException();
		this.seed = pass;
	}

	public String getPrivateKey() {
		return null;
	}

	public String getPublicKey() {
		return null;
	}

	public String getKey() {
		return new MyDESedeKey(this.seed).encode();
	}

}
