/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Keyring
 * 
 * A keyring object is used to store key pairs in a session.
 * Key pairs should be loaded at the beginning of the session
 * and are supposed not to change during the session.
 * If the keyring need to be changed, session need to be
 * terminated and initiated another time. KeyID are stored
 * as 'keyid@method'. We associate an ID to RXKey objects that
 * are indipendent from the encryption algorithm used.
 */
package security;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import security.AES128.MyAES128Key;
import security.DESede.MyDESedeKey;
import security.RSA.MyRSAPrivateKey;
import security.RSA.MyRSAPublicKey;

/**
 * Keyring class
 */
public class Keyring {
	private Hashtable<String, RXKey> decryptionKeys = new Hashtable<String, RXKey>();

	private Hashtable<String, RXKey> encryptionKeys = new Hashtable<String, RXKey>();

	/**
	 * Adds the decryption key
	 * 
	 * @param keyID
	 * @param encKeyValue
	 * @throws InvalidEncryptionMethodException
	 * @throws InvalidKeyException
	 */
	public void addDecryptionKey(String keyID, String method, String deckey)
			throws InvalidEncryptionMethodException, InvalidKeyException {
		/* Instantiate key objects */
		RXKey privateKey = getPrivateRXKeyForMethod(method);
		/* Try to decode keys */
		privateKey.decode(deckey);
		/* Register keys */
		this.decryptionKeys.put(keyID + "@" + method, privateKey);
	}

	/**
	 * Adds an encryption key
	 * 
	 * @param keyID
	 * @param encKeyValue
	 * @throws InvalidEncryptionMethodException
	 * @throws InvalidKeyException
	 */
	public void addEncryptionKey(String keyID, String method, String enckey)
			throws InvalidEncryptionMethodException, InvalidKeyException {
		/* Instantiate key objects */
		RXKey publicKey = getPublicRXKeyForMethod(method);
		/* Try to decode keys */
		publicKey.decode(enckey);
		/* Register keys */
		this.encryptionKeys.put(keyID + "@" + method, publicKey);
	}

	/**
	 * Adds a complete key pair
	 * 
	 * @param keyID
	 * @param enckey
	 * @param deckey
	 * @throws InvalidKeyException
	 * @throws InvalidEncryptionMethodException
	 */
	public void addKeyPair(String keyID, String method, String enckey,
			String deckey) throws InvalidKeyException,
			InvalidEncryptionMethodException {
		/* Instantiate key objects */
		RXKey publicKey = getPublicRXKeyForMethod(method);
		RXKey privateKey = getPrivateRXKeyForMethod(method);
		/* Try to decode keys */
		publicKey.decode(enckey);
		privateKey.decode(deckey);
		/* Register keys */
		this.encryptionKeys.put(keyID + "@" + method, publicKey);
		this.decryptionKeys.put(keyID + "@" + method, privateKey);
	}

	/**
	 * Returns the decryption key byte stream
	 * 
	 * @param keyID
	 * @return
	 * @throws KeyNotFoundException
	 */
	public Object getDecryptionKey(String keyID, String method)
			throws KeyNotFoundException {
		return this.decryptionKeys.get(keyID + "@" + method);
	}

	/**
	 * Returns the encryption key byte stream
	 * 
	 * @param keyID
	 * @return
	 * @throws KeyNotFoundException
	 */
	public Object getEncryptionKey(String keyID, String method)
			throws KeyNotFoundException {
		return this.encryptionKeys.get(keyID + "@" + method);
	}

	public RXKey getPrivateRXKeyForMethod(String methodname)
			throws InvalidEncryptionMethodException {
		if (methodname == null)
			throw new InvalidEncryptionMethodException();
		else if (methodname.equals("AES_128"))
			return new MyAES128Key();
		else if (methodname.equals("DESede"))
			return new MyDESedeKey();
		else if (methodname.equals("RSA_V1_5"))
			return new MyRSAPrivateKey();
		else
			throw new InvalidEncryptionMethodException();
	}

	public RXKey getPublicRXKeyForMethod(String methodname)
			throws InvalidEncryptionMethodException {
		if (methodname == null)
			throw new InvalidEncryptionMethodException();
		else if (methodname.equals("AES_128"))
			return new MyAES128Key();
		else if (methodname.equals("DESede"))
			return new MyDESedeKey();
		else if (methodname.equals("RSA_V1_5"))
			return new MyRSAPublicKey();
		else
			throw new InvalidEncryptionMethodException();
	}

	/**
	 * Returns if a decryption key exists
	 * 
	 * @param keyID
	 * @return
	 */
	public boolean hasDecryptionKey(String keyID, String method) {
		return this.decryptionKeys.containsKey(keyID + "@" + method);
	}

	/**
	 * Returns if an encryption key exists
	 * 
	 * @param keyID
	 * @return
	 */
	public boolean hasEncryptionKey(String keyID, String method) {
		return this.encryptionKeys.containsKey(keyID + "@" + method);
	}

	/**
	 * Returns if both encryption & decryption key exists
	 * 
	 * @param keyID
	 * @return
	 */
	public boolean hasKeyPair(String keyID, String method) {
		return this.encryptionKeys.containsKey(keyID + "@" + method)
				&& this.decryptionKeys.containsKey(keyID + "@" + method);
	}
	
	/**
	 * Returns if the key exists
	 * 
	 * @param keyID
	 * @return
	 */
	public boolean hasKey(String keyID, String method) {
		return this.encryptionKeys.containsKey(keyID + "@" + method)
				|| this.decryptionKeys.containsKey(keyID + "@" + method);
	}

	/**
	 * Removes a decryption key
	 * 
	 * @param keyID
	 */
	public void removeDecryptionKey(String keyID, String method) {
		this.decryptionKeys.remove(keyID + "@" + method);
	}

	/**
	 * Removes an encryption key
	 * 
	 * @param keyID
	 */
	public void removeEncryptionKey(String keyID, String method) {
		this.encryptionKeys.remove(keyID + "@" + method);
	}

	/**
	 * Removes an entire keypair
	 * 
	 * @param keyID
	 */
	public void removeKeyPair(String keyID, String method) {
		this.encryptionKeys.remove(keyID + "@" + method);
		this.decryptionKeys.remove(keyID + "@" + method);
	}

	/**
	 * Get key IDs
	 */
	public Vector getKeyIDs() {
		Vector<String> allKeys = new Vector<String>();
		Iterator deckeys = decryptionKeys.keySet().iterator();
		while (deckeys.hasNext()) {
			allKeys.add((String) deckeys.next());
		}
		Iterator enckeys = encryptionKeys.keySet().iterator();
		while (enckeys.hasNext()) {
			String key = (String) enckeys.next();
			if (!(decryptionKeys.containsKey(key)))
				allKeys.add(key);
		}
		return allKeys;
	}

	public boolean hasEncryptionKey(String id) {
		return this.encryptionKeys.containsKey(id);
	}

	public boolean hasDecryptionKey(String id) {
		return this.decryptionKeys.containsKey(id);
	}
}
