/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * RSA Private Key RXKey class
 * 
 * Private Keys XML is constructed as follows
 * 
 * <RoXanneRSAPrivateKey RSApubExp="..." RSAp="..." RSAdq="..." 
 * 		RSAqInv="..." RSAdp="..." RSAq="..." RSAprivExp="..." RSAmod="..."/>
 *  
 */
package security.RSA;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.security.interfaces.RSAKey;
import java.security.interfaces.RSAPrivateKey;
import java.security.spec.RSAPrivateCrtKeySpec;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import security.Base64;
import security.InvalidKeyException;
import security.RXKey;

/**
 * MyRSAPrivateKey
 */
public class MyRSAPrivateKey implements RXKey, RSAKey, RSAPrivateKey {

	private static final long serialVersionUID = 1336040623128691854L;

	Object key = null;

	private String RSAdp;

	private String RSAdq;

	private String RSAmod;

	private String RSAp;

	private String RSAprivExp;

	private String RSApubExp;

	private String RSAq;

	private String RSAqInv;

	public MyRSAPrivateKey() {
	}

	public MyRSAPrivateKey(BigInteger modulus, BigInteger publicExponent,
			BigInteger exponent, BigInteger p, BigInteger q, BigInteger dp,
			BigInteger dq, BigInteger inv) {
		this.RSApubExp = publicExponent.toString();
		this.RSAp = p.toString();
		this.RSAdq = dq.toString();
		this.RSAqInv = inv.toString();
		this.RSAdp = dp.toString();
		this.RSAq = q.toString();
		this.RSAprivExp = exponent.toString();
		this.RSAmod = modulus.toString();
	}

	public void decode(String data) throws InvalidKeyException {
		ByteArrayInputStream indata = new ByteArrayInputStream(Base64
				.decode(data));
		SAXBuilder sb = new SAXBuilder();
		Document doc = null;
		try {
			doc = sb.build(indata);
		} catch (JDOMException e) {
			throw new InvalidKeyException();
		} catch (IOException e) {
			throw new InvalidKeyException();
		}
		Element root = doc.getRootElement();
		/* Verify root node name */
		if (!(root.getName().equals("RoXanneRSAPrivateKey")))
			throw new InvalidKeyException();
		/* Extract parameters */
		RSApubExp = root.getAttributeValue("RSApubExp");
		RSAp = root.getAttributeValue("RSAp");
		RSAdq = root.getAttributeValue("RSAdq");
		RSAqInv = root.getAttributeValue("RSAqInv");
		RSAdp = root.getAttributeValue("RSAdp");
		RSAq = root.getAttributeValue("RSAq");
		RSAprivExp = root.getAttributeValue("RSAprivExp");
		RSAmod = root.getAttributeValue("RSAmod");
		/* Construct key object */
		update();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see security.RXKey#encode()
	 */
	public String encode() {
		Document doc = new Document();
		Element root = new Element("RoXanneRSAPrivateKey");
		doc.setRootElement(root);
		root.setAttribute("RSApubExp", RSApubExp);
		root.setAttribute("RSAp", RSAp);
		root.setAttribute("RSAdq", RSAdq);
		root.setAttribute("RSAqInv", RSAqInv);
		root.setAttribute("RSAdp", RSAdp);
		root.setAttribute("RSAq", RSAq);
		root.setAttribute("RSAprivExp", RSAprivExp);
		root.setAttribute("RSAmod", RSAmod);
		XMLOutputter op = new XMLOutputter();
		String plainXML = op.outputString(doc);
		return Base64.encodeBytes(plainXML.getBytes());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see security.RXKey#getKey()
	 */
	public Object getKey() {
		return key;
	}

	public BigInteger getRSAdp() {
		return new BigInteger(RSAdp);
	}

	public BigInteger getRSAdq() {
		return new BigInteger(RSAdq);
	}

	public BigInteger getRSAmod() {
		return new BigInteger(RSAmod);
	}

	public BigInteger getRSAp() {
		return new BigInteger(RSAp);
	}

	public BigInteger getRSAprivExp() {
		return new BigInteger(RSAprivExp);
	}

	public BigInteger getRSApubExp() {
		return new BigInteger(RSApubExp);
	}

	public BigInteger getRSAq() {
		return new BigInteger(RSAq);
	}

	public BigInteger getRSAqInv() {
		return new BigInteger(RSAqInv);
	}

	public void update() {
		key = new RSAPrivateCrtKeySpec(getRSAmod(), getRSApubExp(),
				getRSAprivExp(), getRSAp(), getRSAq(), getRSAdp(), getRSAdq(),
				getRSAqInv());
	}

	/* RSA Key Interface methods */

	public BigInteger getPublicExponent() {
		return getRSApubExp();
	}

	public BigInteger getPrivateExponent() {
		return getRSAprivExp();
	}

	public BigInteger getModulus() {
		return getRSAmod();
	}

	public String getAlgorithm() {
		return null;
	}

	public String getFormat() {
		return "X.509";
	}

	public byte[] getEncoded() {
		return null;
	}
}
