/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * RSA Public Key RXKey class
 * 
 * Public keys XML is constructed as follows:
 * 
 * <RoXanneRSAPublicKey RSApubExp="..." RSAmod="..."/>
 *
 */
package security.RSA;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.security.interfaces.RSAKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.RSAPublicKeySpec;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import security.Base64;
import security.InvalidKeyException;
import security.RXKey;

/**
 * MyRSAPublicKey
 */
public class MyRSAPublicKey implements RXKey, RSAKey, RSAPublicKey {

	private static final long serialVersionUID = 143487865793180250L;

	Object key = null;

	private String RSAmod;

	private String RSApubExp;

	public MyRSAPublicKey() {
	}

	public MyRSAPublicKey(BigInteger modulus, BigInteger exponent) {
		this.RSAmod = modulus.toString();
		this.RSApubExp = exponent.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see security.RXKey#decode(java.lang.String)
	 */
	public void decode(String data) throws security.InvalidKeyException {
		ByteArrayInputStream indata = new ByteArrayInputStream(Base64
				.decode(data));
		SAXBuilder sb = new SAXBuilder();
		Document doc = null;
		try {
			doc = sb.build(indata);
		} catch (JDOMException e) {
			throw new InvalidKeyException();
		} catch (IOException e) {
			throw new InvalidKeyException();
		}
		Element root = doc.getRootElement();
		/* Verify root node name */
		if (!(root.getName().equals("RoXanneRSAPublicKey")))
			throw new InvalidKeyException();
		/* Extract parameters */
		RSApubExp = root.getAttributeValue("RSApubExp");
		RSAmod = root.getAttributeValue("RSAmod");
		/* Construct key object */
		update();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see security.RXKey#encode()
	 */
	public String encode() {
		Document doc = new Document();
		Element root = new Element("RoXanneRSAPublicKey");
		doc.setRootElement(root);
		root.setAttribute("RSApubExp", RSApubExp);
		root.setAttribute("RSAmod", RSAmod);
		XMLOutputter op = new XMLOutputter();
		String plainXML = op.outputString(doc);
		return Base64.encodeBytes(plainXML.getBytes());
	}

	public Object getKey() {
		return key;
	}

	public BigInteger getRSAmod() {
		return new BigInteger(RSAmod);
	}

	public BigInteger getRSApubExp() {
		return new BigInteger(RSApubExp);
	}

	public void update() {
		key = new RSAPublicKeySpec(getRSAmod(), getRSApubExp());
	}

	/* RSA Key Interface methods */

	public BigInteger getPublicExponent() {
		return getRSApubExp();
	}

	public BigInteger getModulus() {
		return getRSAmod();
	}

	public String getAlgorithm() {
		return null;
	}

	public String getFormat() {
		return null;
	}

	public byte[] getEncoded() {
		return null;
	}

}
