/**
 Master Project WS-2004 ~ SS-2005, Computer Science
 
 University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)
 
 Student :    Amos Brocco <amos.brocco@unifr.ch>
 Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
 Nicolas Werro <nicolas.werro@unifr.ch>
 		 __  __
  _ _ ___\ \/ /__ _ _ _  _ _  ___
 | '_/ _ \>  </ _` | ' \| ' \/ -_)
 |_| \___/_/\_\__,_|_||_|_||_\___|
 
 roXanne, Secure XML Data Storage Framework
 x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
 x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 This product includes software developed by the JDOM Project (http://www.jdom.org/).
 The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
 Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
 JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
 Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
 MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
 PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
 JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
 **/

/*
 * RSA Security Algorithm
 * 
 * This algorithm is very good by design to be used to encrypt XML data
 * because it is too slow and it requires fixed size blocks (that is it is 
 * not stream oriented); additionally encrypting multiple blocks with the 
 * same key could represent a weakness (i.e. NOT SECURE) in the encryption system permitting
 * an attacker to decipher the key.
 * 
 * Conclusion: RSA for stream / large data is really bad and should be avoided!
 * 
 * Nevertheless I'll leave the code here so that other encryption algorithms can
 * for example, encrypt keys (small data blocks) with RSA ;) 
 * 
 */
package security.RSA;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.Security;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPrivateCrtKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.util.Hashtable;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.bouncycastle.crypto.generators.RSAKeyPairGenerator;
import org.bouncycastle.crypto.params.RSAKeyGenerationParameters;
import org.bouncycastle.crypto.params.RSAKeyParameters;
import org.bouncycastle.crypto.params.RSAPrivateCrtKeyParameters;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import security.Base64;
import security.DecryptionException;
import security.EncryptionException;
import security.EncryptionMethod;
import security.KeyGenerationException;

/**
 * RSA Class
 */
public class RSA implements EncryptionMethod {
	
	private RSAPrivateCrtKeyParameters RSAprivKey;
	
	private RSAKeyParameters RSApubKey;
	
	/**
	 * Encrypts a string using the provided encKey
	 * 
	 * @param data
	 * @param encKey
	 * @return
	 * @throws IllegalBlockSizeException
	 * @throws BadPaddingException
	 * @throws NoSuchAlgorithmException
	 * @throws NoSuchProviderException
	 * @throws NoSuchPaddingException
	 * @throws InvalidKeyException
	 * @throws InvalidCipherTextException
	 */
	public String encryptData(String data, Object key)
	throws IllegalBlockSizeException, BadPaddingException,
	NoSuchAlgorithmException, NoSuchProviderException,
	NoSuchPaddingException, InvalidKeyException, EncryptionException,
	InvalidCipherTextException {
		
		Security.addProvider(new BouncyCastleProvider());
		KeyFactory kf = KeyFactory.getInstance("RSA");
		PublicKey sk = null;
		try {
			sk = kf.generatePublic((RSAPublicKeySpec) ((MyRSAPublicKey) key)
					.getKey());
		} catch (InvalidKeySpecException e) {
			throw new InvalidKeyException();
		}
		Cipher rsa = Cipher.getInstance("RSA/ECB/PKCS1Padding");
		rsa.init(Cipher.ENCRYPT_MODE, sk);
		/* Block size is 100 bytes for encryption and 128 for decryption */
		byte[] block = new byte[100];
		int readbytes;
		byte[] pdata = data.getBytes();
		ByteArrayOutputStream outputWriter = new ByteArrayOutputStream();
		ByteArrayInputStream inputReader = new ByteArrayInputStream(pdata);
		try {
			while ( (readbytes = inputReader.read(block)) != -1) {
				outputWriter.write(rsa.doFinal(copyBytes(block,readbytes)));
			}
			byte[] result = outputWriter.toByteArray();
			outputWriter.flush();
			if (outputWriter != null)
				outputWriter.close(); 
			if (inputReader != null)
				inputReader.close(); 
			return Base64.encodeBytes(result);
		} catch (Exception e) {
			throw new EncryptionException();
		}
	}
	
	/**
	 * Copies length bytes from the given array to a new array
	 * @param arr
	 * @param length
	 * @return
	 */
	public static byte[] copyBytes(byte[] arr, int length) {
		byte[] newArr = null;
		if (arr.length == length)
			newArr = arr;
		else {
			newArr = new byte[length];
			for (int i = 0; i < length; i++)
				newArr[i] = (byte) arr[i];
		}
		return newArr;
	}
	
	/**
	 * Decrypts data using the provided key
	 * 
	 * @throws InvalidCipherTextException RSA/ECB/PKCS1Padding
	 */
	public byte[] decryptData(String data, Object key)
	throws NoSuchAlgorithmException, NoSuchProviderException,
	NoSuchPaddingException, InvalidKeyException,
	IllegalBlockSizeException, BadPaddingException,
	DecryptionException, InvalidCipherTextException {
		
		Security.addProvider(new BouncyCastleProvider());
		KeyFactory kf = KeyFactory.getInstance("RSA");
		PrivateKey sk = null;
		try {
			sk = kf
			.generatePrivate((RSAPrivateCrtKeySpec) ((MyRSAPrivateKey) key)
					.getKey());
		} catch (InvalidKeySpecException e) {
			throw new InvalidKeyException();
		}
		Cipher rsa = Cipher.getInstance("RSA/ECB/PKCS1Padding");
		rsa.init(Cipher.DECRYPT_MODE, sk);
		/* Block size is 100 bytes for encryption and 128 for decryption */
		byte[] block = new byte[128];
		int readbytes;
		ByteArrayOutputStream outputWriter = new ByteArrayOutputStream();
		byte[] pdata = Base64.decode(data);
		ByteArrayInputStream inputReader = new ByteArrayInputStream(pdata);
		try {
			while ( (readbytes = inputReader.read(block)) != -1) {
				outputWriter.write(rsa.doFinal(copyBytes(block,readbytes)));
			}
			byte[] result = outputWriter.toByteArray();
			outputWriter.flush();
			if (outputWriter != null)
				outputWriter.close(); 
			if (inputReader != null)
				inputReader.close(); 
			return result;
		} catch (Exception e) {
			throw new DecryptionException();
		}
	}

	/**
	 * getAlgorithmType returns the type of algorithm. 0 means asymmetric, 1
	 * symmetric
	 */
	public int getAlgorithmType() {
		return 0;
	}
	
	/**
	 * Returns the algorithm name
	 */
	public String getAlgorithmName() {
		return "RSA/ECB/PKCS1Padding";
	}
	
	/**
	 * Generates an RSA keypair
	 */
	public void generateKeyPair(Hashtable parameter)
	throws KeyGenerationException {
		SecureRandom sr = new SecureRandom();
		BigInteger pubExp = new BigInteger(64, 100, new SecureRandom());
		RSAKeyGenerationParameters RSAKeyGenPara = new RSAKeyGenerationParameters(
				pubExp, sr, 1024, 80);
		RSAKeyPairGenerator RSAKeyPairGen = new RSAKeyPairGenerator();
		RSAKeyPairGen.init(RSAKeyGenPara);
		AsymmetricCipherKeyPair keyPair = RSAKeyPairGen.generateKeyPair();
		
		RSAprivKey = (RSAPrivateCrtKeyParameters) keyPair.getPrivate();
		RSApubKey = (RSAKeyParameters) keyPair.getPublic();
	}
	
	/**
	 * Generates a new key (nothing here, only for symmetric algos)
	 */
	public void generateKey(Hashtable parameters) throws KeyGenerationException {
	}
	
	/**
	 * Returns the private key
	 */
	public String getPrivateKey() {
		return new MyRSAPrivateKey(RSAprivKey.getModulus(), RSAprivKey
				.getPublicExponent(), RSAprivKey.getExponent(), RSAprivKey
				.getP(), RSAprivKey.getQ(), RSAprivKey.getDP(), RSAprivKey
				.getDQ(), RSAprivKey.getQInv()).encode();
	}
	
	/**
	 * Returns the public key
	 */
	public String getPublicKey() {
		return new MyRSAPublicKey(RSApubKey.getModulus(), RSApubKey
				.getExponent()).encode();
	}
	
	/**
	 * Returns the key. Nothing because not a symmetric algo
	 */
	public String getKey() {
		return null;
	}
	
}
