/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

package security;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.spec.InvalidKeySpecException;
import java.util.Iterator;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import org.bouncycastle.crypto.InvalidCipherTextException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import security.AES128.AES128;
import security.DESede.DESede;
import security.RSA.RSA;
import debug.Debuginfo;

/**
 * XMLDecrypt
 * 
 * This class is used to perform XML decryption of encrypted XML content The
 * encrypted XML must follow the XML Encryption Syntax, as described in pages
 * 341-369 in Secure XML
 */
public class XMLDecrypt {

	private String algorithm = null;

	private String cipherValue = null;

	// Note that there's no default encryption method or key
	private Document encryptedData;

	private String encryptionType = null;

	private String keyName = null;

	private Keyring kr = null;
	
	Element encDataElement;

	/**
	 * Construct a new XML Decryption object to decrypt the passed encrypted XML
	 * document
	 * 
	 * @param docdata
	 */
	public XMLDecrypt(Document docdata, Keyring kring) {
		this.encryptedData = docdata;
		this.kr = kring;
		this.encDataElement = encryptedData.getRootElement();
	}
	
	/**
	 * Construct a new XML Decryption object to decrypt the passed encrypted XML
	 * element
	 * 
	 * @param docdata
	 */
	public XMLDecrypt(Element encdata, Keyring kring) {
		this.encryptedData = new Document();
		this.kr = kring;
		this.encDataElement = encdata;
		encryptedData.setRootElement(encdata);
	}

	/**
	 * Construct a new XML Decryption object to decrypt the passed XML text
	 * data.
	 * 
	 * @param xmldata
	 * @throws IOException
	 * @throws JDOMException
	 */
	public XMLDecrypt(String xmldata, Keyring kring) throws JDOMException,
			IOException {
		Document doc = new SAXBuilder().build(new ByteArrayInputStream(xmldata
				.getBytes()));
		this.encryptedData = doc;
		this.kr = kring;
		this.encDataElement = encryptedData.getRootElement();
	}

	/**
	 * Decryption method
	 * 
	 * Depending on the requested method we decrypt and return data.
	 * 
	 * @param encString
	 * @param decKey
	 * @param algo
	 * @return
	 * @throws DecryptionException
	 */
	private byte[] decrypt(String encString, Object decKey, String algo)
			throws DecryptionException {
		byte[] dec = null;
		if (!(algo == null) && algo.equals("AES_128")) {
			AES128 aes = new AES128();
			try {
				dec = aes.decryptData(encString, decKey);
			} catch (InvalidKeyException e) {
				new Debuginfo(4, "Invalid key");
				throw new DecryptionException();
			} catch (NoSuchAlgorithmException e) {
				new Debuginfo(4, "No such algorithm " + algo);
				throw new DecryptionException();
			} catch (NoSuchProviderException e) {
				new Debuginfo(4, "No such provider");
				throw new DecryptionException();
			} catch (NoSuchPaddingException e) {
				new Debuginfo(4, "No such padding " + algo);
				throw new DecryptionException();
			} catch (IllegalBlockSizeException e) {
				new Debuginfo(4, "Illegal block size");
				e.printStackTrace();
				throw new DecryptionException();
			} catch (BadPaddingException e) {
				new Debuginfo(4, "Bad padding " + algo);
				throw new DecryptionException();
			} catch (InvalidCipherTextException e) {
				new Debuginfo(4, "Invalid Cipher");
				throw new DecryptionException();
			} catch (InvalidAlgorithmParameterException e) {
				new Debuginfo(4, "Invalid Algorithm parameters");
				throw new DecryptionException();
			}
		} else if (!(algo == null) && algo.equals("DESede")) {
			DESede desede = new DESede();
			try {
				dec = desede.decryptData(encString, decKey);
			} catch (InvalidKeyException e) {
				new Debuginfo(4, "Invalid key");
				throw new DecryptionException();
			} catch (NoSuchAlgorithmException e) {
				new Debuginfo(4, "No such algorithm " + algo);
				throw new DecryptionException();
			} catch (NoSuchProviderException e) {
				new Debuginfo(4, "No such provider");
				throw new DecryptionException();
			} catch (NoSuchPaddingException e) {
				new Debuginfo(4, "No such padding " + algo);
				throw new DecryptionException();
			} catch (IllegalBlockSizeException e) {
				new Debuginfo(4, "Illegal block size");
				throw new DecryptionException();
			} catch (BadPaddingException e) {
				new Debuginfo(4, "Bad padding " + algo);
				throw new DecryptionException();
			} catch (InvalidAlgorithmParameterException e) {
				new Debuginfo(4, "Invalid algorithm " + algo);
				throw new DecryptionException();
			}
		} else if (!(algo == null) && algo.equals("RSA_V1_5")) {
			RSA rsa = new RSA();
			try {
				dec = rsa.decryptData(encString, decKey);
			} catch (InvalidKeyException e) {
				new Debuginfo(4, "Invalid key");
				throw new DecryptionException();
			} catch (NoSuchAlgorithmException e) {
				new Debuginfo(4, "No such algorithm " + algo);
				throw new DecryptionException();
			} catch (NoSuchProviderException e) {
				new Debuginfo(4, "No such provider");
				throw new DecryptionException();
			} catch (NoSuchPaddingException e) {
				new Debuginfo(4, "No such padding " + algo);
				throw new DecryptionException();
			} catch (IllegalBlockSizeException e) {
				new Debuginfo(4, "Illegal block size");
				throw new DecryptionException();
			} catch (BadPaddingException e) {
				new Debuginfo(4, "Bad padding " + algo);
				throw new DecryptionException();
			} catch (InvalidCipherTextException e) {
				new Debuginfo(4, "Invalid cipher " + algo);
				throw new DecryptionException();
			} catch (DecryptionException e) {
				new Debuginfo(4, "Encryption exception " + algo);
				throw new DecryptionException();
			}
		}else {
			throw new DecryptionException();
		}
		return dec;
	}

	/**
	 * Returns the algorithm name found in the Encrypted XML
	 */
	public String getAlgorithmName() {
		return this.algorithm;
	}

	/**
	 * Decrypts all instances of encrypted XML data in the provided document
	 * Note that within this framework it is possible to encrypt whole nodes
	 * only, and decryption is not recursive. This means that this method
	 * provides a complete decryption routine for compatibility with xml-sec
	 * only. For use within this framework, each encrypted data element must be
	 * treated separately or all information about key and algorithm will only
	 * refer to the last EncryptedData element (thus all other information will
	 * get lost!) ! In normal operation, the environment management and JDOM
	 * (patched version) ensure that each encryptedData structure is passed
	 * separately. Additionally, the encryption routine will assure that no
	 * recursive encryption (i.e. additional encryption of, for example, a
	 * KeyInfo element) is allowed. This method returns directly a new Document
	 * structure with decrypted elements in it. As the "interesting" element was
	 * the root of the document, be sure to call getRootElement() on the
	 * returned value to get the element that need to be added as root of the
	 * newly created environment;
	 * 
	 * (for info about XML Encryption cf. pp.346-369 Secure XML)
	 * 
	 * @param encString
	 * @return
	 * @throws DecryptionException
	 */
	public Document getDecryptedData() throws DecryptionException {

		try {
			encryptionType = encDataElement.getAttributeValue("Type");
			/*
			 * Now parse all child elements (which can be EncryptionMethod,
			 * ds:Keyinfo, CipherData or EncryptionProperties)
			 */
			Iterator childIterator = encDataElement.getChildren().iterator();
			while (childIterator.hasNext()) {
				Element encDataChildElement = (Element) childIterator.next();
				if (encDataChildElement.getName().equals("EncryptionMethod"))
					algorithm = namespaceResolve(encDataChildElement
							.getAttributeValue("Algorithm"));
				else if (encDataChildElement.getName().equals("KeyInfo")) {
					/*
					 * Analyze KeyInfo children: they may be EncryptedKey,
					 * AgreementMethod, ds:KeyName, ds:RetrievalMethod; Warning:
					 * Only KeyName is used here!
					 */
					Iterator keyinfochildIterator = encDataChildElement
							.getChildren().iterator();
					while (keyinfochildIterator.hasNext()) {
						Element keyInfoChild = (Element) keyinfochildIterator
								.next();
						if (keyInfoChild.getName().equals("KeyName"))
							keyName = keyInfoChild.getText();
						// Ignoring RetrievalMethod and AgreementMethod
					}
				} else if (encDataChildElement.getName().equals("CipherData")) {
					Iterator cipherDataChildIterator = encDataChildElement
							.getChildren().iterator();
					while (cipherDataChildIterator.hasNext()) {
						Element cipherDataChild = (Element) cipherDataChildIterator
								.next();
						if (cipherDataChild.getName().equals("CipherValue")) {
							cipherValue = cipherDataChild.getText();
						}
					}
				}
			}

			/*
			 * Now, for each EncryptedData element found, we decrypt it and
			 * replace it's value in the tree; normally, there is only one
			 * encryptedData element, so this while cycle is performed only once
			 */
			Object decryptionKey = this.getDecryptionKey(keyName, algorithm);
			if (decryptionKey == null)
				return null;
			byte[] decryptedData = null;
			try {
				decryptedData = this.decrypt(cipherValue, decryptionKey,
						algorithm);
			} catch (DecryptionException e) {
				throw new DecryptionException();
			}

			/*
			 * Now, depending on the Encryption type, return a new Document
			 * object; first, whole document encryption (Warning! This method
			 * returns directly a new Document, ignoring other EncryptedData
			 * elements!)
			 */
			if (!(encryptionType == null)
					&& (encryptionType
							.equals("http://www.isi.edu/in-notes/iana/assignments/media-types/text/xml"))) {
				return new SAXBuilder().build(new ByteArrayInputStream(decryptedData));
			}
			/*
			 * Single element encryption: the element gets replaced by the root
			 * of the decrypted XML content (default mode within this framework)
			 * (for namespace cf. page 349)
			 */
			else if (!(encryptionType == null)
					&& (encryptionType
							.equals("http://www.w3.org/2001/04/xmlenc#Element"))) {
				int contentIndex = encryptedData.indexOf(encDataElement);
				try {
					Document decDoc = new SAXBuilder()
							.build(new ByteArrayInputStream(decryptedData));
					Element rootEl = decDoc.getRootElement();
					rootEl.detach();
					encryptedData.setContent(contentIndex, rootEl);
				} catch (Exception e) {
					// If it's not possible to decrypt data, leave the node
					// as it is
					new Debuginfo(3, "Unable to replace element");
					throw new DecryptionException();
					//e.printStackTrace();
				}
			}
			/*
			 * Content replacement (for namespace cf. page 349)
			 */
			else if (!(encryptionType == null)
					&& (encryptionType
							.equals("http://www.w3.org/2001/04/xmlenc#Content"))) {
				int contentIndex = encryptedData.indexOf(encDataElement);
				try {
					Document decDoc = new SAXBuilder()
							.build(new ByteArrayInputStream(decryptedData));
					Iterator decChildIterator = decDoc.getRootElement()
							.getChildren().iterator();
					encryptedData.removeContent(contentIndex);
					while (decChildIterator.hasNext())
						encryptedData.addContent(contentIndex,
								(Element) decChildIterator.next());
				} catch (Exception e) {
					// If it's not possible to replace data, ignore
					new Debuginfo(3, "Unable to replace content");
					throw new DecryptionException();
				}
			} else {
				new Debuginfo(3, "Unknown encryption type");
				throw new DecryptionException();
			}

		} catch (Exception e) {
			new Debuginfo(3, "Error on data decryption");
			//e.printStackTrace();
			throw new DecryptionException();
		}
		return encryptedData;
	}

	/**
	 * Returns the decryption key
	 * 
	 * @param keyname
	 * @param algo
	 * @return
	 * @throws KeyNotFoundException
	 * @throws KeyNotFoundException
	 * @throws NoSuchAlgorithmException
	 * @throws InvalidKeyException
	 * @throws InvalidKeySpecException
	 */
	private Object getDecryptionKey(String keyname, String algo)
			throws KeyNotFoundException {
		return kr.getDecryptionKey(keyname, algo);
	}

	/**
	 * Returns the encryption type found in the Encrypted XML Default in roXanne
	 * is Element encryption
	 */
	public String getEncryptionType() {
		return this.encryptionType;
	}

	/**
	 * Returns the key name found in the Encrypted XML
	 */
	public String getKeyName() {
		return this.keyName;
	}

	/**
	 * Returns the algoritm name depending on the namespace value passed as
	 * parameter
	 * 
	 * @param attributeValue
	 * @return
	 */
	private String namespaceResolve(String algorithmNS) {
		if (algorithmNS
				.equals("http://www.w3.org/2001/04/xmlenc#tripledes-cbc"))
			return "DESede";
		else if (algorithmNS
				.equals("http://www.w3.org/2001/04/xmlenc#aes128-cbc"))
			return "AES_128";
		else if (algorithmNS
				.equals("http://www.w3.org/2001/04/xmlenc#aes256-cbc"))
			return "AES_256";
		else if (algorithmNS.equals("http://www.w3.org/2001/04/xmlenc#rsa-1_5"))
			return "RSA_V1_5";
		else if (algorithmNS
				.equals("http://www.w3.org/2001/04/xmlenc#rsa-oaep-mgf1p"))
			return "RSA_OAEP";
		else if (algorithmNS.equals("http://www.w3.org/2001/04/xmlenc#dh"))
			return "DH";
		else if (algorithmNS
				.equals("http://www.w3.org/2001/04/xmlenc#kw-tripledes"))
			return "KW_TRIPLE_DES";
		else if (algorithmNS
				.equals("http://www.w3.org/2001/04/xmlenc#kw-aes128"))
			return "KW_AES_128";
		else if (algorithmNS
				.equals("http://www.w3.org/2001/04/xmlenc#kw-aes256"))
			return "KW_AES_256";
		else if (algorithmNS.equals("http://www.w3.org/2000/09/xmldsig#sha1"))
			return "SHA1";
		else if (algorithmNS.equals("http://www.w3.org/2000/09/xmldsig#sha256"))
			return "SHA256";
		else if (algorithmNS
				.equals("http://www.w3.org/TR/1999/REC-xpath-19991116"))
			return "XPATH";
		else if (algorithmNS.equals("http://www.w3.org/2000/09/xmldsig#base64"))
			return "BASE_64";
		else if (algorithmNS
				.equals("http://www.w3.org/2001/04/xmlenc#EncryptedKey"))
			return "ENC_KEY";
		else if (algorithmNS.equals("http://www.w3.org/2000/09/xmldsig#"))
			return "XML_DSIG";
		else if (algorithmNS.equals("http://www.w3.org/2001/04/xmlenc#Element"))
			return "ELEMENT";
		else if (algorithmNS.equals("http://www.w3.org/2001/04/xmlenc#Content"))
			return "CONTENT";
		else if (algorithmNS
				.equals("http://www.isi.edu/in-notes/iana/assignments/media-types/text/xml"))
			return "DOCUMENT";
		else if (algorithmNS.equals("http://bogus.bogus"))
			return "BOGUS";
		else
			return null;
	}

}
