/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

package security;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import org.bouncycastle.crypto.InvalidCipherTextException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

import security.AES128.AES128;
import security.DESede.DESede;
import security.RSA.RSA;
import debug.Debuginfo;

/**
 * XMLEncrypt
 * 
 * This class is used to perform encryption XML content The encrypted XML must
 * follow the XML Encryption Syntax, as described in pages 341-369 in Secure XML
 */
public class XMLEncrypt {

	// Note that there's no default encryption method or key

	private String algorithm = null;

	private String keyName = null;

	private String dectext = null;

	private Keyring kr = null;

	private Document docdata;

	/**
	 * Construct a new XML Decryption object to decrypt the passed XML text
	 * data.
	 * 
	 * @param xmldata
	 * @throws IOException
	 * @throws JDOMException
	 */
	public XMLEncrypt(String xmldata, Keyring kring) throws JDOMException,
			IOException {
		this.dectext = xmldata;
		this.kr = kring;
	}

	/**
	 * Construct a new XML Decryption object to decrypt the passed encrypted XML
	 * document
	 * 
	 * @param docdata
	 */
	public XMLEncrypt(Document docdata, Keyring kring) {
		XMLOutputter op = new XMLOutputter();
		this.dectext = op.outputString(docdata);
		this.docdata = docdata;
		this.kr = kring;
	}

	/**
	 * Sets the key name for the Encrypted XML
	 */
	public void setKeyName(String keyname) {
		this.keyName = keyname;
	}

	/**
	 * Sets the algorithm name for the Encrypted XML
	 */
	public void setAlgorithmName(String algorithm) {
		this.algorithm = algorithm;
	}

	// Generate a Complete XML Encrypted File.
	public Document getEncryptedData() throws KeyNotFoundException,
			EncryptionException {
		try {
		Document doc = new Document();
		Namespace dsNamespace = Namespace.getNamespace("ds",
				"http://www.w3.org/2000/09/xmldsig#");
		Element encdata = new Element("EncryptedData");
		encdata.setNamespace(Namespace
				.getNamespace("http://www.w3.org/2001/04/xmlenc#"));
		encdata.addNamespaceDeclaration(dsNamespace);
		encdata
				.setAttribute("Type",
						"http://www.w3.org/2001/04/xmlenc#Element");
		doc.setRootElement(encdata);
		Element encmethod = new Element("EncryptionMethod");
		encmethod.setNamespace(Namespace.NO_NAMESPACE);
		encmethod.setAttribute("Algorithm", algorithmResolve(this.algorithm));
		encdata.addContent(encmethod);
		Element keyInfo = new Element("KeyInfo");
		keyInfo.setNamespace(dsNamespace);
		encdata.addContent(keyInfo);
		Element keyName = new Element("KeyName");
		keyName.setNamespace(dsNamespace);
		keyName.setText(this.keyName);
		keyInfo.addContent(keyName);
		Element cipherData = new Element("CipherData");
		encdata.addContent(cipherData);
		Element cipherValue = new Element("CipherValue");
		cipherData.addContent(cipherValue);
		/* Get key */
		Object key = getEncryptionKey(this.keyName, this.algorithm);
		/* Call encryption */
		String encText = encrypt(dectext, key, this.algorithm);
		cipherValue.setText(encText);
		return doc;
		} catch(Exception e) {
			/* If encryption was not possible return the plain data */
			if(docdata==null) {
				try {
					docdata = new SAXBuilder().build(new ByteArrayInputStream(
							dectext.trim().getBytes()));
				} catch (Exception a) { new Debuginfo(5,"Cannot generate plain document to resolve encryption problem!"); }
			new Debuginfo(3,"Error while encrypting data: using plain text");
			}
			return docdata;
		}
	}

	/**
	 * Returns the namespace of the specified algorithm
	 * 
	 * @param attributeValue
	 * @return
	 */
	private String algorithmResolve(String algorithmName) {
		if (algorithmName.equals("DESede"))
			return "http://www.w3.org/2001/04/xmlenc#tripledes-cbc";
		else if (algorithmName.equals("AES_128"))
			return "http://www.w3.org/2001/04/xmlenc#aes128-cbc";
		else if (algorithmName.equals("AES_256"))
			return "http://www.w3.org/2001/04/xmlenc#aes256-cbc";
		else if (algorithmName.equals("RSA_V1_5"))
			return "http://www.w3.org/2001/04/xmlenc#rsa-1_5";
		else if (algorithmName.equals("RSA_OAEP"))
			return "http://www.w3.org/2001/04/xmlenc#rsa-oaep-mgf1p";
		else if (algorithmName.equals("DH"))
			return "http://www.w3.org/2001/04/xmlenc#dh";
		else if (algorithmName.equals("KW_TRIPLE_DES"))
			return "http://www.w3.org/2001/04/xmlenc#kw-tripledes";
		else if (algorithmName.equals("KW_AES_128"))
			return "http://www.w3.org/2001/04/xmlenc#kw-aes128";
		else if (algorithmName.equals("KW_AES_256"))
			return "http://www.w3.org/2001/04/xmlenc#kw-aes256";
		else if (algorithmName.equals("SHA1"))
			return "http://www.w3.org/2000/09/xmldsig#sha1";
		else if (algorithmName.equals("SHA256"))
			return "http://www.w3.org/2000/09/xmldsig#sha256";
		else if (algorithmName.equals("XPATH"))
			return "http://www.w3.org/TR/1999/REC-xpath-19991116";
		else if (algorithmName.equals("BASE_64"))
			return "http://www.w3.org/2000/09/xmldsig#base64";
		else if (algorithmName.equals("ENC_KEY"))
			return "http://www.w3.org/2001/04/xmlenc#EncryptedKey";
		else if (algorithmName.equals("XML_DSIG"))
			return "http://www.w3.org/2000/09/xmldsig#";
		else if (algorithmName.equals("ELEMENT"))
			return "http://www.w3.org/2001/04/xmlenc#Element";
		else if (algorithmName.equals("CONTENT"))
			return "http://www.w3.org/2001/04/xmlenc#Content";
		else if (algorithmName.equals("DOCUMENT"))
			return "http://www.isi.edu/in-notes/iana/assignments/media-types/text/xml";
		else if (algorithmName.equals("BOGUS"))
			return "http://bogus.bogus";

		else
			return null;
	}

	private Object getEncryptionKey(String keyname, String algo)
			throws KeyNotFoundException {
		return kr.getEncryptionKey(keyname, algo);
	}

	private String encrypt(String decString, Object encKey, String algo)
			throws EncryptionException {
		String enc = decString;
		if (!(algo == null) && algo.equals("AES_128")) {
			AES128 aes = new AES128();
			try {
				enc = aes.encryptData(decString, encKey);
			} catch (InvalidKeyException e) {
				new Debuginfo(4, "Invalid key");
				throw new EncryptionException();
			} catch (NoSuchAlgorithmException e) {
				new Debuginfo(4, "No such algorithm " + algo);
				throw new EncryptionException();
			} catch (NoSuchProviderException e) {
				new Debuginfo(4, "No such provider");
				throw new EncryptionException();
			} catch (NoSuchPaddingException e) {
				new Debuginfo(4, "No such padding " + algo);
				throw new EncryptionException();
			} catch (IllegalBlockSizeException e) {
				new Debuginfo(4, "Illegal block size");
				throw new EncryptionException();
			} catch (BadPaddingException e) {
				new Debuginfo(4, "Bad padding " + algo);
				throw new EncryptionException();
			} catch (InvalidCipherTextException e) {
				new Debuginfo(4, "Invalid Cipher");
				throw new EncryptionException();
			} catch (InvalidAlgorithmParameterException e) {
				new Debuginfo(4, "Invalid Algorithm parameters");
				throw new EncryptionException();
			}
		} else if (!(algo == null) && algo.equals("DESede")) {
			DESede desede = new DESede();
			try {
				enc = desede.encryptData(decString, encKey);
			} catch (InvalidKeyException e) {
				new Debuginfo(4, "Invalid key");
				throw new EncryptionException();
			} catch (NoSuchAlgorithmException e) {
				new Debuginfo(4, "No such algorithm " + algo);
				throw new EncryptionException();
			} catch (NoSuchProviderException e) {
				new Debuginfo(4, "No such provider");
				throw new EncryptionException();
			} catch (NoSuchPaddingException e) {
				new Debuginfo(4, "No such padding " + algo);
				throw new EncryptionException();
			} catch (IllegalBlockSizeException e) {
				new Debuginfo(4, "Illegal block size");
				throw new EncryptionException();
			} catch (BadPaddingException e) {
				new Debuginfo(4, "Bad padding " + algo);
				throw new EncryptionException();
			} catch (InvalidAlgorithmParameterException e) {
				new Debuginfo(4, "Invalid algorithm " + algo);
				throw new EncryptionException();
			}
		} else if (!(algo == null) && algo.equals("RSA_V1_5")) {
			RSA rsa = new RSA();
			try {
				enc = rsa.encryptData(decString, encKey);
			} catch (InvalidKeyException e) {
				new Debuginfo(4, "Invalid key");
				throw new EncryptionException();
			} catch (NoSuchAlgorithmException e) {
				new Debuginfo(4, "No such algorithm " + algo);
				throw new EncryptionException();
			} catch (NoSuchProviderException e) {
				new Debuginfo(4, "No such provider");
				throw new EncryptionException();
			} catch (NoSuchPaddingException e) {
				new Debuginfo(4, "No such padding " + algo);
				throw new EncryptionException();
			} catch (IllegalBlockSizeException e) {
				new Debuginfo(4, "Illegal block size");
				throw new EncryptionException();
			} catch (BadPaddingException e) {
				new Debuginfo(4, "Bad padding " + algo);
				throw new EncryptionException();
			} catch (InvalidCipherTextException e) {
				new Debuginfo(4, "Invalid cipher text " + algo);
				throw new EncryptionException();
			} catch (EncryptionException e) {
				new Debuginfo(4, "Encryption exception " + algo);
				throw new EncryptionException();
			}
		}else {
			throw new EncryptionException();
		}
		return enc;
	}

}
