/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Multithreaded TCP Server
 */
package server;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Vector;

import sessions.SessionManager;
import application.ConfigurationRegistry;
import debug.Debuginfo;

/**
 * MultithreadedServer
 */
public class MultithreadedServer {

	int buffersize;

	ConfigurationRegistry conf = null;

	int maxthreads;

	int port;

	int requesttimeout = 0;

	private SessionManager sm;
	
	Socket socket;

	Vector<ThreadedQueryHandler> threads = new Vector<ThreadedQueryHandler>();
	

	/**
	 * Constructor
	 * 
	 * @param smanager
	 */
	public MultithreadedServer(SessionManager smanager) {
		new Debuginfo(1, "Server object initialized");
		sm = smanager;
	}

	/**
	 * Loads the configuration
	 * 
	 * @param cr
	 */
	public void loadConfig(ConfigurationRegistry cr) {
		this.conf = cr;
		updateConfiguration();
		new Debuginfo(1, "Configuration successfully updated from registry");
	}
	
	/**
	 * Stops serving
	 * @throws IOException
	 */
	public void stopServing() throws IOException {
		if(!(socket == null) && (socket.isConnected())) {
			socket.shutdownInput();
			socket.shutdownOutput();
		}
	}

	/**
	 * Starts the server
	 * 
	 * @throws Exception
	 */
	public void startServing() throws Exception {
		new Debuginfo(1, "Trying to allocate working threads and init socket");
		// Allocates some ThreadedQueryHandlers
		for (int i = 0; i < this.maxthreads; ++i) {
			try {
				ThreadedQueryHandler w = new ThreadedQueryHandler(
						this.buffersize, sm);
				w.setServer(this);
				(new Thread(w, "ThreadedQueryHandler #" + i)).start();
				threads.addElement(w);
			} catch (java.lang.OutOfMemoryError e) {
				new Debuginfo(3, "Not enough memory to allocate working thread");
			}
		}
		new Debuginfo(1, "Allocated " + threads.size()
				+ " working query handlers threads");

		ServerSocket serversocket = null;
		// Initialize the socket
		try {
			serversocket = new ServerSocket(port);
		} catch (Exception e) {
			// If we cannot use the defined port try any free port
			serversocket = new ServerSocket(0);
		}
		new Debuginfo(1, "Server started and listening on port "
				+ serversocket.getLocalPort());
		while (true) {

			// Start listening
			socket = serversocket.accept();

			ThreadedQueryHandler w = null;
			synchronized (threads) {
				// If we run out of threads, create new ones
				if (threads.isEmpty()) {
					new Debuginfo(1, "Need to init new query handlers");
					ThreadedQueryHandler ws = new ThreadedQueryHandler(
							this.buffersize, sm);
					ws.setServer(this);
					ws.setSocket(socket);
					(new Thread(ws, "additional worker")).start();
				} else {
					// ...else re-use one of the existing threads
					w = (ThreadedQueryHandler) threads.elementAt(0);
					threads.removeElementAt(0);
					new Debuginfo(1,
							"Using one of the available query handlers, remaining "
									+ threads.size());
					w.setSocket(socket);
				}
			}
		}
	}

	/**
	 * Re-reads the configuration from the registry
	 *  
	 */
	private void updateConfiguration() {
		new Debuginfo(1,
				"Updating server configuration from configuration registry");
		this.requesttimeout = this.conf.getIntValue("ServerRequestTimeout", 10);
		this.port = this.conf.getIntValue("ServerPort", 8351);
		this.maxthreads = this.conf.getIntValue("ServerMaxThreads", 3);
		this.buffersize = this.conf.getIntValue("BufferSize", 16384000);
	}
	

}
