/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * XML request Handler
 * 
 * Here we parse each incoming request (in XML format), execute KeyRing operations and
 * launch XPath, XUpdate,...
 * 
 */
package server;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;
import debug.Debuginfo;
import environments.RootEnvironment;
import security.Base64;
import sessions.Session;
import sessions.SessionManager;

/**
 * Query Handler, parses incoming XML requests
 * For every request a session object is created (or retrieved from existing
 * sessions).
 */
public class QueryHandler {
	
	private String address;
	
	private Element answerroot;
	
	private Session currentSession;
	
	private PrintStream out;
	
	private boolean persistentSession = false;
	
	private String sid;
	
	private SessionManager smanager;
	
	private ByteArrayInputStream xml;
	
	private Document xmlanswer;
	
	/**
	 * Returns a new query handler objects to execute the given query. The query
	 * gets executed in the current Session, meaning that keys stored remains
	 * available.
	 * 
	 * @param sm
	 * @param query
	 * @param ip
	 */
	public QueryHandler(SessionManager sm, ByteArrayInputStream query, String ip,
			PrintStream output) {
		this.smanager = sm;
		xmlanswer = new Document();
		answerroot = new Element("roxannequeryresult");
		xmlanswer.setRootElement(answerroot);
		xml = query;
		address = ip;
		out = output;
	}
	
	/**
	 * Adds a new key in session's keychain. After adding a key we update all
	 * environment depending on that key If we add a writing key an environment
	 * the environment gets automatically writable.
	 * 
	 * @param keyinfo
	 */
	private String addKeyToSession(Element answerroot, Element keyinfo) {
		String keyID = null;
		try {
			smanager.domAccess.acquire();
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}
		smanager.acquiredDataLock(currentSession);
		keyID = keyinfo.getAttributeValue("id");
		String method = keyinfo.getAttributeValue("method");
		String encKeyValue = keyinfo.getChildText("encryption");
		String decKeyValue = keyinfo.getChildText("decryption");
		if (!(keyID == null)) {
			answerroot.setAttribute("id",keyID);
			answerroot.setAttribute("method",method);
			if (!(encKeyValue == null)) {
				/* If the key already exist, remove the old key, then add the new */
				if (currentSession.getKeyRing().hasEncryptionKey(keyID,
						method)) {
					/* Force environment close */
					currentSession.getEnvironmentManager().onWriteKeyRemoved(
							keyID, method);
					try {
						/* Try to add new key */
						currentSession.getKeyRing().addEncryptionKey(keyID,
								method, encKeyValue);
						currentSession.getEnvironmentManager().onWriteKeyAdded(
								keyID, method);
						answerroot.setAttribute("encryption","added");
					} catch (Exception e) {
						answerroot.setAttribute("encryption","failed");
					}
				} else {
					try {
						/* Try to add new key */
						currentSession.getKeyRing().addEncryptionKey(keyID,
								method, encKeyValue);
						currentSession.getEnvironmentManager().onWriteKeyAdded(
								keyID, method);
						answerroot.setAttribute("encryption","added");
					} catch (Exception e) {
						answerroot.setAttribute("encryption","failed");
					}
				}
			}
			if (!(decKeyValue == null)) {
				try {
					currentSession.getKeyRing().addDecryptionKey(keyID, method,
							decKeyValue);
					answerroot.setAttribute("decryption","added");
				} catch (Exception e) {
					answerroot.setAttribute("decryption","failed");
				}
			}
			
		} else {
			answerroot.setAttribute("id","!invalid!");
			new Debuginfo(3, "Null key id received, adding failed");
		}
		// Release Lock
		smanager.aboutToReleaseDataLock(currentSession);
		smanager.domAccess.release();
		smanager.releasedDataLock(currentSession);
		return keyID;
	}
	
	/**
	 * Executes a XML request A request has the following form: <roxannequery
	 * sessionid=""|Persistent="true"> <keychain><addkey id="" Type=""
	 * algoritm=""> <encryption>... </encryption> <decryption>... </decryption>
	 * </addkey> <removekey id="" flag="encryption|decryption|both"> </keychain>
	 * <request><action
	 * type="getsession|closesession|xpath|xupdate|encrypt|decrypt|sign|verify">
	 * <!-- action parameters here --> </action> <action ...> </action>
	 * </request> </roxannequery>
	 * 
	 * By using the getsession request, we can get a session id that can be
	 * reused later. A session lasts maximum 1 hour from the last request.
	 */
	public String execute() {
		/* current session and session id are, at this time unknown */
		currentSession = null;
		sid = null;
		String login = null;
		this.persistentSession = false;
		/* Create a new SAXBuilder object to parse the query */
		SAXBuilder sb = new SAXBuilder();
		Document doc = null;
		/* Try to parse the received query text */
		try {
			doc = sb.build(xml);
			//doc = sb.build(new ByteArrayInputStream(xml.getBytes()));
		} catch (JDOMException e1) {
			/* Raised if the query is invalid XML */
			new Debuginfo(2, "Invalid XML query received");
			return null;
		} catch (IOException e1) {
			/* Raised? Hopefully not since we're dealing with in-memory data */
			new Debuginfo(2, "Cannot parse XML query");
			return null;
		}
		/* At this time the query is parsed */
		try {
			Element root = doc.getRootElement();
			login = root.getAttributeValue("login");
			String password = root.getAttributeValue("password");
			String control = root.getAttributeValue("control");
			String reqSid = root.getAttributeValue("sessionId");
			/* First verify login data */
			if (!(root.getName().equals("roxannequery")))
				throw new Exception("Invalid query");
			/* First verify login data */
			if ((login == null) || (password == null)
					|| !(smanager.checkLogin(login, password)))
				throw new Exception("Invalid login data");
			if (!(reqSid == null)
					&& (smanager.existSession(login, address, reqSid))) {
				currentSession = smanager.getSession(login, address, reqSid);
				if (!(root.getAttribute("persistent") == null)
						&& (root.getAttributeValue("persistent").equals("true")))
					this.persistentSession = true;
				else
					this.persistentSession = false;
				sid = reqSid;
				answerroot.setAttribute("sessionId", reqSid);
			} else {
				/*
				 * If the session is persistent it is not removed after
				 * executing the query
				 */
				if (!(root.getAttribute("persistent") == null)
						&& (root.getAttributeValue("persistent").equals("true")))
					this.persistentSession = true;
				else
					this.persistentSession = false;
				sid = smanager.createSession(login, address);
				currentSession = smanager.getSession(login, address, sid);
				answerroot.setAttribute("sessionId", sid);
				answerroot.setAttribute("persistent", new Boolean(
						persistentSession).toString());
			}
			currentSession.setPersistent(persistentSession);
			/* Check the control requests */
			if ((smanager.isAdministrator(login)) && !(control == null)
					&& (control.equals("shutdown"))) {
				/* Shut down procedure */
				// 1. Acquire lock
				try { smanager.domAccess.acquire();
				} catch (InterruptedException e1) {	}
				smanager.acquiredDataLock(currentSession);
				// 2. Force closing of all environments
				smanager.onClosing();
				// 3. Wait for few seconds before exiting
				int shutwaittime = smanager.getRegistry().getIntValue("ShutdownWaitTime",10);
				for (int w = 0; w <= shutwaittime; w++) {
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
					}
					new Debuginfo(1, "Shutting down application in " + (10 - w)
							+ " seconds");
					if(smanager.getRootEnvironment().getSyncedStatus())
						new Debuginfo(1, "Data synced on disk. Exiting.");
				}
				System.exit(0);
			}
			/* Config reloading */
			if ((smanager.isAdministrator(login)) && !(control == null)
					&& (control.equals("reloadconfig"))) {
				/* Reload config procedure */
				// 1. Acquire lock
				try { smanager.domAccess.acquire();
				} catch (InterruptedException e1) {	}
				smanager.acquiredDataLock(currentSession);
				// 2. Reload config
				smanager.getRegistry().reloadConfig();
				new Debuginfo(1, "Configuration reloaded.");
				smanager.aboutToReleaseDataLock(currentSession);
				smanager.domAccess.release();
				smanager.releasedDataLock(currentSession);
				}
			
			/* Perform actions on keychain */
			if (!(root.getChild("keychain") == null))
				performKeyChainActions(root);
			/* Perform requests */
			if (!(root.getChild("requests") == null))
				performRequestActions(root);
			/* Finished */
		} catch (Exception e) {
			new Debuginfo(4, "Error! " + e.getMessage());
			///e.printStackTrace();
			Element error = new Element("FatalError");
			error.setText(e.getLocalizedMessage());
			answerroot.addContent(error);
		}
		if(!(currentSession==null))
			currentSession.updateTimestamp();
		/* If the session was not requested to be persistent, delete it */
		if (!(currentSession == null) && !persistentSession)
			smanager.deleteSession(login, address, sid);
		XMLOutputter op = new XMLOutputter();
		try {
			op.output(xmlanswer, out);
		} catch (IOException e2) {
			new Debuginfo(4,"Error! Cannot send reply");
		}
		out.flush();
		return null;
	}
	
	/**
	 * Puts attributes in an element into an hashtable
	 * 
	 * @param request
	 * @return
	 */
	private Hashtable getAttributeHashTable(Element request) {
		Hashtable<String, String> ht = new Hashtable<String, String>();
		List attributes = request.getAttributes();
		Iterator attIt = attributes.iterator();
		while (attIt.hasNext()) {
			Attribute at = (Attribute) attIt.next();
			ht.put(at.getName(), at.getValue());
		}
		return ht;
	}
	
	/**
	 * Performs actions on the keychain of the current session; actions include
	 * AddKey and RemoveKey. For security reasons it is not possible to get a
	 * key stored in the keychain.
	 * 
	 * @param root
	 */
	private void performKeyChainActions(Element root) {
		Element keychainresult = new Element("keychain");
		Iterator keychainiterator = root.getChild("keychain").getChildren()
		.iterator();
		while (keychainiterator.hasNext()) {
			Element keychainchild = (Element) keychainiterator.next();
			if (keychainchild.getName().equals("addkey")) {
				Element answer = new Element("addkey");
				addKeyToSession(answer, keychainchild);
				keychainresult.addContent(answer);
			} else if (keychainchild.getName().equals("removekey")) {
				Element answer = new Element("removekey");
				removeKeyFromSession(answer, keychainchild);
				keychainresult.addContent(answer);
			} else {
				new Debuginfo(3, "Unknown keychain action received "
						+ keychainchild.getName());
			}
		}
		answerroot.addContent(keychainresult);
	}
	
	/**
	 * Performs actions; actions involving accessing the data are subjected to
	 * the domAccess semaphore to avoid concurrent dom access;
	 * 
	 * @param root
	 */
	private void performRequestActions(Element root) {
		/*
		 * This will stop the updater thread, that will release the important
		 * lock
		 */
		// Acquire Lock
		Element requestsresult = new Element("requests");
		try {
			smanager.domAccess.acquire();
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}
		smanager.acquiredDataLock(currentSession);
		/* Fix the root environment && environment manager link */
		RootEnvironment renv = currentSession.getEnvironmentManager()
		.getRootEnvironment();
		renv.setEnvironmentManager(currentSession.getEnvironmentManager());
		/* Process requests subnode */
		Iterator requestIterator = root.getChild("requests").getChildren(
		"action").iterator();
		while (requestIterator.hasNext()) {
			Element request = (Element) requestIterator.next();
			String command = request.getAttributeValue("type");
			String encoded64 = request.getAttributeValue("encoded");
			if (!command.equals(null)) {
				// Switch between possible requests
				
				/*
				 * =========================================================
				 * PERFORM XPATH
				 * =========================================================
				 */
				if (command.equals("xpath")) {
					currentSession.getEnvironmentManager()
					.disableModificationNotify();
					String xpath = request.getText();
					if (!(encoded64 == null) && encoded64.equals("true"))
						xpath = Base64.decode(request.getText()).toString();
					// Recursion depth
					String depthValue = request.getAttributeValue("depth");
					int depth = -1;
					if (!(depthValue == null))
						try {
							depth = Integer.parseInt(depthValue);
						} catch (Exception e) {
							depth = -1;
						}
						this.currentSession.performXPath(requestsresult, xpath, depth);
						currentSession.getEnvironmentManager()
						.enabledModifyNotify();
						/*
						 * =========================================================
						 * PERFORM XUPDATE
						 * =========================================================
						 */
				} else if (command.equals("xupdate")) {
					currentSession.getEnvironmentManager()
					.enableModificationNotify();
					this.currentSession.performXUpdate(requestsresult,
							(Element) request.getChildren().get(0));
					currentSession.getEnvironmentManager()
					.disableModificationNotify();
					/*
					 * =========================================================
					 * PERFORM NODE ENCRYPTION
					 * =========================================================
					 */
				} else if (command.equals("encrypt")) {
					String encrypt = request.getText();
					if (!(encoded64 == null) && encoded64.equals("true"))
						encrypt = Base64.decode(request.getText()).toString();
					String method = request.getAttributeValue("method");
					String keyid = request.getAttributeValue("keyId");
					if (!(method == null) && !(keyid == null))
						this.currentSession.encryptNode(requestsresult, encrypt,
								method, keyid);
					/*
					 * =========================================================
					 * PERFORM KEY GENERATION
					 * =========================================================
					 */
				} else if (command.equals("genkey")) {
					currentSession.getEnvironmentManager()
					.disableModificationNotify();
					String method = request.getAttributeValue("method");
					this.currentSession.generateKey(
							getAttributeHashTable(request), requestsresult,
							method);
					currentSession.getEnvironmentManager()
					.enableModificationNotify();
					
					/*
					 * =========================================================
					 * PERFORM NODE DECRYPTION
					 * =========================================================
					 */
				} else if (command.equals("decrypt")) {
					String decrypt = request.getText();
					if (!(encoded64 == null) && encoded64.equals("true"))
						decrypt = Base64.decode(request.getText()).toString();
					this.currentSession.decryptNode(requestsresult, decrypt);
					/*
					 * =========================================================
					 * PERFORM NODE HASH
					 * =========================================================
					 */
				} else if (command.equals("hash")) {
					String hash = request.getText();
					String digest = request.getAttributeValue("method");
					if (!(encoded64 == null) && encoded64.equals("true"))
						hash = Base64.decode(request.getText()).toString();
					this.currentSession.createHash(requestsresult, hash, digest);
					/*
					 * =========================================================
					 * GET STATUS
					 * =========================================================
					 */
				} else if (command.equals("status")) {
					String status = request.getText();
					if (!(encoded64 == null) && encoded64.equals("true"))
						status = Base64.decode(request.getText()).toString();
					this.currentSession.getStatus(requestsresult);
					/*
					 * =========================================================
					 * GET NODE INFO
					 * =========================================================
					 */
				} else if (command.equals("nodeinfo")) {
					String nodeinfo = request.getText();
					if (!(encoded64 == null) && encoded64.equals("true"))
						nodeinfo = Base64.decode(request.getText()).toString();
					this.currentSession.getNodeInfo(requestsresult, nodeinfo);
					/*
					 * =========================================================
					 * PERFORM NAMESPACE BINDING
					 * =========================================================
					 */
				} else if (command.equals("nsbind")) {
					currentSession.getEnvironmentManager()
					.disableModificationNotify();
					String prefix = request.getAttributeValue("prefix");
					String uri = request.getAttributeValue("uri");
					this.currentSession.bindNamespace(requestsresult,
							prefix, uri);
					currentSession.getEnvironmentManager()
					.enableModificationNotify();
					/*
					 * =========================================================
					 * PERFORM NAMESPACE UNBINDING
					 * =========================================================
					 */
				} else if (command.equals("nsunbind")) {
					currentSession.getEnvironmentManager()
					.disableModificationNotify();
					String prefix = request.getAttributeValue("prefix");
					this.currentSession.unbindNamespace(requestsresult,
							prefix);
					currentSession.getEnvironmentManager()
					.enableModificationNotify();
					/*
					 * =========================================================
					 * PERFORM NAMESPACE COMPLETE NAMESPACE UNBINDING
					 * =========================================================
					 */
				} else if (command.equals("nsunbindall")) {
					currentSession.getEnvironmentManager()
					.disableModificationNotify();
					currentSession.getEnvironmentManager()
					.enableModificationNotify();
					String prefix = request.getAttributeValue("prefix");
					this.currentSession.unbindAllNamespaces(requestsresult);
					currentSession.getEnvironmentManager()
					.enableModificationNotify();
				} 
			}
		}
		/*
		 * When all request are terminated we can start the updater thread and
		 * release the lock
		 */
		smanager.aboutToReleaseDataLock(currentSession);
		smanager.domAccess.release();
		smanager.releasedDataLock(currentSession);
		answerroot.addContent(requestsresult);
	}
	
	/**
	 * Removes a key pair from the session keychain. Warning! This forces the
	 * closing of all environments depending on this key If we remove a reading
	 * key, the environment gets closed (implies removing the entire key pair)
	 * If we remove a writing key only, the environment gets updated and then
	 * locked readonly
	 * 
	 * @param keyinfo
	 */
	private String removeKeyFromSession(Element answerroot, Element keyinfo) {
		String keyID = keyinfo.getAttributeValue("id");
		String flag = keyinfo.getAttributeValue("flag");
		String method = keyinfo.getAttributeValue("method");
		if (!(keyID == null)
				&& currentSession.getKeyRing().hasKey(keyID, method)) {
			try {
				smanager.domAccess.acquire();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			smanager.acquiredDataLock(currentSession);
			answerroot.setAttribute("id",keyID);
			answerroot.setAttribute("method",method);
			/* Perform key remove */
			if ((flag.equals("encryption")) || (flag.equals("both"))) {
				currentSession.getEnvironmentManager().onWriteKeyRemoved(keyID,
						method);
				try {
					currentSession.getKeyRing().removeEncryptionKey(keyID, method);
					answerroot.setAttribute("encryption","removed");
				} catch (Exception e) {
					answerroot.setAttribute("encryption","failed");
				}
			} else if ((flag.equals("decryption")) || (flag.equals("both"))) {
				currentSession.getEnvironmentManager().onReadKeyRemoved(keyID,
						method);
				try {
					currentSession.getKeyRing().removeDecryptionKey(keyID, method);
					answerroot.setAttribute("decryption","removed");
				} catch (Exception e) {
					answerroot.setAttribute("decryption","failed");
				}
			}
			// Release Lock
			smanager.aboutToReleaseDataLock(currentSession);
			smanager.domAccess.release();
			smanager.releasedDataLock(currentSession);
			return keyID;
		} else {
			answerroot.setAttribute("id","!invalid!");		
			new Debuginfo(3, "Invalid key id received, remove failed");
			return null;
		}
	}
	
}
