/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * ThreadedQueryHandler
 * 
 * This thread manages an incoming connection. It reads the complete request,
 * then initializes a new QueryHandler objects which does the job and
 * then waits for another request.
 */
package server;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.Date;
import java.util.Vector;

import debug.Debuginfo;

import sessions.SessionManager;

/**
 * Each threaded query handler is used to manage an incoming connection. The
 * request string is collected by this thread and then passed to a QueryHandler
 * object that actually parses and performs the requested actions.
 */
public class ThreadedQueryHandler implements Runnable {

	static final byte[] EOL = { (byte) '\r', (byte) '\n' };

	byte[] buf;

	int BUF_SIZE = 8192;

	private MultithreadedServer mts;

	private SessionManager sm;

	private Socket socket;
	
	private int responseMaxTime = 30000;

	public ThreadedQueryHandler(int buffersize, SessionManager sman) {
		if (buffersize != 0)
			BUF_SIZE = buffersize;
		buf = new byte[BUF_SIZE];
		new Debuginfo(1, "ThreadedQueryHandler created, buffer size set to "
				+ this.BUF_SIZE + " bytes");
		socket = null;
		sm = sman;
		this.responseMaxTime = sm.getRegistry().getIntValue("responseTimeout",30000);
	}

	/**
	 * Read request from socket then use a QueryHandler object to parse and
	 * execute commands
	 */
	void handleClient() throws IOException {
		InputStream is = new BufferedInputStream(socket.getInputStream());
		PrintStream ps = new PrintStream(socket.getOutputStream());
		
		
		/*
		 * To avoid blocking for too much time while reading the request,
		 * we set a maximum timeout (that can be modified in the configuration
		 * file) and then raise a java.io.InterruptedIOException, 
		 * and subsequently close the connection
		 */
		long queryInitTime = (new Date()).getTime();
		socket.setSoTimeout(mts.requesttimeout);
		socket.setTcpNoDelay(true);
		/* Clear the buffer... fill it with some zeros */
		for (int i = 0; i < BUF_SIZE; i++) {
			buf[i] = 0;
		}
		/* Voilà: pico bello! (as we said in the army :P ) ...Now read request */
		try {
			int nread = 0, r = 0;
			int opentags = 0;
			boolean gotHeader = false;
			boolean countOpenTags = false;
			boolean requestComplete = false;
			boolean breakon = false;
			/* The request size is limited to the size of the incoming buffer 
			 * this cycles is repeated for every complete xml tag */
			while (!requestComplete && (nread < BUF_SIZE)) {
				r = is.read(buf, nread, BUF_SIZE - nread);
				if (r == -1) { /* Are we at the EOF ? */
					return;	}
				for(int k=nread; k <= nread+r; k++) {
					if((k > 0) && (buf[k-1]== (byte) '<'))
						if(buf[k]== (byte) '/' )
							opentags--;
						else {
							countOpenTags = true;
							opentags++;
						}
					
					/* Several possibilities for closing tags */
					if((k > 0) && (buf[k-1]== (byte) '/') && (buf[k]== (byte) '>') )
						{ opentags--; }
					if((k > 0) && (buf[k-1]== (byte) '?') && (buf[k]== (byte) '>') )
						opentags--;
					if((k > 0) && (buf[k-1]== (byte) '-') && (buf[k]== (byte) '>') )
						opentags--;
						
					/* Are we inside a closing tag? */
					breakon = (opentags == 0) && (buf[k]== (byte) '>');

					if(countOpenTags && (opentags == 0) && breakon)
						if(gotHeader)
							requestComplete = true;
						else {
							gotHeader = true;
							countOpenTags = false;
							breakon = false;
						}
				}
				nread += r;
			}
			long queryStartTime = (new Date()).getTime();
			new Debuginfo(1, "Received query of "
					+ nread + " bytes in "
					+ (queryStartTime - queryInitTime) + " ms");
			/* Disable socket timeout */
			socket.setSoTimeout(0);
			/* Now we have the complete XML request, process it */
			QueryHandler qh = new QueryHandler(this.sm, new ByteArrayInputStream(buf,0,nread),
					socket.getInetAddress().toString(), ps);
			qh.execute();
			/* Set a new socket timeout */
			socket.setSoTimeout(this.responseMaxTime);
			/* Flush output, so that the sender receives the result */
			if(socket.isConnected() && socket.isBound()) {
				ps.flush();
				ps.close();
			}
			socket.close();
			long queryEndTime = (new Date()).getTime();
			new Debuginfo(1, "Query processed in "
					+ (queryEndTime - queryStartTime) + " ms");
			return;
		} catch (SocketTimeoutException e) {
			new Debuginfo(3, "Request timed out");
		} finally {
			if (!socket.isClosed()) {
				socket.close();
			}
		}
	}

	/**
	 * Accept a request
	 */
	public synchronized void run() {
		while (true) {
			if (socket == null) {
				/* nothing to do */
				try {
					wait();
				} catch (InterruptedException e) {
					/* should not happen */
					continue;
				}
			}
			try {
				new Debuginfo(1, "Received request from "
						+ socket.getRemoteSocketAddress().toString()
						+ " (Port:" + socket.getPort() + ") to local port "
						+ socket.getLocalPort());
				handleClient();
			} catch (Exception e) {
				e.printStackTrace();
			}
			/*
			 * When finished go back in wait queue if there's fewer than
			 * numHandler connections.
			 */
			socket = null;
			Vector<ThreadedQueryHandler> pool = mts.threads;
			synchronized (pool) {
				if (pool.size() >= mts.maxthreads) {
					/* too many threads, exit this one */
					return;
				} else {
					pool.addElement(this);
					new Debuginfo(1, "Releasing query handler, now available "
							+ pool.size() + " (open sessions count "
							+ sm.getSessionsCount() + ")");
					sm.startUpdater();
				}
			}
		}
	}

	/**
	 * Sets the server object that fired up this thread
	 * @param mts
	 */
	public void setServer(MultithreadedServer mts) {
		this.mts = mts;
	}

	/**
	 * Sets the socket to use
	 * @param s
	 */
	synchronized void setSocket(Socket s) {
		this.socket = s;
		notify();
	}

}
