/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Session
 * 
 * When the client issues a new query, a Session object is created. Each
 * session is identified with a random string identifier (cf. IDGenerator.java).
 * A session can be persistent or not; persistent sessions are not destroyed at 
 * the end of the query, preserving the loaded keys. This class contains all
 * the methods to access the data tree and to perform operations on it.
 */
package sessions;

import java.io.IOException;
import java.security.MessageDigest;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import org.jaxen.JaxenException;
import org.jaxen.SimpleNamespaceContext;
import org.jaxen.XPath;
import org.jaxen.XPathSyntaxException;
import org.jaxen.jdom.DocumentNavigator;
import org.jaxen.jdom.JDOMXPath;
import org.jaxup.UpdateException;
import org.jaxup.jdom.JDOMDocumentUpdater;
import org.jaxup.xupdate.XUpdate;
import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.IllegalAddException;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.Text;
import org.jdom.output.XMLOutputter;
import org.saxpath.SAXPathException;

import debug.Debuginfo;

import security.Base64;
import security.EncryptionException;
import security.KeyGenerationException;
import security.KeyNotFoundException;
import security.Keyring;
import security.XMLEncrypt;
import security.AES128.AES128;
import security.DESede.DESede;
import security.RSA.RSA;
import environments.Environment;
import environments.EnvironmentManager;
import environments.RootEnvironment;

/**
 * For each incoming request, a session is created; a Session object provides
 * way of storing keys (using the private Keyring field) and accessing data.
 * 
 * @author mriya3
 */
public class Session {
	
	private EnvironmentManager envmanager;
	
	private Keyring keys;
	
	private Hashtable<String, String> namespacebindings;
	
	public boolean persistent = false;
	
	private RootEnvironment rootEnvironment;
	
	private SessionManager sm;
	
	private long timestamp;
	
	/**
	 * Session
	 * 
	 * A new session is created when the user logs in to the server. The
	 * sessions holds pointers to various objects, such as the environment
	 * manager object, a keyring object,...
	 */
	public Session(SessionManager sessionmanager) {
		this.sm = sessionmanager;
		this.keys = new Keyring();
		this.envmanager = new EnvironmentManager(this);
		this.rootEnvironment = this.sm.getRootEnvironment();
		this.namespacebindings = new Hashtable<String, String>();
	}
	
	/**
	 * Adds a new namespace binding
	 * 
	 * Note: you should also bind namespaces without prefix by providing a
	 * custom prefix; and in the XPath you should use that prefix to reference
	 * nodes, example:
	 * 
	 * <mynode xmlns="http://www....">
	 * 
	 * Should be referenced by binding his namespace to a certain prefix (ex.
	 * bogus) and then referencing the node in XPath using that prefix, i.e.
	 * bogus:mynode
	 * 
	 * @param answerroot
	 * @param prefix
	 * @param uri
	 */
	public void bindNamespace(Element answ, String prefix, String uri) {
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "nsbind");
		answ.addContent(resultSet);
		if (!(prefix == null) && !(uri == null)) {
			this.namespacebindings.put(prefix, uri);
			resultSet.setAttribute("prefix", prefix);
			resultSet.setAttribute("uri", uri);
			generateCustomReply(resultSet, 200,
					"Success", "Action performed successfully");
		} else {
			generateCustomReply(resultSet, 130, "Cannot set namespace binding",
			"Prefix and/or Uri null");
		}
	}
	
	/**
	 * Binds namespaces in the provided XPath engine
	 * 
	 * @param xpo
	 */
	public void bindNamespaces(XPath xpo) {
		Iterator nskeys = namespacebindings.keySet().iterator();
		while (nskeys.hasNext()) {
			String prefix = (String) nskeys.next();
			String uri = namespacebindings.get(prefix);
			try {
				xpo.addNamespace(prefix, uri);
			} catch (JaxenException e) {
				// Invalid namespace
			} catch (NullPointerException e) {
				// Invalid namespace
			}
		}
	}
	
	/**
	 * Binds namespaces in the provided namespace context
	 * 
	 * @param xpo
	 */
	public void bindNamespaces(SimpleNamespaceContext snc) {
		Iterator nskeys = namespacebindings.keySet().iterator();
		while (nskeys.hasNext()) {
			String prefix = (String) nskeys.next();
			String uri = namespacebindings.get(prefix);
			snc.addNamespace(prefix, uri);
		}
	}
	
	/**
	 * Calculates an hash value of the selected nodes
	 * This method can be used to obtain a quasi-Signature of a node
	 * 
	 * @param answ
	 * @param xpathlocation
	 * @param hasher
	 */
	public void createHash(Element answ, String xpathlocation, String hasher) {
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "hash");
		resultSet.setAttribute("method", hasher);
		resultSet.setAttribute("location", xpathlocation);
		answ.addContent(resultSet);
		
		MessageDigest md = null;
		try {
			md = MessageDigest.getInstance(hasher);
		} catch (Exception e) {
			generateCustomReply(resultSet, 140, "Error", "Algorithm not found "
					+ e.getMessage());
			return;
		}
		
		// Before and after this action all environments get closed!
		
		// TODO: Check this
		//envmanager.closeAllEnvironments();
		
		/* First perform the xpath in order to get the node to encrypt */
		List xpathresult = doXPath(xpathlocation);
		this.envmanager.enableModificationCheck();
		try {
			/* We require that the xpath points to a single node */
			if ((xpathresult == null) || !(xpathresult.size() == 1)
					|| !(xpathresult.get(0) instanceof Element)) {
				generateCustomReply(resultSet, 141,
						"Cannot locate data to calculate hash from",
				"No node found or multiple node found at the provided location");
				return;
			}
			/*
			 * If the located node is the root of his environment we simply need
			 * to change the Environment attributes and wait for automatic
			 * update
			 */
			Element target = (Element) xpathresult.get(0);
			
			// TODO: Check this
			envmanager.updateSubEnvironments(target.getEnvironment());
			
			/*
			 * The needed environment have been created, so block new
			 * environment creation, because for hashing purpose it's sufficient
			 */
			
			// TODO: Check this
			//envmanager.disableEnvCreation();
			envmanager.disableNodeVisiting();
			
			/* Get XML */
			XMLOutputter op = new XMLOutputter();
			String xmldata = op.outputString(target);
			/* Get the hash value */
			md.update(xmldata.getBytes());
			resultSet.setAttribute("hashvalue", Base64.encodeBytes(md.digest()));
		} catch (Exception e) {
			generateCustomReply(resultSet, 142, "Error",
					"Cannot calculate hash: " + e.getMessage());
			return;
		} finally {
			/* And then release envCreation lock */
			
			// TODO: Check this
			//envmanager.enableEnvCreation();
			envmanager.enableNodeVisiting();
			
			this.envmanager.disableModificationCheck();
		}
		
	}
	
	/**
	 * Decrypts an encrypted node. In order to decrypt a node the required key
	 * has to be in the keyring. It is only possible to decrypt one node at time
	 * (to avoid confusion), so the XPath location must point to exactly one
	 * element object.
	 * 
	 * @param answ
	 * @param xpathlocation
	 */
	public void decryptNode(Element answ, String xpathlocation) {
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "decrypt");
		resultSet.setAttribute("location", xpathlocation);
		answ.addContent(resultSet);
		// Before and after this action all environments get closed!
		envmanager.closeAllEnvironments();
		/* First perform the xpath in order to get the node to encrypt */
		List xpathresult = doXPath(xpathlocation);
		this.envmanager.enableModificationCheck();
		this.envmanager.enableModificationNotify();
		try {
			/* We require that the xpath points to a single node */
			if (!(xpathresult.size() == 1)
					&& !(xpathresult.get(0) instanceof Element)) {
				generateCustomReply(resultSet, 150,
						"Cannot locate data to decrypt",
				"No node found or multiple node found at the provided location");
				return;
			}
			/*
			 * If the located node is not the root of his environment we do
			 * nothing
			 */
			Element target = (Element) xpathresult.get(0);
			if (!(target.getEnvironment().getRootNode().equals(target))) {
				generateCustomReply(resultSet, 151, "Warning",
				"Node is not encrypted or insufficient rights to decrypt it");
				return;
			}
			/*
			 * The needed environment have been created, so block new
			 * environment creation so that eventual encrypted nodes below the
			 * node do not get decrypted
			 */
			envmanager.disableEnvCreation();
			/* Environment */
			Environment oldenv = target.getEnvironment();
			/* Now get the base node */
			Element basenode = target.getEnvironment().getBaseNode();
			/* Get base node's parent */
			Element basenodeparent = basenode.getParentElement();
			/* Update environment link */
			basenode.getEnvironment().recursiveSetEnvironmentLink(target,
					basenode.getEnvironment());
			/* Replace basenode wit new node */
			if (basenodeparent == null)
				rootEnvironment.setRootNode(target);
			else {
				envmanager.disableNodeVisiting();
				int index = basenodeparent.indexOf(basenode);
				target.setParentLink(null);
				basenodeparent.setContent(index, target);
				envmanager.enableNodeVisiting();
			}
			/* Remove unneeded environment */
			envmanager.deleteEnvironment(oldenv);
			/* Generate reply message */
			generateCustomReply(resultSet, 200, "Success",
			"Action performed successfully");
		} catch (IllegalAddException e) {
			e.printStackTrace();
			generateCustomReply(resultSet, 152,
					"Cannot perform node decryption", e.getMessage());
			return;
		} finally {
			/* And then release envCreation lock */
			envmanager.enableEnvCreation();
			this.envmanager.disableModificationCheck();
		}
	}
	
	/**
	 * Disables node visiting, preventing encrypteddata nodes to be
	 * transparently decrypted. This also prevent accessing already decrypted
	 * data contained in existing environments.
	 *  
	 */
	public void disableNodeVisiting() {
		this.envmanager.disableNodeVisiting();
	}
	
	/**
	 * Performs an XPath query on the DOM tree, starting from the root element
	 * of the root environment.
	 * 
	 * @param xpathquery
	 * @return
	 */
	public List doXPath(String xpathquery) {
		Document root = this.rootEnvironment.getRootDocument();
		List results = null;
		XPath xpath = null;
		try {
			xpath = new JDOMXPath(xpathquery);
		} catch (JaxenException e) {
			return null;
		}
		try {
			bindNamespaces(xpath);
			results = xpath.selectNodes(root);
		} catch (JaxenException e) {
			return null;
		}
		return results;
	}
	
	/**
	 * Enables node visiting when traversing the DOM tree When enabled encrypted
	 * nodes are, when possible, transparently decrypted. When disabled
	 * encrypted nodes are not visited also if a corresponding environment
	 * already exists.
	 *  
	 */
	public void enableNodeVisiting() {
		this.envmanager.enableNodeVisiting();
	}
	
	/**
	 * Encrypts a DOM element/node specified by the provided XPath query. The
	 * method returns an error if no elements or multiple elements correspond to
	 * the XPath location. The node is encrypted using the requested method and
	 * the key referenced by the keyID which has to be already present in the
	 * keyring
	 * 
	 * @param answ
	 * @param xpathlocation
	 * @param method
	 * @param keyID
	 */
	public void encryptNode(Element answ, String xpathlocation, String method,
			String keyID) {
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "encrypt");
		resultSet.setAttribute("location", xpathlocation);
		answ.addContent(resultSet);
		/* Check if the needed keys are available */
		if (!(keys.hasEncryptionKey(keyID, method))) {
			generateCustomReply(resultSet, 160, "Key not available",
			"Requested key "+keyID+"@"+method+" was not found in the keyring");
			return;
		}
		// Before and after this action all environments get closed!
		
		// TODO: Check this
		//envmanager.closeAllEnvironments();
		
		/* First perform the xpath in order to get the node to encrypt */
		List xpathresult = doXPath(xpathlocation);
		
		this.envmanager.enableModificationCheck();
		this.envmanager.enableModificationNotify();
		try {
			/* We require that the xpath points to a single node */
			if ((xpathresult == null) || !(xpathresult.size() == 1)
					|| !(xpathresult.get(0) instanceof Element)) {
				generateCustomReply(resultSet, 161,
						"Cannot locate data to encrypt",
				"No node found or multiple node found at the provided location");
				return;
			}
			
			Element target = (Element) xpathresult.get(0);
			
			/*
			 * We deny the encryption of the root node of the root environment
			 */
			if (rootEnvironment.getRootNode().equals(target)) {
				generateCustomReply(resultSet, 162,
						"Encryption of root node not allowed",
				"It is not possible to encrypt the root node of the database");
				return;
			}
			
			/*
			 * If the located node is the root of his environment we simply need
			 * to change the Environment attributes and wait for automatic
			 * update
			 */
			
			// TODO: check this
			envmanager.updateSubEnvironments(target.getEnvironment());
			
			if (target.getEnvironment().getRootNode().equals(target)) {
				target.getEnvironment().setAlgorithm(method);
				target.getEnvironment().setKeyName(keyID);
				/* Setting environment as modified */
				target.getEnvironment().setModified();
				/* Generate reply */
				generateCustomReply(resultSet, 200, "Success",
				"Action performed successfully");
				return;
			}
			/*
			 * The needed environments have been created, so block new
			 * environment creation so that eventual encrypted nodes below the
			 * node do not get decrypted
			 */
			// TODO: Check this
			//envmanager.disableEnvCreation();
			envmanager.disableNodeVisiting();
			
			/* Get XML */
			XMLOutputter op = new XMLOutputter();
			String xmldata = op.outputString(target);
			XMLEncrypt xenc = new XMLEncrypt(xmldata, keys);
			/*
			 * Set method and keyID (method is automagically valid because
			 * before we tested the keys, that also depend on valid algoritm
			 * names ;)
			 */
			xenc.setAlgorithmName(method);
			xenc.setKeyName(keyID);
			/* Encrypt data */
			Document xmlenc = xenc.getEncryptedData();
			Element rootnode = (Element) xmlenc.getRootElement().detach();
			target.getEnvironment().recursiveSetEnvironmentLink(rootnode,
					target.getEnvironment());
			/* Replace the target node with the new root */
			if (target.getParent() == null)
				rootEnvironment.setRootNode(rootnode);
			else {
				Element parent = target.getParentElement();
				int index = parent.indexOf(target);
				parent.setContent(index, rootnode);
			}
			generateCustomReply(resultSet, 200, "Success",
			"Action performed successfully");
		} catch (IllegalAddException e) {
			generateCustomReply(resultSet, 163,
					"Cannot perform node encryption", e.getMessage());
			return;
		} catch (IOException e) {
			generateCustomReply(resultSet, 163,
					"Cannot perform node encryption", e.getMessage());
			return;
		} catch (JDOMException e) {
			generateCustomReply(resultSet, 163,
					"Cannot perform node encryption", e.getMessage());
			return;
		} catch (EncryptionException e) {
			generateCustomReply(resultSet, 163,
					"Cannot perform node encryption",
			"Cannot initialize encrypter");
			return;
		} catch (KeyNotFoundException e) {
			generateCustomReply(resultSet, 163,
					"Cannot perform node encryption", "Key not found");
			return;
		} finally {
			/* And then release envCreation lock */
			//	TODO: Check this
			//envmanager.enableEnvCreation();
			envmanager.enableNodeVisiting();
			
			this.envmanager.disableModificationCheck();
		}
	}
	
	/**
	 * This method returns a well formatted XML text to be used to return custom
	 * (for example error) replies to the user.
	 * 
	 * @param i
	 * @param string
	 * @param message
	 * @return
	 */
	public void generateCustomReply(Element answroot, int i,
			String description, String message) {
		Element result = new Element("result");
		result.setAttribute("exitCode", Integer.toString(i));
		answroot.addContent(result);
		Element desc = new Element("description");
		desc.setText(description);
		result.addContent(desc);
		Element msg = new Element("detail");
		msg.setText(message);
		result.addContent(msg);
	}
	
	/**
	 * Generates and returns a new key for the given algorithm
	 * 
	 * @param answerroot
	 * @param method
	 */
	public void generateKey(Hashtable parameters, Element answerroot,
			String method) {
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "genkey");
		resultSet.setAttribute("method",method);
		answerroot.addContent(resultSet);
		if (method.equals("AES_128")) {
			AES128 aes = new AES128();
			try {
				aes.generateKey(parameters);
				generateKeyReply(resultSet, 200, aes.getKey());
			} catch (KeyGenerationException e) {
				generateCustomReply(resultSet, 170, "Error in key generation",
				"Cannot generate");
			}
			
		} else if (method.equals("DESede")) {
			DESede desede = new DESede();
			try {
				desede.generateKey(parameters);
				generateKeyReply(resultSet, 200, desede.getKey());
			} catch (KeyGenerationException e) {
				generateCustomReply(resultSet, 170, "Error in key generation",
				"Cannot generate");
				
			}
		} else if (method.equals("RSA_V1_5")) {
			RSA rsa = new RSA();
			try {
				rsa.generateKeyPair(parameters);
				generateKeypairReply(resultSet, 200, rsa.getPrivateKey(), rsa.getPublicKey());
			} catch (KeyGenerationException e) {
				generateCustomReply(resultSet, 170, "Error in key generation",
				"Cannot generate");
				
			}
		} else {
			generateCustomReply(resultSet, 171, "Cannot generate key",
					"Unknown algorithm " + method);
		}
	}
	
	/**
	 * This method returns a well formatted XML text to be used to return
	 * keypairs
	 * 
	 * @param i
	 * @param string
	 * @param message
	 * @return
	 */
	private void generateKeypairReply(Element answroot, int i, String privkey,
			String pubkey) {
		Element result = new Element("result");
		result.setAttribute("exitCode", Integer.toString(i));
		answroot.addContent(result);
		Element desc = new Element("decryption");
		desc.setText(privkey);
		result.addContent(desc);
		Element msg = new Element("encryption");
		msg.setText(pubkey);
		result.addContent(msg);
	}
	
	/**
	 * This method returns a well formatted XML text to be used to return keys
	 * 
	 * @param i
	 * @param string
	 * @param message
	 * @return
	 */
	private void generateKeyReply(Element answroot, int i, String key) {
		Element result = new Element("result");
		result.setAttribute("exitCode", Integer.toString(i));
		answroot.addContent(result);
		Element desc = new Element("key");
		desc.setText(key);
		result.addContent(desc);
	}
	
	/**
	 * Returns the environment manager object
	 * 
	 * @return
	 */
	public EnvironmentManager getEnvironmentManager() {
		return this.envmanager;
	}
	
	/**
	 * Returns the keyring storing keys for this session
	 * 
	 * @return
	 */
	public Keyring getKeyRing() {
		return this.keys;
	}
	
	/**
	 * @param answerroot
	 * @param nodeinfo
	 */
	public void getNodeInfo(Element answerroot, String nodeinfo) {
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "nodeinfo");
		resultSet.setAttribute("location", nodeinfo);
		answerroot.addContent(resultSet);
		/* First perform the xpath in order to get the node to encrypt */
		List xpathresult = doXPath(nodeinfo);
		try {
			/* We require that the xpath points to a single node */
			if (!(xpathresult.size() == 1)) {
				generateCustomReply(resultSet, 180,
						"Cannot locate data to get info",
				"No node found or multiple node found at the provided location");
				return;
			}
			/*
			 * If the located node is not the root of his environment we do
			 * nothing
			 */
			Object thetarget = xpathresult.get(0);
			if (thetarget instanceof Element) {
				Element target = (Element) thetarget;
				resultSet.setAttribute("type", "Element");
				resultSet.setAttribute("isInEncrypted", new Boolean(!(target
						.getEnvironment() == rootEnvironment)).toString());
				resultSet.setAttribute("isSelfEncrypted", new Boolean(target
						.getEnvironment().getRootNode() == target).toString());
				if (target.getEnvironment() != rootEnvironment) {
					resultSet.setAttribute("keyId", target.getEnvironment()
							.getKeyName());
					resultSet.setAttribute("method", target.getEnvironment()
							.getAlgorithm());
					resultSet.setAttribute("readonly", Boolean.toString(target
							.getEnvironment().isReadonly()));
				}
				resultSet.setAttribute("hasText", new Boolean(!(target
						.getText() == null)).toString());
			} else if (thetarget instanceof Text) {
				Text target = (Text) thetarget;
				resultSet.setAttribute("type", "Text");
				resultSet.setAttribute("isInEncrypted",
						new Boolean(!(target.getParentElement()
								.getEnvironment() == rootEnvironment))
								.toString());
				if (target.getParentElement().getEnvironment() != rootEnvironment) {
					resultSet.setAttribute("keyId", target.getParentElement()
							.getEnvironment().getKeyName());
					resultSet
					.setAttribute("method", target
							.getParentElement().getEnvironment()
							.getAlgorithm());
					resultSet.setAttribute("readonly", Boolean.toString(target
							.getParentElement().getEnvironment().isReadonly()));
				}
			} else if (thetarget instanceof Attribute) {
				Attribute target = (Attribute) thetarget;
				resultSet.setAttribute("type", "Attribute");
				resultSet.setAttribute("isInEncrypted", new Boolean(!(target
						.getParent().getEnvironment() == rootEnvironment))
						.toString());
				if (target.getParent().getEnvironment() != rootEnvironment) {
					resultSet.setAttribute("keyId", target.getParent()
							.getEnvironment().getKeyName());
					resultSet.setAttribute("method", target.getParent()
							.getEnvironment().getAlgorithm());
					resultSet.setAttribute("readonly", Boolean.toString(target
							.getParent().getEnvironment().isReadonly()));
					
				}
			} else if (thetarget instanceof Namespace) {
				Namespace ns = (Namespace) thetarget;
				resultSet.setAttribute("type", "Namespace");
				/* Get node info */
				resultSet.setAttribute("prefix", ns.getPrefix());
				resultSet.setAttribute("uri", ns.getURI());
			} else {
				throw new IllegalAddException("Invalid node type");
			}
		} catch (IllegalAddException e) {
			e.printStackTrace();
			generateCustomReply(resultSet, 181, "Cannot get node info", e
					.getMessage());
			return;
		} finally {
			/* And then release envCreation lock */
			envmanager.enableEnvCreation();
			this.envmanager.disableModificationCheck();
		}
	}
	
	/**
	 * Returns the root environment object
	 * 
	 * @return
	 */
	public RootEnvironment getRootEnvironment() {
		return rootEnvironment;
	}
	
	/**
	 * Return the status (loaded keys and namespace bindings) of the current
	 * session and additionally it renews the session
	 * 
	 * @param answerroot
	 */
	public void getStatus(Element answerroot) {
		this.updateTimestamp();
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "status");
		answerroot.addContent(resultSet);
		
		/* Informations about the stored keys */
		Element keychain = new Element("keychain");
		resultSet.addContent(keychain);
		Iterator<String> keyiter = keys.getKeyIDs().iterator();
		while (keyiter.hasNext()) {
			String completeid = (String) keyiter.next();
			String[] idinfo = completeid.split("@");
			if (idinfo.length == 2) {
				String id = idinfo[0];
				String method = idinfo[1];
				Element key = new Element("key");
				key.setAttribute("id", id);
				key.setAttribute("method", method);
				boolean hasEncryption = keys.hasEncryptionKey(completeid);
				boolean hasDecryption = keys.hasDecryptionKey(completeid);
				key.setAttribute("hasEncryption", Boolean
						.toString(hasEncryption));
				key.setAttribute("hasDecryption", Boolean
						.toString(hasDecryption));
				keychain.addContent(key);
			}
		}
		/* Namespace bindings */
		Element nsb = new Element("nsbindings");
		resultSet.addContent(nsb);
		Iterator nskeys = namespacebindings.keySet().iterator();
		while (nskeys.hasNext()) {
			String prefix = (String) nskeys.next();
			String uri = namespacebindings.get(prefix);
			Element bind = new Element("nsbind");
			bind.setAttribute("prefix", prefix);
			bind.setAttribute("uri", uri);
			nsb.addContent(bind);
		}
	}
	
	/**
	 * Returns the creation timestamp of this session
	 * 
	 * @return
	 */
	public long getTimestamp() {
		return this.timestamp;
	}
	
	/**
	 * peformXpath
	 * 
	 * An XPath expression should return a list of nodes, or a value. The
	 * expected return value should be specified in order to raise an exception
	 * earlier if the expected result is not valid. Note that it is also
	 * possible to provide a null expected value to accept whatever result the
	 * XPath query might return.
	 * 
	 * Generated output codes: 103: Invalid XPath query
	 */
	public void performXPath(Element answ, String xpathquery, int depth) {
		Document root = this.rootEnvironment.getRootDocument();
		List results = null;
		XPath xpath = null;
		this.envmanager.disableModificationCheck();
		/* Default result */
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "xpath");
		resultSet.setAttribute("depth", Integer.toString(depth));
		answ.addContent(resultSet);
		try {
			xpathquery.trim();
			xpath = new JDOMXPath(xpathquery);
		} catch (JaxenException e) {
			/* Invalid XPath query */
			generateCustomReply(resultSet, 190, "Invalid XPath query", e
					.getMessage());
			return;
		}
		
		
		
		/* If an selector is already here, omit the element name in the
		 * subselector
		 */
		boolean omitBaseElement = false;
		try {
			if(xpathquery.charAt(xpathquery.length()-1)==']') {
				omitBaseElement = true;
			}
		} catch(Exception nse) {}
		
		if(omitBaseElement) {
			/* Context should now be treated with care:
			 * we override it to the complete xpath query, because
			 * it is a "virtual context", created with selectors
			 * on node attributes. The overriden selector is now empty.
			 */
			resultSet.setAttribute("context", new String(xpathquery));
			resultSet.setAttribute("selector", "");
		} else {
			resultSet.setAttribute("context", getXPathContext(xpathquery));
			resultSet.setAttribute("selector", getXPathSelector(xpathquery));
		}
		
		
		try {
			bindNamespaces(xpath);
			results = xpath.selectNodes(root);
		} catch (JaxenException e) {
			/* Cannot perform XPath */
			generateCustomReply(resultSet, 191, "Cannot perform XPath", e
					.getMessage());
			return;
		} catch (NullPointerException e) {
			/* Cannot perform XPath */
			generateCustomReply(resultSet, 191, "Cannot perform XPath", e
					.getMessage());
			return;
		}
		/* Iterate over results */
		Iterator resultIter = results.iterator();
		while (resultIter.hasNext()) {
			Object resItem = resultIter.next();
			/* *************** Attribute node *************** */
			if (resItem instanceof Attribute) {
				Attribute attr = (Attribute) resItem;
				Element result = new Element("result");
				result.setAttribute("type", "Attribute");
				/* Get node info */
				result.setAttribute("isInEncrypted", new Boolean(!(attr
						.getParent().getEnvironment() == rootEnvironment))
						.toString());
				if (attr.getParent().getEnvironment() != rootEnvironment) {
					result.setAttribute("keyId", attr.getParent()
							.getEnvironment().getKeyName());
					result.setAttribute("method", attr.getParent()
							.getEnvironment().getAlgorithm());
					result.setAttribute("readonly", Boolean.toString(attr
							.getParent().getEnvironment().isReadonly()));
				}
				/* Resolve relative XPath */
				result.setAttribute("selector", resolveReverseSelector(results,
						attr, omitBaseElement));
				/* Set the attribute name */
				String attname = ((Attribute) resItem).getName();
				result.setAttribute("name", attname);
				/* Set the value */
				String value = ((Attribute) resItem).getValue();
				result.setAttribute("value", value);
				resultSet.addContent(result);
				/* *************** Element node *************** */
			} else if (resItem instanceof Element) {
				Element elem = (Element) resItem;
				Element result = new Element("result");
				result.setAttribute("type", "Element");
				/* Get node info */
				result.setAttribute("isInEncrypted", new Boolean(!(elem
						.getEnvironment() == rootEnvironment)).toString());
				result.setAttribute("isSelfEncrypted", new Boolean(((elem
						.getEnvironment() != this.rootEnvironment) && (elem
								.getEnvironment().getRootNode() == elem))).toString());
				if (elem.getEnvironment() != rootEnvironment) {
					result.setAttribute("keyId", elem.getEnvironment()
							.getKeyName());
					result.setAttribute("method", elem.getEnvironment()
							.getAlgorithm());
					result.setAttribute("readonly", Boolean.toString(elem
							.getEnvironment().isReadonly()));
				}
				result.setAttribute("hasText", new Boolean(!(elem.getTextTrim()
						.equals(""))).toString());
				/* Resolve relative XPath */
				result.setAttribute("selector", resolveReverseSelector(results,
						elem, omitBaseElement));
				/* Get content */
				XMLOutputter op = new XMLOutputter();
				/* Clone portion of the tree */
				Element value = null;
				if (depth < 0)
					value = (Element) ((Element) resItem).clone();
				else
					value = (Element) ((Element) resItem).maxDepthClone(depth);
				result.addContent(value);
				resultSet.addContent(result);
				/* **************** Text node ******************* */
			} else if (resItem instanceof Text) {
				Text text = (Text) resItem;
				Element result = new Element("result");
				result.setAttribute("type", "Text");
				result
				.setAttribute("isInEncrypted",
						new Boolean(!(text.getParentElement()
								.getEnvironment() == rootEnvironment))
								.toString());
				if (text.getParentElement().getEnvironment() != rootEnvironment) {
					resultSet.setAttribute("keyId", text.getParentElement()
							.getEnvironment().getKeyName());
					resultSet.setAttribute("method", text.getParentElement()
							.getEnvironment().getAlgorithm());
					resultSet.setAttribute("readonly", Boolean.toString(text
							.getParentElement().getEnvironment().isReadonly()));
				}
				/* Resolve relative XPath */
				result.setAttribute("selector", resolveReverseSelector(results,
						text, omitBaseElement));
				String value = ((Text) resItem).getValue();
				result.setText(value);
				resultSet.addContent(result);
				/* **************** Namespace node ******************* */
			} else if (resItem instanceof Namespace) {
				Namespace ns = (Namespace) resItem;
				Element result = new Element("result");
				result.setAttribute("type", "Namespace");
				/* Get node info */
				result.setAttribute("prefix", ns.getPrefix());
				result.setAttribute("uri", ns.getURI());
				/* Add the result */
				resultSet.addContent(result);
			} else {
				new Debuginfo(1,"Don't know how to handle XPath result of type "+resItem.getClass());
			}
		}
		this.envmanager.enableModificationCheck();
	}
	
	/**
	 * Returns the selector part of the provided xpath query
	 * 
	 * @param xpathquery
	 * @return
	 */
	private String getXPathSelector(String xpathquery) {
		int lastSlash = xpathquery.lastIndexOf("/");
		if (lastSlash == -1)
			return "";
		return new String(xpathquery.substring(lastSlash + 1)).trim();
	}
	
	/**
	 * Returns the context part of the provided xpath query
	 * 
	 * @param xpathquery
	 * @return
	 */
	private String getXPathContext(String xpathquery) {
		int lastSlash = xpathquery.lastIndexOf("/");
		if (lastSlash < 0)
			return "";
		else
			return new String(xpathquery.substring(0, lastSlash + 1)).trim();
	}
	
	/**
	 * Resolve the relative XPath for the given node
	 * 
	 * This method is used to univocally identify each result of an XPath
	 * expression.
	 * 
	 * @param results
	 * @param attr
	 * @return
	 */
	private String resolveReverseSelector(List results, Object node, boolean omitbase) {
		if (node instanceof Element) {
			Element element = (Element) node;
			/* First get the element name and namespace */
			String name = element.getName();
			String namespaceprefix = element.getNamespacePrefix();
			String namespaceuri = element.getNamespaceURI();
			
			/* If we should omit the base element, reset its name */
			if(omitbase==true)
				name="";
			
			/*
			 * Resolve/Correct the namespace prefix if a namespace exist that is
			 * when no prefix is specified because an implicit / default
			 * namespace is used; it's important to have the prefix value
			 */
			if (!(namespaceuri.equals("")) && (namespaceprefix.equals("")))
				namespaceprefix = getBoundNamespacePrefix(namespaceuri);
			if (!(namespaceprefix.equals("")))
				namespaceprefix = namespaceprefix + ":";
			/*
			 * At this point, if uri and prefix are both empty strings it means
			 * that no namespace is defined for this node; now we look in the
			 * result list for other elements with the same name and same
			 * namespace uri (prefix does not count, because two nodes with
			 * different prefixes but that point to the same uri are the same)
			 * and we get their index
			 */
			int index = getUniqueNodeIndex(results, node, namespaceuri);
			/* If the returned index is 0 we need no indexing */
			if (index == 0)
				return namespaceprefix + name;
			else
				return namespaceprefix + name + "[" + index + "]";
		} else if (node instanceof Attribute) {
			Attribute attr = (Attribute) node;
			/* First get the element name and namespace */
			String name = attr.getName();
			String namespaceprefix = attr.getNamespacePrefix();
			String namespaceuri = attr.getNamespaceURI();
			if (!(namespaceuri.equals("")) && (namespaceprefix.equals("")))
				namespaceprefix = getBoundNamespacePrefix(namespaceuri);
			if (!(namespaceprefix.equals("")))
				namespaceprefix = namespaceprefix + ":";
			int index = getUniqueNodeIndex(results, node, namespaceuri);
			/* If the returned index is 0 we need no indexing */
			if (index == 0)
				return "@" + name;
			else
				return "@" + name + "[" + index + "]";
		} else if (node instanceof Text) {
			Text text = (Text) node;
			int index = getUniqueNodeIndex(results, node, "");
			/* If the returned index is 0 we need no indexing */
			if (index == 0)
				return "text()";
			else
				return "text()[" + index + "]";
		} else if (node instanceof Namespace) {
			Namespace ns = (Namespace) node;
			int index = getUniqueNodeIndex(results, node, "");
			String namespaceprefix = ns.getPrefix();
			/* If the returned index is 0 we need no indexing */
			if (namespaceprefix.equals(""))
				namespaceprefix = getBoundNamespacePrefix(ns.getURI());
			if (index == 0)
				return namespaceprefix;
			else
				return namespaceprefix + "[" + index + "]";
		}
		return "";
	}
	
	/**
	 * Returns the index of the node with the given properties
	 * 
	 * @param results
	 * @param node
	 * @param namespaceuri
	 * @return
	 */
	private int getUniqueNodeIndex(List results, Object node,
			String namespaceuri) {
		int index = 0; /*
		* The index of the node, to be returned; 0 means that no
		* index is necessary to reference the given node
		*/
		int currentindex = 1; /* First child position is 1 */
		Iterator nodes = results.iterator();
		/* *********** Node type : Element ********** */
		if (node instanceof Element) {
			Element element = (Element) node;
			while (nodes.hasNext()) {
				Object currentnode = nodes.next();
				if (currentnode instanceof Element) {
					Element currentelement = (Element) currentnode;
					if (currentnode == node) // If we reach the current node set
						// the index
						index = currentindex;
					/*
					 * If we have another element with the same
					 * "charateristics", increase the index
					 */
					else if ((currentelement.getName()
							.equals(element.getName()))
							&& (currentelement.getNamespaceURI()
									.equals(namespaceuri)))
						currentindex++;
				}
			}
			if (currentindex > 1)
				return index;
			else
				return 0;
			/* *********** Node type : Attribute ********** */
		} else if (node instanceof Attribute) {
			Attribute attr = (Attribute) node;
			while (nodes.hasNext()) {
				Object currentnode = nodes.next();
				if (currentnode instanceof Attribute) {
					Attribute currentattr = (Attribute) currentnode;
					if (currentnode == node) // If we reach the current node set
						// the index
						index = currentindex;
					/*
					 * If we have another element with the same
					 * "charateristics", increase the index
					 */
					else if ((currentattr.getName().equals(attr.getName()))
							&& (currentattr.getNamespaceURI()
									.equals(namespaceuri)))
						currentindex++;
				}
			}
			if (currentindex > 1)
				return index;
			else
				return 0;
			/* *********** Node type : Text ********** */
		} else if (node instanceof Text) {
			while (nodes.hasNext()) {
				Object currentnode = nodes.next();
				if (currentnode instanceof Text) {
					if (currentnode == node) // If we reach the current node set
						// the index
						index = currentindex;
					/*
					 * If we have another element with the same
					 * "charateristics", increase the index
					 */
					else
						currentindex++;
				}
			}
			if (currentindex > 1)
				return index;
			else
				return 0;
			/* *********** Node type : Namespace ********** */
		} else if (node instanceof Namespace) {
			Namespace ns = (Namespace) node;
			while (nodes.hasNext()) {
				Object currentnode = nodes.next();
				if (currentnode instanceof Text) {
					Namespace currentns = (Namespace) currentnode;
					if (currentnode == node) // If we reach the current node set
						// the index
						index = currentindex;
					/*
					 * If we have another element with the same
					 * "charateristics", increase the index
					 */
					else if (currentns.getURI().equals(namespaceuri))
						currentindex++;
				}
			}
			if (currentindex > 1)
				return index;
			else
				return 0;
		} else {
			return 0;
		}
	}
	
	/**
	 * Get the prefix given the namespace uri
	 * 
	 * @param namespaceuri
	 * @return
	 */
	private String getBoundNamespacePrefix(String namespaceuri) {
		Iterator prefixes = namespacebindings.keySet().iterator();
		while (prefixes.hasNext()) {
			String prefix = (String) prefixes.next();
			if (!(namespacebindings.get(prefix) == null)
					&& ((String) namespacebindings.get(prefix))
					.equals(namespaceuri))
				return prefix;
		}
		return "";
	}
	
	/**
	 * performXUpdate
	 * 
	 * This method performs an XUpdate XML Query on the database. It then
	 * returns an XML data containing informations about the result of the
	 * query. Note that the XML DOM tree actually get's modified (if the XUpdate
	 * query does this), but it does nothing if the query is not a valid XUpdate
	 * query of if the XPath used for node selection is not valid.
	 * 
	 * Generated errors: 101: Cannot create DOM object for the provided XUpdate
	 * query 102: IO error on DOM object creation 103: XPath error 104: SAXPath
	 * exception 105: Cannot perform XUpdate query On success: 200: Requested
	 * action completed successfully
	 * 
	 * @param xupdatequery
	 * @return
	 */
	public void performXUpdate(Element answ, Element xupdate) {
		String result = null;
		Document root = this.rootEnvironment.getRootDocument();
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "xupdate");
		answ.addContent(resultSet);
		
		/* We should first isolate the XUpdate element */
		Element xupdatequery = (Element) xupdate.detach();
		Document xupdatedoc = new Document();
		xupdatedoc.setRootElement(xupdatequery);
		this.envmanager.enableModificationCheck();
		this.envmanager.enableModificationNotify();
		
		/* Now create an xupdate engine and try to perform update request */
		// Original navigator was
		XUpdate xupdateengine = new XUpdate(new JDOMDocumentUpdater(),
				DocumentNavigator.getInstance(), this);
		try {
			xupdateengine.runUpdate(root, xupdatedoc);
		} catch (XPathSyntaxException e) {
			generateCustomReply(resultSet, 120, "Invalid XPath Query", e
					.getMessage());
			return;
		} catch (JaxenException e) {
			generateCustomReply(resultSet, 120, "Invalid XPath Query", e
					.getMessage());
			return;
		} catch (SAXPathException e) {
			generateCustomReply(resultSet, 121, "SAXPath exception", e
					.getMessage());
			return;
		} catch (UpdateException e) {
			generateCustomReply(resultSet, 122, "Cannot perform XUpdate query",
					e.getMessage());
			return;
		} catch (IllegalAddException e) {
			generateCustomReply(resultSet, 122, "Cannot perform XUpdate query",
					e.getMessage());
			return;
		} catch (Exception e) {
			generateCustomReply(resultSet, 122, "Cannot perform XUpdate query",
					e.getMessage());
			return;
		}
		/* Success ;) */
		generateCustomReply(resultSet, 200,
				"Success", "Action performed successfully");
		this.envmanager.disableModificationCheck();
	}
	
	/**
	 * Sets session' persistance
	 * 
	 * @param b
	 */
	public void setPersistent(boolean b) {
		this.persistent = b;
	}
	
	/**
	 * Sets the root environment both here and in the associated
	 * environment manager
	 * 
	 * @param env
	 */
	public void setRootEnvironment(RootEnvironment env) {
		this.rootEnvironment = env;
		this.envmanager.setRootEnvironment(this.rootEnvironment);
	}
	
	/**
	 * Sets the timestamp for this session
	 * 
	 * @param timestamp
	 */
	public void setTimestamp(long timestamp) {
		this.timestamp = timestamp;
	}
	
	/**
	 * Removes a namespace binding
	 * 
	 * @param answerroot
	 * @param prefix
	 */
	public void unbindNamespace(Element answ, String prefix) {
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "nsunbind");
		answ.addContent(resultSet);
		if (!(prefix == null) || (!(namespacebindings.containsKey(prefix)))) {
			this.namespacebindings.remove(prefix);
			generateCustomReply(resultSet, 200,
					"Success", "Action performed successfully");
		} else {
			generateCustomReply(resultSet, 110,
					"Cannot remove namespace binding", "Invalid prefix");
		}
	}
	
	
	/**
	 * Removes all namespace bindings
	 * 
	 * @param answerroot
	 * @param prefix
	 */
	public void unbindAllNamespaces(Element answ) {
		Element resultSet = new Element("action");
		resultSet.setAttribute("type", "nsunbindall");
		answ.addContent(resultSet);
		this.namespacebindings.clear();
		generateCustomReply(resultSet, 200,
				"Success", "Action performed successfully");
	}
	
	/**
	 * Updates the session timestamp to now
	 *  
	 */
	public void updateTimestamp() {
		Date tmpdate = new Date();
		this.timestamp = tmpdate.getTime();
	}
	
}
