/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

package application;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import terminalemulator.TerminalCallbacksListener;
import actions.ConnectAction;
import actions.DecryptAction;
import actions.EncryptAction;
import actions.ExitAction;
import actions.ExportToDbAction;
import actions.ExportToXMLAction;
import actions.ImportFromDbAction;
import actions.LogoutAction;
import actions.ManageKeysAction;
import actions.ManageNamespacesAction;
import actions.SetupNewSession;
import actions.XPathAction;
import actions.XUpdateAction;

public class UserInterface extends ApplicationWindow {

	class TerminalInputListener implements TerminalCallbacksListener {

		/*
		 * (non-Javadoc)
		 * 
		 * @see terminalemulator.TerminalCallbacksListener#getContext()
		 */
		public Context getContext() {
			return context;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see terminalemulator.TerminalCallbacksListener#onCommand(java.lang.String)
		 */
		public void onCommand(String commandline) {
			context.executeXPath(commandline);
		}

	}

	/* End of toolbar items */

	static final int BUILD = 1529;

	static final String CODENAME = "Quahog";

	static final boolean DEVELOPMENTVERSION = true;

	static final int MAJOR = 0;

	static final int MINOR = 8;

	static final int RELEASE = 15;

	static String splashimage = "icons/xclicksplash.jpg";

	public static void main(String[] args) {
		/* Create and display splash */
		final Display display = new Display();
		final Image image = new Image(display, splashimage);
		final Shell splash = new Shell(SWT.ON_TOP);
		final UserInterface window = new UserInterface();

		Label label = new Label(splash, SWT.NONE);
		label.setImage(image);
		FormLayout layout = new FormLayout();
		splash.setLayout(layout);
		FormData labelData = new FormData();
		labelData.right = new FormAttachment(100, 0);
		labelData.bottom = new FormAttachment(100, 0);
		label.setLayoutData(labelData);

		splash.pack();
		Rectangle splashRect = splash.getBounds();
		Rectangle displayRect = display.getBounds();
		int x = (displayRect.width - splashRect.width) / 2;
		int y = (displayRect.height - splashRect.height) / 2;
		splash.setLocation(x, y);
		splash.open();
		/* Continue loading main interface */	
		window.setBlockOnOpen(true);
		window.addStatusLine();
		/* Show splash */
		/* Async start */display.asyncExec(new Runnable() {
			public void run() {
				try {
					Thread.sleep(2000);
				} catch (Throwable e) {}
				splash.close();
			}});
		/* Show window */
		/* Async start */display.syncExec(new Runnable() {
			public void run() {
				window.open();
			}});

		
		Display.getCurrent().dispose();
	}
	
	private Composite _composite;

	private ConnectAction _connect = new ConnectAction(this);

	private DecryptAction _decryptaction = new DecryptAction(this);

	private EncryptAction _encryptaction = new EncryptAction(this);
	
	private ExitAction _exitAction = new ExitAction(this);

	private ExportToDbAction _exporttodbaction = new ExportToDbAction(this);

	private ExportToXMLAction _exporttoxmlaction = new ExportToXMLAction(this);
	
	private ImportFromDbAction _importfromdbaction = new ImportFromDbAction(this);
	
	private LogoutAction _logout = new LogoutAction(this);
	
	private ManageKeysAction _managekeysaction = new ManageKeysAction(this);
	
	private ManageNamespacesAction _managenamespacesaction = new ManageNamespacesAction(
			this);

	private SetupNewSession _setupnewsession = new SetupNewSession(this);

	private XPathAction _xpathaction = new XPathAction(this);

	private XUpdateAction _xupdateaction = new XUpdateAction(this);

	public Text commandBox;
	
	private Context context;

	Image icon = new Image(Display.getDefault(),"icons/application.gif");

	private MenuManager importMenu;

	terminalemulator.TerminalEmulator output;
	
	public UserInterface() {
		this(null);
	}
	
	public UserInterface(Shell shell) {
		super(shell);
		//this.setBlockOnOpen(true);
		addMenuBar();
		addToolBar(SWT.FLAT | SWT.WRAP);
		addStatusLine();

	}
	

	protected void configureShell(Shell shell) {

		shell.setText("roXanne Framework - no session - x.click");

		super.configureShell(shell);
		shell.setImage(icon);
	}

	protected Control createContents(Composite parent) {
		_composite = new Composite(parent, SWT.NONE);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		_composite.setLayout(gridLayout);
		GridData searchGridData = new GridData();
		searchGridData.horizontalAlignment = GridData.FILL;
		searchGridData.verticalAlignment = GridData.FILL;
		searchGridData.grabExcessHorizontalSpace = true;
		searchGridData.grabExcessVerticalSpace = true;
		output = new terminalemulator.TerminalEmulator(_composite, SWT.MULTI
				| SWT.WRAP | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.TRAVERSE_TAB_NEXT, new TerminalInputListener());
		/* Set terminal font */
		Font courier = new Font(Display.getDefault(), "Courier", Display
				.getDefault().getSystemFont().getFontData()[0].height,
				SWT.NORMAL);
		output.setFont(courier);
		output.setLayoutData(searchGridData);
		output.setBackground(_composite.getDisplay().getSystemColor(
				SWT.COLOR_BLACK));
		output.setForeground(_composite.getDisplay().getSystemColor(
				SWT.COLOR_WHITE));
		output.setEditable(false);

		output.outputText(new StyleRange(), "\n  roXanne Framework, x.click "
				+ MAJOR + "." + MINOR + "." + RELEASE + "." + BUILD + "-"
				+ CODENAME, SWT.COLOR_WHITE, SWT.BOLD);
		output.setEditable(false);
		output
				.outputText(
						new StyleRange(),
						"\n\n\tx.core, x.click (C)2004-2005 Amos Brocco <amos.brocco@unifr.ch>"
								+ "\n\tx.mill Database<->XML Migration Tool (C)2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>"
								+ "\n\n\tThis program is free software; you can redistribute it and/or"
								+ "\n\tmodify it under the terms of the GNU General Public License"
								+ "\n\tas published by the Free Software Foundation; either version 2"
								+ "\n\tof the License, or (at your option) any later version."
								+ "\n"
								+ "\n\tThis program is distributed in the hope that it will be useful,"
								+ "\n\tbut WITHOUT ANY WARRANTY; without even the implied warranty of"
								+ "\n\tMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
								+ "\n\tGNU General Public License for more details."
								+ "\n"
								+ "\n\tYou should have received a copy of the GNU General Public License"
								+ "\n\talong with this program; if not, write to the Free Software"
								+ "\n\tFoundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA."
								+ "\n\n\tThis application uses and contains parts (either modified under the terms of the respective"
								+ "\n\tlicenses or unmodified) of the following software:"
								+ "\n\t\tBouncy Castle Crypto APIs (C) 2000 - 2004 The Legion Of The Bouncy Castle "
								+ "\n\t\tJDOM Copyright (C) 2004 Jason Hunter & Brett McLaughlin."
								+ "\n\t\tJAXEN  Copyright 2003 (C) The Werken Company. All Rights Reserved."
								+ "\n\t\tJAXUP  Copyright 2002-2003 Erwin Bolwidt."
								+ "\n\t\tBase64 Tool (C) Robert Harder"
								+ "\n\t\tJArgs (C) 2001-2003 Steve Purcell (C) 2002 Vidar Holen (C) 2002 Michal Ceresna (C) 2005 Ewan Mellor."
								+ "\n\tand others. Please refer to documentation or source code for complete copyright info.",
						SWT.COLOR_GRAY, SWT.NORMAL);
		output.outputText(new StyleRange(), "\n\n\tAvailable memory "
				+ Runtime.getRuntime().maxMemory() + " bytes, used "
				+ Runtime.getRuntime().freeMemory() + " bytes\n\n",
				SWT.COLOR_DARK_GRAY, SWT.NORMAL);

		output.setPrompt(" $> ");
		//output.showPrompt();
		output.pack();
		//output.setFocus();

		/*
		 * GridData expGridData = new GridData();
		 * expGridData.horizontalAlignment = GridData.FILL;
		 * expGridData.grabExcessHorizontalSpace = true; Label expLabel = new
		 * Label(_composite, SWT.NONE); expLabel.setText("Command (Press TAB for
		 * autocompletion)"); commandBox = new Text(_composite, SWT.BORDER);
		 * commandBox.setLayoutData(expGridData);
		 */
		setStatus("x.click - roXanne Framework");

		context = new Context(this);
		output.setContext(context);

		setupNoSessionActions();

		return _composite;
	}

	/* Creates the menu */
	protected MenuManager createMenuManager() {
		MenuManager menuManager = new MenuManager();
		/* File menu */
		MenuManager fileMenu = new MenuManager("&File");
		fileMenu.add(_setupnewsession);
		fileMenu.add(new Separator());
		fileMenu.add(_connect);
		fileMenu.add(_logout);
		fileMenu.add(new Separator());
		fileMenu.add(_exitAction);
		menuManager.add(fileMenu);
		/* Session menu */
		MenuManager sessionMenu = new MenuManager("&Session");
		sessionMenu.add(_managekeysaction);
		sessionMenu.add(_managenamespacesaction);
		menuManager.add(sessionMenu);
		/* Data menu */
		MenuManager dataMenu = new MenuManager("&Data");
		dataMenu.add(_xpathaction);
		dataMenu.add(_xupdateaction);
		dataMenu.add(new Separator());
		dataMenu.add(_encryptaction);
		dataMenu.add(_decryptaction);
		menuManager.add(dataMenu);
		/* Import menu */
		
		dataMenu.add(new Separator());
		importMenu = new MenuManager("&Import");
		dataMenu.add(importMenu);
		importMenu.add(_importfromdbaction);
		dataMenu.add(new Separator());
		MenuManager exportMenu = new MenuManager("&Export");
		dataMenu.add(exportMenu);
		exportMenu.add(_exporttoxmlaction);
		exportMenu.add(_exporttodbaction);
		return menuManager;
	}

	protected ToolBarManager createToolBarManager(int style) {
		ToolBarManager toolBarManager = new ToolBarManager(style);
		toolBarManager.add(_setupnewsession);
		toolBarManager.add(new Separator());
		toolBarManager.add(_connect);
		toolBarManager.add(_logout);
		toolBarManager.add(new Separator());
		toolBarManager.add(_xpathaction);
		toolBarManager.add(_xupdateaction);
		toolBarManager.add(new Separator());
		toolBarManager.add(_encryptaction);
		toolBarManager.add(_decryptaction);
		toolBarManager.add(new Separator());
		toolBarManager.add(_exitAction);
		return toolBarManager;
	}

	/**
	 * @return
	 */
	public Composite getComposite() {
		return this._composite;
	}

	public Context getContext() {
		return this.context;
	}

	protected void initializeBounds() {
		getShell().setMaximized(true);
		Rectangle splashRect = getShell().getBounds();
		Rectangle displayRect = Display.getCurrent().getBounds();
		int x = (displayRect.width - splashRect.width) / 2;
		int y = (displayRect.height - splashRect.height) / 2;
		getShell().setLocation(x, y);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see application.ExecutorCallbacksListener#onServerResponseReceived(java.lang.String,
	 *      java.lang.String)
	 */
	public void onServerResponseReceived(String request, String response) {
		output.appendText("\n\nReceived response from server " + response);

	}

	public void setupConnectedActions() {
		_exitAction.setEnabled(true);
		_xpathaction.setEnabled(true);
		_setupnewsession.setEnabled(false);
		_encryptaction.setEnabled(true);
		_decryptaction.setEnabled(true);
		_xupdateaction.setEnabled(true);
		_logout.setEnabled(true);
		_connect.setEnabled(false);
		output.setEditable(true);
		_managekeysaction.setEnabled(true);
		_managenamespacesaction.setEnabled(true);
		_importfromdbaction.setEnabled(true);
		_exporttoxmlaction.setEnabled(true);
		_exporttodbaction.setEnabled(true);
		this.setStatus("Ready");
	}

	public void setupNoSessionActions() {
		_exitAction.setEnabled(true);
		_xpathaction.setEnabled(false);
		_setupnewsession.setEnabled(true);
		_xupdateaction.setEnabled(false);
		_encryptaction.setEnabled(false);
		_decryptaction.setEnabled(false);
		_managekeysaction.setEnabled(false);
		_managenamespacesaction.setEnabled(false);
		_connect.setEnabled(false);
		_logout.setEnabled(false);
		_importfromdbaction.setEnabled(false);
		_exporttoxmlaction.setEnabled(false);
		output.setEditable(false);
		_exporttodbaction.setEnabled(false);
		this.setStatus("No session");
	}

	public void setupNotConnectedActions() {
		_exitAction.setEnabled(true);
		_xpathaction.setEnabled(false);
		_setupnewsession.setEnabled(true);
		_xupdateaction.setEnabled(false);
		_encryptaction.setEnabled(false);
		_decryptaction.setEnabled(false);
		_connect.setEnabled(true);
		_importfromdbaction.setEnabled(false);
		_logout.setEnabled(false);
		_managekeysaction.setEnabled(false);
		_exporttoxmlaction.setEnabled(false);
		_managenamespacesaction.setEnabled(false);
		output.setEditable(false);
		_exporttodbaction.setEnabled(false);
		this.setStatus("Not connected");
	}

	public void setupWaitingReply() {
		_exitAction.setEnabled(true);
		_xpathaction.setEnabled(false);
		_setupnewsession.setEnabled(false);
		_xupdateaction.setEnabled(false);
		_encryptaction.setEnabled(false);
		_decryptaction.setEnabled(false);
		_connect.setEnabled(false);
		output.setEditable(false);
		_logout.setEnabled(false);
		_exporttoxmlaction.setEnabled(false);
		_importfromdbaction.setEnabled(false);
		_managekeysaction.setEnabled(false);
		_managenamespacesaction.setEnabled(false);
		_exporttodbaction.setEnabled(false);
		this.setStatus("Executing request... please wait");
	}

}
