/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Terminal Emulator Widget
 * 
 * This widgets simulates a terminal emulator. It is derived from the
 * StyledText class. It is used to return data to user as well as
 * prompting for input (particularly XPath queries).
 */
package terminalemulator;

import java.util.Date;
import java.util.Iterator;
import java.util.Stack;
import java.util.Vector;

import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ST;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import application.Context;

/**
 * A TerminalEmulator defines a terminal look alike widget to prompt user for
 * input and present output of commands.
 * 
 * @author mriya3
 *  
 */
public class TerminalEmulator extends org.eclipse.swt.custom.StyledText {

	/**
	 * The ContextIMenuListener is invoked by a TerminalEmulator object just
	 * before the popup menu is shown, so that, depending on the context, menu
	 * items can be enabled or disabled
	 * 
	 * @author mriya3
	 *  
	 */
	class ContextIMenuListener implements IMenuListener {

		/**
		 * Called just before the popup menu is shown
		 */
		public void menuAboutToShow(IMenuManager arg0) {
			if (getSelectionCount() <= 0)
				_copyaction.setEnabled(false);
			else
				_copyaction.setEnabled(true);

			Clipboard clipboard = new Clipboard(Display.getDefault());
			String plainText = (String) clipboard.getContents(TextTransfer
					.getInstance());
			if (!(plainText == null) && getEditable())
				_pasteaction.setEnabled(true);
			else
				_pasteaction.setEnabled(false);
		}
	}

	/**
	 * In the terminal emulator we have to process every keypress indipendently.
	 * To do that we use this KeyListener implementation that, in fact, provides
	 * prompt editing functions and history. As soon as the Return key is
	 * pressed, the current command line is sent to the associated
	 * TerminalCallbacksListener implementation. The command history can be
	 * accessed by pressing UP and DOWN keys
	 * 
	 * @author mriya3
	 *  
	 */
	class TerminalInputKeyListener implements KeyListener {

		/* Last caret position in text */
		int lastCaretOffset = 0;

		/**
		 * Keyboard event processing
		 */
		public void keyPressed(KeyEvent e) {
			if (!getEditable())
				return;
			if (getCaretOffset() < (promptPosition - 1))
				setCaretOffset(getCharCount());
			if (lastCaretOffset == 0)
				lastCaretOffset = getCharCount();
			switch (e.keyCode) {
			case SWT.ARROW_LEFT:
				if (getCaretOffset() <= promptPosition) {
					setCaretOffset(promptPosition);
					e.doit = false;
				}
				break;
			case SWT.KEYPAD_CR:
			case SWT.CR: /* Enter or Return */

				if (getEditable()) {
					if (!getActualCmdLine().trim().equals("")) {
						String cmdline = getActualCmdLine().trim();
						if (!(cmdline.trim().equals(""))) {
							/* Look if its a variable set*/
							if((!(_context==null)) && (cmdline.length()>7) && cmdline.substring(0,5).equals("!set ")) {
								String[] values = cmdline.split(" ");
								if(values.length==3) {
									String key = values[1];
									String value = values[2];
									_context.setVariable(key,value);
									append("\nVariable "+key+" set to "+value);
									showPrompt();
								} else {
									append("\nInvalid set command");
									showPrompt();
								}
							} else if ((!(_context==null)) && (cmdline.length()>7) && cmdline.substring(0,5).equals("!get ")) {
								String[] values = cmdline.split(" ");
								if(values.length==2) {
									String key = values[1];
									String value = _context.getVariable(key);
									if(value!=null)
										append("\nVariable "+key+" is set to "+value);
									else
										append("\nVariable "+key+" not set");
									showPrompt();
								} else {
									append("\nInvalid get command");
									showPrompt();
								}
							} else {
								history.pop();
								history.push(cmdline);
								history.push("");
								indexInHistory = history.size() - 1;
								setEditable(false);
								listener.onCommand(cmdline);
							}
						}
					} else {
						setEnterEnabled(true);
						showPrompt();
						setEnterEnabled(false);
					}
				}
				e.doit = false;
				break;
			case SWT.HOME: /* Home */
				setCaretOffset(promptPosition);
				break;
			case SWT.ARROW_UP: /* Up */
				if (history.size() > 1) {
					indexInHistory -= 1;
					if (indexInHistory < 0)
						indexInHistory = 0;
					removeCompleteCommandLine();
					appendText((String) history.get(indexInHistory));
					setCaretOffset(getCharCount());
				} else
					setCaretOffset(lastCaretOffset);
				e.doit = false;
				break;
			case SWT.ARROW_DOWN: /* Down */
				if (history.size() > 1) {
					indexInHistory += 1;
					if (indexInHistory > (history.size() - 1))
						indexInHistory = history.size() - 1;
					removeCompleteCommandLine();
					appendText((String) history.get(indexInHistory));
					setCaretOffset(getCharCount());
				}
				e.doit = false;
				break;
			default:
				/* Re-launch key event */
				keyTyped(e);
			}
			lastCaretOffset = getCaretOffset();
		}

		public void keyReleased(KeyEvent e) {
		}

		public void keyTyped(KeyEvent e) {
		}
	}

	/**
	 * The TerminalInputTraverseListener implementation is used to avoid losing
	 * widget focus when pressing the TAB key, because it is used for the auto
	 * completion feature.
	 * 
	 * @author mriya3
	 *  
	 */
	class TerminalInputTraverseListener implements TraverseListener {

		/**
		 * Constructor
		 */
		public TerminalInputTraverseListener() {
		}

		/**
		 * Called when special keys are pressed; in this case we are interested
		 * only in the TAB key.
		 */
		public void keyTraversed(TraverseEvent arg0) {
			if (arg0.detail == SWT.TRAVERSE_TAB_NEXT) {
				long elapsed = new Date().getTime() - lastTab;
				arg0.doit = true;
				if (elapsed < 180) {
					doAddStarSelector();
				} else {
					doAutoCompletion();
				}
				arg0.detail = SWT.TRAVERSE_NONE;
				lastTab = new Date().getTime();
			}
		}
	}

	/* Variables and properties */
	private CopyAction _copyaction = new CopyAction(this);

	private PasteAction _pasteaction = new PasteAction(this);

	public int cursorPosition = 0;

	private Stack history = new Stack();

	private int indexInHistory = 1;

	boolean insEnabled = true;

	private TerminalCallbacksListener listener;

	private MenuManager menu_manager;

	private String prompt;

	public int promptPosition = 0;

	long lastTab = 0;

	boolean enabledEnter = true;

	private Context _context;

	/**
	 * Constructor for the TerminalEmulator. This constructor calls the
	 * StyledText constructor (super class) but also accepts an additional
	 * parameter to specify a reference to a callback listener implementation.
	 * 
	 * @param parent
	 * @param style
	 */
	public TerminalEmulator(Composite parent, int style,
			TerminalCallbacksListener listener) {
		super(parent, style);
		super.setEditable(false);
		this.listener = listener;
		addVerifyListener(new VerifyListener() {
			public void verifyText(VerifyEvent e) {
				if ((e.start < promptPosition) || (e.keyCode == SWT.ARROW_UP)
						|| (e.keyCode == SWT.ARROW_DOWN)
						|| (e.keyCode == SWT.HOME) || (e.keyCode == SWT.CR)
						|| (e.keyCode == SWT.KEYPAD_CR)
						|| ((!enabledEnter) && (e.text.indexOf("\n") >= 0))) {
					e.doit = false;
				}

			}
		});

		addTraverseListener(new TerminalInputTraverseListener());
		addKeyListener(new TerminalInputKeyListener());
		setupMenu();
		history.push("");
	}

	/**
	 *  Sets the context
	 */  
	public void setContext(Context c) {
		this._context = c;
	}
	
	
	/**
	 * Appends new text at the end
	 * 
	 * @param text
	 */
	public void appendText(String text) {
		setEnterEnabled(true);
		append(text);
		setSelection(getCharCount());
		setEnterEnabled(false);
	}

	/**
	 * Enables or disables pressing enter
	 * @param value
	 */
	public void setEnterEnabled(boolean value) {
		enabledEnter = value;
	}

	/**
	 * Append new text 
	 */
	public void append(String text) {
		setEnterEnabled(true);
		super.append(text);
		setEnterEnabled(false);
	}
	
	/**
	 * Overloaded method that also enables/disables
	 * context menu actions
	 */
	public void setEditable(boolean value) {
		super.setEditable(value);
		if(value==true)
			_pasteaction.setEnabled(true);
		else
			_pasteaction.setEnabled(false);
	}

	/**
	 * Returns the actual (maybe incomplete) command line
	 * 
	 * @return
	 */
	public String getActualCmdLine() {
		if (promptPosition != getCharCount())
			return getText(promptPosition, getCharCount() - 1);
		else
			return "";
	}

	/**
	 * Inserts text at the current caret position
	 * 
	 * @param text
	 */
	public void insertText(String text) {
		setEnterEnabled(true);
		if ((getCaretOffset() == getCharCount())) {
			appendText(text);
		} else {
			int currentCaret = getCaretOffset();
			int posInLine = getCaretOffset() - promptPosition;
			String currLine = getActualCmdLine();
			String secondpart = "";
			if (posInLine < 0) {
				currentCaret += 1;
				posInLine = 0;
			}
			secondpart = currLine.substring(posInLine);
			removeLastCharacters(secondpart.length());
			appendText(text + secondpart);
			setCaretOffset(currentCaret + 1);
		}
		setEnterEnabled(false);
	}

	/**
	 * Adds text formatted with the given stylerange, color and style.
	 * 
	 * @param styleRange
	 * @param text
	 * @param foregroundcolor
	 * @param fontstyle
	 */
	public void outputText(StyleRange styleRange, String text,
			int foregroundcolor, int fontstyle) {
		setEnterEnabled(true);
		styleRange.start = getCharCount();
		styleRange.length = text.length();
		styleRange.foreground = getShell().getDisplay().getSystemColor(
				foregroundcolor);
		styleRange.fontStyle = fontstyle;

		append(text);
		setStyleRange(styleRange);
		setSelection(getCharCount());
		setEnterEnabled(false);
	}

	/**
	 * Removes all the entered input up to the start of the prompt.
	 */
	public void removeCompleteCommandLine() {
		int startSelection = promptPosition;
		int endSelection = getCharCount();
		setSelection(startSelection, endSelection);
		invokeAction(ST.DELETE_PREVIOUS);
	}

	/**
	 * Removes n last characters from the command line
	 */
	public void removeLastCharacters(int n) {
		for (int i = 1; i < n; i++)
			this.invokeAction(ST.DELETE_PREVIOUS);
	}

	/**
	 * Sets focus on this widget
	 */
	public boolean setFocus() {
		if (!(this.isFocusControl()))
			if (getCaretOffset() < promptPosition)
				setCaretOffset(promptPosition);
		return super.setFocus();
	}

	/**
	 * Sets the prompt text
	 * 
	 * @param text
	 */
	public void setPrompt(String text) {
		this.prompt = text;
	}

	/**
	 * Setup the context menu and actions
	 */
	private void setupMenu() {
		menu_manager = new MenuManager();
		this.setMenu(menu_manager.createContextMenu(this));
		menu_manager.addMenuListener(new ContextIMenuListener());
		menu_manager.add(_copyaction);
		menu_manager.add(_pasteaction);
	}

	/**
	 * Shows the prompt
	 */
	public void showPrompt() {
		setEnterEnabled(true);
		outputText(new StyleRange(), "\n" + this.prompt, SWT.COLOR_GREEN,
				SWT.BOLD);
		promptPosition = getCharCount();
		setSelection(promptPosition);
		setEnterEnabled(false);
	}

	/**
	 * Adds the /* at the end of the command line
	 *  
	 */
	public void doAddStarSelector() {
		String cmdLine = getActualCmdLine().trim();
		if (cmdLine.endsWith("/*") || cmdLine.endsWith("["))
			return;
		else if (cmdLine.endsWith("/"))
			appendText("*");
		else
			appendText("/*");
	}

	/**
	 * Performs input autocompletion based on context words list
	 * 
	 * Amos' method... don't know if it's good... but HEY! It works ;) !
	 */
	private void doAutoCompletion() {

		final class Helpers {
			public Vector filterSamePrefix(String prefix, Vector words) {
				Vector result = new Vector();
				for (int i = 0; i < words.size(); i++) {
					String value = ((String) words.get(i));
					if (value.length() >= prefix.length())
						if (((String) words.get(i)).substring(0,
								prefix.length()).equals(prefix)) {
							result.add(words.get(i));
						}
				}
				return result;
			}

			public boolean occursInEvery(String prefix, Vector sameRadix) {
				int k = 0;
				for (int i = 0; i < sameRadix.size(); i++) {
					String value = ((String) sameRadix.get(i));
					if (value.length() >= prefix.length())
						if (value.substring(0, prefix.length()).equals(prefix)) {
							k++;
						}
				}
				return k == sameRadix.size();
			}
		}

		String cmdLine = getActualCmdLine().trim();
		if (cmdLine.equals("")) {
			appendText(listener.getContext().getContextPath());
			return;
		}
		Helpers h = new Helpers();
		String matcher = cmdLine;
		String prefix = "";
		Vector wordsList = h.filterSamePrefix(cmdLine, listener.getContext()
				.getWordList());
		if (wordsList.size() < 1)
			return;
		if (wordsList.size() == 1) {
			if (!(wordsList.get(0).equals(cmdLine))) {
				appendText(((String) wordsList.get(0)).substring(cmdLine
						.length()));
				return;
			}
		}
		Iterator wordsIterator = wordsList.iterator();
		while (wordsIterator.hasNext()) {
			String word = (String) wordsIterator.next();
			for (int k = 1; k < word.length(); k++) {
				prefix = word.substring(0, k);
				if (!(h.occursInEvery(prefix, wordsList))) {
					if (!matcher.equals(cmdLine)) {

						appendText(matcher.substring(cmdLine.length()));
					}
					return;
				}
				matcher = prefix;
			}
		}
	}

}
