/**
Master Project WS-2004 ~ SS 2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Collects data to encrypt a node
 */
package wizards.encrypt;

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import application.Context;

/**
 * @author mriya3
 *  
 */
public class SetEncryptionPage extends WizardPage {

	Context context;

	/* Fields */
	Text location;

	Combo methodCombo;

	Combo idCombo;

	String[] methods = { "AES_128", "DESede", "RSA_V1_5" };

	private boolean canContinue;

	protected SetEncryptionPage(Context context) {
		super("SetEncryptionPage");
		setTitle("Encrypt node");
		setDescription("Please set encryption parameters");
		this.context = context;
	}

	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 2;
		layout.verticalSpacing = 5;

		GridData gd = new GridData(GridData.FILL_HORIZONTAL);

		/* XUpdate modification combobox */
		Label methodLabel = new Label(container, SWT.NULL);
		methodLabel.setText("&Method:");

		methodCombo = new Combo(container, SWT.READ_ONLY);
		methodCombo.setItems(methods);

		methodCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if (methodCombo.getText().trim().length() > 0) {
					idCombo.removeAll();
					Vector items = context.getEncryptionKeys(methodCombo
							.getText());
					if (items.size() > 0) {
						Iterator it_iter = items.iterator();
						while (it_iter.hasNext()) {
							idCombo.add((String) it_iter.next());
						}
					}
				}
				validate();
			}
		});

		methodCombo.setLayoutData(gd);

		Label idLabel = new Label(container, SWT.NULL);
		idLabel.setText("&Key:");
		idCombo = new Combo(container, SWT.READ_ONLY);

		idCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});

		idCombo.setLayoutData(gd);

		/* XPath context input box */
		Label contextLabel = new Label(container, SWT.NULL);
		contextLabel.setText("&Location:");

		location = new Text(container, SWT.BORDER | SWT.SINGLE);

		location.setText(context.getContextPath() + context.getSelector());

		location.setLayoutData(gd);

		location.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});

		canContinue = false;
		setPageComplete(false);
		setControl(container);
	}

	void onEnterPage() {
		getWizard().getContainer().updateButtons();
		validate();
	}

	private void validate() {

		if (methodCombo.getText().trim().length() == 0) {
			updateStatus("Encryption method to use must be selected.");
			return;
		}

		if (idCombo.getText().trim().length() == 0) {
			updateStatus("Encryption key must be selected.");
			return;
		}

		if (location.getText().trim().length() == 0) {
			updateStatus("Location to encrypt must be specified.");
			return;
		}

		saveData();
		canContinue = true;
		getWizard().getContainer().updateButtons();
		updateStatus(null);
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		if(message==null)
			canContinue = false;
	}

	public boolean canFlipToNextPage() {
		return canContinue;
	}

	/**
	 * Saves values entered
	 *  
	 */
	public void saveData() {
		EncryptionWizard wizard = (EncryptionWizard) getWizard();
		wizard.setMethod(new String(methodCombo.getText()));
		wizard.setKeyId(new String(idCombo.getText()));
		wizard.setLocation(new String(location.getText()));
	}

	public IWizardPage getNextPage() {
		saveData();
		DoEncryptionPage page = ((EncryptionWizard) getWizard()).executexupdate_page;
		page.onEnterPage();
		return page;
	}

}
