/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Generates a new key
 */
package wizards.genkey;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import application.Context;
import application.Executor;
import application.ExecutorCallbacksListener;

/**
 * @author mriya3
 *  
 */
public class ExecuteGenKeyPage extends WizardPage {

	private Context context;

	private Text logText;

	private Button saveLogButton;
	
	private Button saveEncButton;
	
	private Button saveDecButton;

	private String algorithm;

	private String password;

	private Button saveKeyButton;

	class GenKeyResultListener implements ExecutorCallbacksListener {

		public void dataAvailable(Executor ex, final String request,
				BufferedReader response) {

			Document doc = null;
			try {
				/* Parse result */
				doc = (new SAXBuilder()).build(response);
			} catch (final JDOMException e) {
				/* Sync start */context.getDisplay().syncExec(new Runnable() {
					public void run() {
						logText.append("Cannot execute xupdate " + request
								+ "\n");
						e.printStackTrace();
						/* Sync end */}
				});
				return;
			} catch (final IOException e) {
				/* Sync start */context.getDisplay().syncExec(new Runnable() {
					public void run() {
						logText.append("Cannot connect to server\n");
						e.printStackTrace();
						/* Sync end */}
				});
				return;
			}

			/* Parse answer */
			Element rootNode = doc.getRootElement();
			Element requests = rootNode.getChild("requests");
			Element action = requests.getChild("action");
			Element result = action.getChild("result");
			String exitCode = result.getAttributeValue("exitCode");
			final String motivation = result.getText();
			final Element key = result.getChild("key");
			/* For asymmetric encryption we have two keys */
			final Element enckey = result.getChild("encryption");
			final Element deckey = result.getChild("decryption");
			if (exitCode.equals("200"))
						/* Save key data, symmetric type */
						if(key!=null) {
							/* Sync start */context.getDisplay().syncExec(new Runnable() {
								public void run() {
							logText.append("New key for method " + algorithm
									+ " correctly created.\n");
							setMessage("Key generation finished.");
							String keydata = key.getText();
							((GenKeyWizard) getWizard()).setKeyData(keydata);
							setPageComplete(true);
							saveKeyButton.setEnabled(true);
							((GenKeyWizard) getWizard()).isFinished();
							getWizard().getContainer().updateButtons();
								/* Sync end */}
							});
						} else {
							/* Sync start */context.getDisplay().syncExec(new Runnable() {
								public void run() {
							logText.append("New keypair for method " + algorithm
									+ " correctly created.\n");
							setMessage("Keypair generation finished.");
							String enckeydata = enckey.getText();
							String deckeydata = deckey.getText();
							((GenKeyWizard) getWizard()).setEncKeyData(enckeydata);
							((GenKeyWizard) getWizard()).setDecKeyData(deckeydata);
							setPageComplete(true);
							/* Enable save keys buttons */
							saveEncButton.setEnabled(true);
							saveDecButton.setEnabled(true);
							((GenKeyWizard) getWizard()).isFinished();
							getWizard().getContainer().updateButtons();
								/* Sync end */}
							});
						}	
							

			else
				/* Sync start */context.getDisplay().syncExec(new Runnable() {
					public void run() {
						setErrorMessage("Key generation failed.");
						logText.append("Error! " + motivation + "\n");
						setPageComplete(false);
						/* Sync end */}
				});
		}
	}

	/**
	 * Constructor
	 * @param context
	 */
	protected ExecuteGenKeyPage(Context context) {
		super("Generate new key");
		setTitle("Generate new key");
		setDescription("Generating key. See log for information");
		this.context = context;
	}

	/**
	 * Creates controls in this page
	 */
	public void createControl(Composite composite) {
		Composite container = new Composite(composite, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 1;
		layout.verticalSpacing = 5;

		Label logLabel = new Label(container, SWT.NULL);
		logLabel.setText("Operation log: ");

		logText = new Text(container, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		GridData gd2 = new GridData(GridData.FILL_BOTH);
		logText.setLayoutData(gd2);

		Composite bar = new Composite(container, SWT.NULL);
		GridLayout barlayout = new GridLayout();
		bar.setLayout(barlayout);
		barlayout.numColumns = 4;
		barlayout.verticalSpacing = 5;
		
		saveLogButton = new Button(bar, SWT.NULL | SWT.RIGHT);
		saveLogButton.setText("Save log...");
		saveLogButton.setEnabled(true);
		saveLogButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				DataOutputStream out = null;
				try {
					FileDialog fd = new FileDialog(getShell(), SWT.SAVE);
					fd.setText("Save log to file");
					fd.setFilterPath(System.getProperty("os.path"));
					String[] filterExt = { "*.txt", "*.*" };
					fd.setFilterExtensions(filterExt);
					String selected = fd.open();
					File file = new File(selected);
					out = new DataOutputStream(new FileOutputStream(file));
					out.writeBytes(logText.getText());
					out.close();
					logText.append("\nLog saved to file: " + selected);
				} catch (Exception e1) {
					logText.append("\nCannot save log to file!");
					try {
						if (out != null)
							out.close();
					} catch (IOException e2) {
						e2.printStackTrace();
					}
				}
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		
		saveKeyButton = new Button(bar, SWT.NULL | SWT.RIGHT);
		saveKeyButton.setText("Save key");
		saveKeyButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				DataOutputStream out = null;
				try {
					FileDialog fd = new FileDialog(getShell(), SWT.SAVE);
					fd.setText("Save key to file");
					fd.setFilterPath(System.getProperty("os.path"));
					String[] filterExt = { "*.rsk", "*.*" };
					fd.setFilterExtensions(filterExt);
					String selected = fd.open();
					File file = new File(selected);
					out = new DataOutputStream(new FileOutputStream(file));
					out.writeUTF(((GenKeyWizard) getWizard()).getKeyData());
					out.close();
					logText.append("\nKey saved to file: " + selected);
				} catch (Exception e1) {
					logText.append("\nCannot save key to file!");
					try {
						if (out != null)
							out.close();
					} catch (IOException e2) {
						e2.printStackTrace();
					}
				}
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		saveKeyButton.setEnabled(false);
		
		
		saveEncButton = new Button(bar, SWT.NULL | SWT.RIGHT);
		saveEncButton.setText("Save encryption key");
		saveEncButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				DataOutputStream out = null;
				try {
					FileDialog fd = new FileDialog(getShell(), SWT.SAVE);
					fd.setText("Save encryption key");
					fd.setFilterPath(System.getProperty("os.path"));
					String[] filterExt = { "*.rsk", "*.*" };
					fd.setFilterExtensions(filterExt);
					String selected = fd.open();
					File file = new File(selected);
					out = new DataOutputStream(new FileOutputStream(file));
					out.writeUTF(((GenKeyWizard) getWizard()).getEncKeyData());
					out.close();
					logText.append("\nEncryption key saved to file: " + selected);
				} catch (Exception e1) {
					logText.append("\nCannot save key to file!");
					try {
						if (out != null)
							out.close();
					} catch (IOException e2) {
						e2.printStackTrace();
					}
				}
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		saveEncButton.setEnabled(false);
		
		saveDecButton = new Button(bar, SWT.NULL | SWT.RIGHT);
		saveDecButton.setText("Save decryption key");
		saveDecButton.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				DataOutputStream out = null;
				try {
					FileDialog fd = new FileDialog(getShell(), SWT.SAVE);
					fd.setText("Save decryption key");
					fd.setFilterPath(System.getProperty("os.path"));
					String[] filterExt = { "*.rsk", "*.*" };
					fd.setFilterExtensions(filterExt);
					String selected = fd.open();
					File file = new File(selected);
					out = new DataOutputStream(new FileOutputStream(file));
					out.writeUTF(((GenKeyWizard) getWizard()).getDecKeyData());
					out.close();
					logText.append("\nDecryption key saved to file: " + selected);
				} catch (Exception e1) {
					logText.append("\nCannot save key to file!");
					try {
						if (out != null)
							out.close();
					} catch (IOException e2) {
						e2.printStackTrace();
					}
				}
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		saveDecButton.setEnabled(false);

		setPageComplete(false);
		setControl(container);
	}

	/**
	 * Performs some actions when accessing this page
	 *  
	 */
	void onEnterPage() {
		GenKeyWizard wizard = (GenKeyWizard) getWizard();
		password = wizard.getPassword();
		algorithm = wizard.getAlgorithmName();
		executeGenKey();
	}

	/**
	 * Launch GenKey execution
	 */
	private void executeGenKey() {
		String query = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><roxannequery login=\""
				+ context.getSession().getLogin()
				+ "\" password=\""
				+ context.getSession().getPassword()
				+ "\" sessionId=\""
				+ context.getSession().getSessionId()
				+ "\" persistent=\"true\"><requests><action type=\"genkey\" method=\""
				+ algorithm
				+ "\" password=\""
				+ password
				+ "\"/></requests></roxannequery>";
		Executor ex = new Executor(new GenKeyResultListener(), context
				.getSession());
		ex.setQuery(query);
		ex.start();
	}

	/**
	 * Returns if we can go to next page (finish)
	 */
	public boolean canFlipToNextPage() {
		return false;
	}

}
