/**
Master Project WS-2004 ~ SS 2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Collects data to generate a new key
 */
package wizards.genkey;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import application.Context;

/**
 * @author mriya3
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class GenKeyParamPage extends WizardPage {

	Context context;

	/* Fields */
	Text password;

	Combo algorithmName;

	String[] methods = { "AES_128", "DESede", "RSA_V1_5" };

	private boolean canContinue;

	protected GenKeyParamPage(Context context) {
		super("GenKeyParameters");
		setTitle("Key Generation");
		setDescription("Setup key parameters");
		this.context = context;
	}

	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 2;
		layout.verticalSpacing = 5;

		GridData gd = new GridData(GridData.FILL_HORIZONTAL);

		/* Method selection */
		Label methodLabel = new Label(container, SWT.NULL);
		methodLabel.setText("&Method:");

		algorithmName = new Combo(container, SWT.READ_ONLY);
		algorithmName.setItems(methods);

		algorithmName.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});
		algorithmName.setLayoutData(gd);
		Label pwdLabel = new Label(container, SWT.NULL);
		pwdLabel.setText("&Password:");
		password = new Text(container, SWT.BORDER | SWT.SINGLE);
		password.setText("");
		password.setLayoutData(gd);
		password.setEchoChar('*');
		password.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});
		password.setEnabled(false);
		
		canContinue = false;
		setPageComplete(false);
		setControl(container);
	}

	/**
	 * Actions executed when entering this page
	 *
	 */
	void onEnterPage() {
		validate();
		getWizard().getContainer().updateButtons();
	}

	/**
	 * Validates the current entries
	 *
	 */
	private void validate() {
		
		if (algorithmName.getText().trim().length() == 0) {
				updateStatus("A method must be specified.");
				return;
		} else if(algorithmName.getText().equals("AES_128") 
				|| algorithmName.getText().equals("DESede")) {
			password.setEnabled(true);
		} else {
			password.setEnabled(false);
		}
		
		if (password.getEnabled() && (password.getText().trim().length() == 0)) {
			updateStatus("A password must be specified.");
			return;
		}

		saveData();
		canContinue = true;
		getWizard().getContainer().updateButtons();
		updateStatus(null);

	}

	/**
	 * Updates the status message, if null the page is complete
	 * @param message
	 */
	private void updateStatus(String message) {
		setErrorMessage(message);
		if(message==null)
			canContinue = false;
	}

	/**
	 * True if we can go to next page
	 */
	public boolean canFlipToNextPage() {
		return canContinue;
	}

	/**
	 * Saves values entered
	 *  
	 */
	public void saveData() {
		GenKeyWizard wizard = (GenKeyWizard) getWizard();
		wizard.setAlgorithmName(new String(algorithmName.getText()));
		wizard.setPassword(new String(password.getText()));
	}

	/**
	 * Returns the next page
	 */
	public IWizardPage getNextPage() {
		saveData();
		ExecuteGenKeyPage page = ((GenKeyWizard) getWizard()).dogenkey_page;
		page.onEnterPage();
		return page;
	}

}
