/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Students :   Amos Brocco <amos.brocco@unifr.ch>
			 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

package wizards.importfromdb;

import java.util.Vector;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;

public class XmillPage1 extends WizardPage {
	private static String[] DATABASES = {};


	private List dbList;


	private boolean canContinue = false;
	
	private Button insDbInfoCheck;

	private Button tryTableNotFound;

	private Button tryErrorGeneral;

	private Button tryDbNotFound;

	
	public XmillPage1() {
		super("wizardPage");
		setTitle("XMill Database Importer");
		setDescription("Choose database");
	}

	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.verticalSpacing = 9;

		Label label;

		GridData gd = new GridData(GridData.FILL_HORIZONTAL);

		label = new Label(container, SWT.NULL);
		label.setText("&Choose Database:");

		dbList = new List(container, SWT.BORDER | SWT.V_SCROLL);
		GridData gdlist = new GridData(GridData.FILL_BOTH);
		dbList.setLayoutData(gdlist);
		dbList.setItems(DATABASES);
		
		dbList.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				validate();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		
		
		insDbInfoCheck = new Button( container, SWT.CHECK );
		insDbInfoCheck.setText( "Import database information" );
		insDbInfoCheck.setSelection(false);
		
		tryTableNotFound = new Button( container, SWT.CHECK );
		tryTableNotFound.setText( "Continue if a table is not found" );
		tryTableNotFound.setSelection(false);
		
		tryDbNotFound = new Button( container, SWT.CHECK );
		tryDbNotFound.setText( "Continue if database is not found" );
		tryDbNotFound.setSelection(false);
		
		tryErrorGeneral = new Button( container, SWT.CHECK );
		tryErrorGeneral.setText( "Force continue on error" );
		tryErrorGeneral.setSelection(false);


		setControl(container);
		validate();
	}




	private void createLine(Composite parent, int ncol) {
		Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL
				| SWT.BOLD);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = ncol;
		line.setLayoutData(gridData);
	}
	
	void onEnterPage() {
		canContinue = false;
		XmillWizard wizard = (XmillWizard) getWizard();
		
		Vector dbs = new Vector();
		try {
			wizard.initDbConnection();
			dbs = wizard.getDatabaseList();
		} catch (Exception e) {
			dbList.setItems(new String[0]);
			updateStatus("Cannot get tables from database");
			return;
		}
		DATABASES = (String[]) dbs.toArray(new String[0]);
		getWizard().getContainer().updateButtons();
		dbList.setItems(DATABASES);
		validate();
			
	}

	public boolean canFlipToNextPage() {
		return canContinue;
	}
	

	private void validate() {
		if((dbList.getSelectionCount()==0)) {
			updateStatus("Select one database from list");
			return;
		}

		saveData();
		canContinue = true;
		getWizard().getContainer().updateButtons();
		updateStatus(null);
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
	}
	
	public void saveData() {
		XmillWizard wizard = (XmillWizard) getWizard();
		wizard.setDatabase(dbList.getItem(dbList.getSelectionIndex()));
		wizard.setExportDbInfo(insDbInfoCheck.getSelection());
		wizard.setTryOnDatabaseNotfound(tryDbNotFound.getSelection());
		wizard.setTryOnErrorGeneral(tryErrorGeneral.getSelection());
		wizard.setTryOnTableNotfound(tryTableNotFound.getSelection());

	}

	public IWizardPage getNextPage() {
		saveData();
		XmillPage2 page = (XmillPage2) ((XmillWizard) getWizard()).page2;
		page.onEnterPage();
		return page;
	}
}
