/**
Master Project WS-2004 ~ SS-2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Students :   Amos Brocco <amos.brocco@unifr.ch>
			 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

package wizards.importfromdb;

import java.util.Vector;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Text;

import support.ImportFromMySQL;
import support.ImportFromPostgreSQL;
import support.ImportInterface;
import application.Context;
import application.UserInterface;

public class XmillWizard extends Wizard {
	public XmillPage0 page0;

	public XmillPage1 page1;

	public XmillPage2 page2;

	public XmillPage3 page3;
	
	public XmillPage4 page4;
	
	public XmillPage5 page5;


	private boolean finished;

	private String provider;

	private String server;

	private String port;

	private String user;

	private String password;

	private String db;
	
	ImportInterface importer;

	private Vector dblist;

	private Vector tblist;

	private Vector tbtoexport;

	private Vector tbtoexportdata;

	private Vector tbtoexportstructure;

	private String xmldata;

	private String importloc;

	private String contextlocation;

	private Vector tables;

	private UserInterface ui;

	private boolean exportDbInfo = false;

	private boolean tryContinueOnErrorGeneral;

	private boolean tryContinueOnDatabaseNotFound;

	private boolean tryContinueOnTableNotFound;
	

	public XmillWizard(UserInterface ui) {
		super();
		this.ui = ui;
		this.contextlocation = ui.getContext().getContextPath();
		setNeedsProgressMonitor(true);

	}

	public void addPages() {
		page0 = new XmillPage0();
		page1 = new XmillPage1();
		page2 = new XmillPage2();
		page3 = new XmillPage3();
		page4 = new XmillPage4();
		page5 = new XmillPage5(ui.getContext());
		addPage(page0);
		addPage(page1);
		addPage(page2);
		addPage(page3);
		addPage(page4);
		addPage(page5);
	}

	public boolean performFinish() {
		return true;
	}
	
	public boolean canFinish() {
		return finished;
	}
	
	public void isFinished() {
		finished = true;
	}
	
	/**
	 * METHODS TO STORE AND RETRIEVE DATA BETWEEN PAGES
	 */
	
	public void setProvider(String provider) {
		this.provider = provider;
	}
	
	public String getProvider() {
		return provider;
	}
	
	public void setServer(String server) {
		this.server = server;
	}
	
	public String getServer() {
		return server;
	}
	
	public void setServerPort(String port) {
		this.port  = port;
	}
	
	public String getServerPort() {
		return this.port;
	}
	
	public void setUsername(String user) {
		this.user = user;
	}
	
	public String getUsername() {
		return user;
	}
	
	public void setPassword(String pwd) {
		this.password = pwd;
	}
	
	public String getPassword() {
		return this.password;
	}
	
	
	
	public void initDbConnection() throws Exception {
		if(provider.equals("MySQL")) {
			/* Initialize MySQL importer */
			importer = new ImportFromMySQL();
			importer.initConnection(server,port,"",user,password);
		} else if(provider.equals("PostgreSQL")) {
			/* Initialize PostgreSQL importer */
			importer = new ImportFromPostgreSQL();
			importer.initConnection(server,port,"template1",user,password);
		}
	}
	
	public Vector getDatabaseList() throws Exception {
		if(importer==null)
			initDbConnection();
		this.dblist = importer.listDatabase();
		return dblist;
	}
	
	public void setDatabase(String database) {
		this.db = database;
	}
	
	public Vector getTablesList() {
		return this.tables;
	}
	
	public void setTablesToExportData(Vector tables) {
		this.tbtoexportdata = tables;
	}
	
	public void setTablesToExportStructure(Vector tables) {
		this.tbtoexportstructure = tables;
	}
	
	public void setLogTextbox(Text log) {
		importer.setLogText(log);
	}
	
	public void exportData() throws Exception {
		if(exportDbInfo)
			importer.setInformationDatabaseExport();
		if(tryContinueOnErrorGeneral)
			importer.setContinueOnErrorGeneral();
		if(tryContinueOnTableNotFound)
			importer.setContinueOnTableNotFound();
		if(tryContinueOnDatabaseNotFound)
			importer.setContinueOnDatabaseNotFound();
		importer.setTablesToExportData(tbtoexportdata);
		importer.setTablesToExportStructure(tbtoexportstructure);
		xmldata = importer.exportData();
	}
	
	public String getXMLData() {
		return xmldata;
	}

	public void setImportLocation(String text) {
		this.importloc = text;
	}
	
	public String getImportLocation() {
		return importloc;
	}

	/**
	 * @return
	 */
	public String getContextLocation() {
		return contextlocation;
	}

	/**
	 * @param tables
	 */
	public void setTables(Vector tables) {
		this.tables = tables;		
	}

	/**
	 * @return
	 * @throws Exception
	 */
	public Vector getFullTablesList() throws Exception {
		importer.selectDatabase(db);
		this.tblist = importer.listTables();
		return tblist;
	}
	
	public Context getContext() {
		return this.ui.getContext();
	}

	public void setExportDbInfo(boolean selection) {
		this.exportDbInfo = selection;	
	}
	
	
	public void setTryOnTableNotfound(boolean selection) {
		this.tryContinueOnTableNotFound = selection;	
	}
	
	public void setTryOnDatabaseNotfound(boolean selection) {
		this.tryContinueOnDatabaseNotFound = selection;
	}
	
	public void setTryOnErrorGeneral(boolean selection) {
		this.tryContinueOnErrorGeneral = selection;
	}

	public void forceClose() {
		importer.forceCloseConnection();		
	}
	
	
	
	

}
