/**
Master Project WS-2004 ~ SS 2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Collects data to perform XUpdate
 */
package wizards.session;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import application.Context;

/**
 * Wizard to configure a session
 * 
 * Requires the user to enter session parameters such as server address, server
 * port, login username and password
 * 
 * @author mriya3
 *  
 */
public class ConfigureSessionPage extends WizardPage {

	Context context;

	/* Fields */
	Text addressText;

	Text portText;

	Text loginText;

	Text passwordText;

	int portNumber = 0;

	protected ConfigureSessionPage(Context context) {
		super("ConfigureSessionPage");
		setTitle("Configure session");
		setDescription("Session parameters configuration");
		this.context = context;
	}

	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 2;
		layout.verticalSpacing = 5;

		GridData gd = new GridData(GridData.FILL_HORIZONTAL);

		Label addressLabel = new Label(container, SWT.NULL);
		addressLabel.setText("Server &Address:");

		addressText = new Text(container, SWT.BORDER | SWT.SINGLE);
		addressText.setText("localhost");
		addressText.setLayoutData(gd);

		addressText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});

		Label portLabel = new Label(container, SWT.NULL);
		portLabel.setText("&Server Port:");

		portText = new Text(container, SWT.BORDER | SWT.SINGLE);
		portText.setText("8351");
		portText.setLayoutData(gd);
		portText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});

		createLine(container, 2);

		Label loginLabel = new Label(container, SWT.NULL);
		loginLabel.setText("&Login:");

		loginText = new Text(container, SWT.BORDER | SWT.SINGLE);
		loginText.setLayoutData(gd);

		loginText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});

		Label passwordLabel = new Label(container, SWT.NULL);
		passwordLabel.setText("&Password:");

		passwordText = new Text(container, SWT.BORDER | SWT.SINGLE);
		passwordText.setEchoChar('*');
		passwordText.setLayoutData(gd);

		/* Import 'old' data */
		if (!(context.getSession() == null)) {
			addressText.setText(context.getSession().getServerAddress());
			portText.setText(new Integer(context.getSession().getServerPort())
					.toString());
			loginText.setText(context.getSession().getLogin());
			passwordText.setText(context.getSession().getPassword());
		}

		setPageComplete(false);
		setControl(container);
		validate();
	}

	private void validate() {
		/* Validate input */
		if (addressText.getText().trim().equals("")) {
			updateStatus("Please provide a valid server address");
			return;
		}

		try {
			portNumber = 0;
			portNumber = new Integer(portText.getText()).intValue();
		} catch (NumberFormatException e) {
			updateStatus("Please enter a valid port number");
			return;
		}

		if (portNumber <= 0) {
			updateStatus("Please enter a valid port number");
			return;
		}

		if (loginText.getText().trim().equals("")) {
			updateStatus("Please enter a login name");
			return;
		}

		updateStatus(null);
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	public String getServerAddress() {
		return addressText.getText();
	}

	public String getServerPort() {
		return portText.getText();
	}

	public String getLogin() {
		return loginText.getText();
	}

	public String getPassword() {
		return passwordText.getText();
	}

	public int getPortNumber() {
		return this.portNumber;
	}

	private void createLine(Composite parent, int ncol) {
		Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL
				| SWT.BOLD);
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = ncol;
		line.setLayoutData(gridData);
	}

}
