/**
Master Project WS-2004 ~ SS 2005, Computer Science

University of Fribourg, Information Systems Research Group (http://diuf.unifr.ch/is)

Student :    Amos Brocco <amos.brocco@unifr.ch>
Supervisors: Prof. Dr. Andreas Meier <andreas.meier@unifr.ch>
             Nicolas Werro <nicolas.werro@unifr.ch>
        __  __
 _ _ ___\ \/ /__ _ _ _  _ _  ___
| '_/ _ \>  </ _` | ' \| ' \/ -_)
|_| \___/_/\_\__,_|_||_|_||_\___|

roXanne, Secure XML Data Storage Framework
x.core & x.click Copyright (C) 2004-2005 Amos Brocco <amos.brocco@unifr.ch>
x.mill Copyright (C) 2004-2005 Andrea Ghirlanda <andrea.ghirlanda@unifr.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

This product includes software developed by the JDOM Project (http://www.jdom.org/).
The Jaxup library is Copyright (c) 2001 - 2003 Erwin Bolwidt, used and modified under the BSD License.
Encryption modules used are Copyright (c) 2000 - 2004 The Legion Of The Bouncy Castle (http://www.bouncycastle.org).
JAXEN  is Copyright 2003 (C) The Werken Company. All Rights Reserved.
Base64 was released as Public Domain by its author Robert Harder (rob@iharder.net)
MySQL JDBC Connector is (c) 1995-2005 MySQL AB. All rights reserved. Released under the GNU/GPL License.
PostgreSQL JDBC Driver is Copyright (c) 2004-2005 The PostgreSQL Global Development Group, distributed under the BSD license
JArgs was created Steve Purcell, now mantained by Ewan Mellorby and released under the BSD licence.
**/

/*
 * Collects data to perform XUpdate
 */
package wizards.xupdate;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import application.Context;

/**
 * @author mriya3
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class EnterXUpdatePage extends WizardPage {

	Context context;

	/* Fields */
	Text contextLocation;

	Combo modificationCombo;

	Text modificationCommand;

	String[] modifications = { "xupdate:insert-before", "xupdate:insert-after",
			"xupdate:append", "xupdate:update", "xupdate:remove" };

	private boolean canContinue;

	protected EnterXUpdatePage(Context context) {
		super("EnterXUpdatePage");
		setTitle("Execute XUpdate query");
		setDescription("Please enter XUpdate parameters");
		this.context = context;
	}

	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 2;
		layout.verticalSpacing = 5;

		GridData gd = new GridData(GridData.FILL_HORIZONTAL);

		/* XUpdate modification combobox */
		Label modificationLabel = new Label(container, SWT.NULL);
		modificationLabel.setText("&Modification:");

		modificationCombo = new Combo(container, SWT.READ_ONLY);
		modificationCombo.setItems(modifications);

		modificationCombo.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});

		modificationCombo.setLayoutData(gd);

		/* XPath context input box */
		Label contextLabel = new Label(container, SWT.NULL);
		contextLabel.setText("&Selection:");

		contextLocation = new Text(container, SWT.BORDER | SWT.SINGLE);

		contextLocation.setText(context.getContextPath()
				+ context.getSelector());

		contextLocation.setLayoutData(gd);

		contextLocation.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});

		GridData gd2 = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
		Label modificationCommandLabel = new Label(container, SWT.NULL);
		modificationCommandLabel.setText("Content: ");
		modificationCommandLabel.setLayoutData(gd2);

		modificationCommand = new Text(container, SWT.BORDER | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		GridData gd3 = new GridData(GridData.FILL_BOTH);
		modificationCommand.setLayoutData(gd3);

		modificationCommand.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				validate();
			}
		});

		canContinue = false;
		setPageComplete(false);
		setControl(container);
	}

	void onEnterPage() {
		modificationCommand.setText("");
		getWizard().getContainer().updateButtons();
		validate();
	}

	private void validate() {
		if (this.modificationCombo.getText().equals(modifications[4]))
			modificationCommand.setEnabled(false);
		else
			modificationCommand.setEnabled(true);

		if (contextLocation.getText().trim().length() == 0) {
			updateStatus("Context XPath location must be specified.");
			return;
		}

		if (modificationCombo.getText().equals("")) {
			updateStatus("Modification must be selected.");
			return;
		}
		saveData();
		canContinue = true;
		getWizard().getContainer().updateButtons();
		updateStatus(null);
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		if(message==null)
			canContinue = false;
	}

	public boolean canFlipToNextPage() {
		return canContinue;
	}

	/**
	 * Saves values entered
	 *  
	 */
	public void saveData() {
		XUpdateWizard wizard = (XUpdateWizard) getWizard();
		wizard.setContextLocation(new String(contextLocation.getText()));
		wizard.setModification(new String(modificationCombo.getText()));
		wizard
				.setModificationCommand(new String(modificationCommand
						.getText()));
	}

	public IWizardPage getNextPage() {
		saveData();
		ExecuteXUpdatePage page = ((XUpdateWizard) getWizard()).executexupdate_page;
		page.onEnterPage();
		return page;
	}

}
