#!/usr/bin/python
"""
	roXanne Framework simple Python interface
	(c)2005 Amos Brocco <amos.brocco@unifr.ch>

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""
from socket import *
import xml.dom.minidom

class Pyrx:
	def __init__(self):
		self.__sessionId = None
		self.__loginName = None
		self.__password = None
		self.__connection = None
		self.__serverAddress = None
		self.__serverPort = None

	def createSession(self, serverAddress, serverPort=8351, loginName="", password=""):
		""" Creates a new session on the server """
		if serverPort is None:
			serverPort = 8351
		self.__serverAddress = serverAddress
		self.__serverPort = serverPort
		self.__loginName = loginName
		self.__password = password
		try:
			s = socket(AF_INET, SOCK_STREAM)
			s.connect((self.__serverAddress, self.__serverPort))
			s.send("""<?xml version="1.0" encoding="UTF-8"?>
				<roxannequery login="%s" password="%s" 
				persistent="true"/>""" % (self.__loginName, self.__password))
			data = s.recv(8192)
			s.close()
			doc = xml.dom.minidom.parseString(data)
			self.__sessionId = doc.firstChild.getAttribute("sessionId")
		except:
			raise Exception, "Cannot connect to server %s , port %s" % (self.__serverAddress, self.__serverPort)

	def isConnected(self):
		""" Returns true if the session is set """
		return self.__sessionId is not None

	def getSessionId(self):
		""" Returns the session Id """
		return self.__sessionId

	def __execute(self, data, resultSize=8192, mantainSession="true"):
		""" Executes a given command within the current session """
		if self.__sessionId is None:
			raise Exception, "No active session"
		try:
			s = socket(AF_INET, SOCK_STREAM)
			s.connect((self.__serverAddress, self.__serverPort))
			header = """<?xml version="1.0" encoding="UTF-8"?>
				<roxannequery login="%s" password="%s" sessionId="%s"
				persistent="%s">""" % (self.__loginName, self.__password, self.__sessionId, mantainSession)
			s.send(header + data + """</roxannequery>""")
			result = s.recv(resultSize)
			s.close()
			return result
		except:
			raise Exception, "Cannot communicate with server"
			
	def closeSession(self):
		""" Closes the session on the server """
		self.__execute(data="",mantainSession="false")
		self.__sessionId = None
		
	def generateKey(self, methodName, password=None):
		""" Generates a new key """
		cmd = """<requests><action type="genkey" method="%s" password="%s"/></requests>""" % (methodName, password)
		result = self.__execute(data=cmd)
		doc = xml.dom.minidom.parseString(result)
		resultNode = doc.firstChild.firstChild.firstChild.firstChild
		exitCode = resultNode.getAttribute("exitCode")
		if not exitCode == "200":
			raise Exception, "Error on key generation"
		if len(resultNode.childNodes) == 1:
			return [resultNode.firstChild.firstChild.nodeValue]
		else:
			return [resultNode.firstChild.firstChild.nodeValue, resultNode.lastChild.firstChild.nodeValue]

	def addKey(self, methodName, keyId, keyType, keyData):
		""" Adds a key to the server """
		cmd = """<keychain><addkey method="%s" id="%s"><%s>%s</%s></addkey></keychain>""" % (methodName, keyId, keyType, keyData, keyType)	
		result = self.__execute(data=cmd)
		doc = xml.dom.minidom.parseString(result)
		addKeyNode = doc.firstChild.firstChild.firstChild
		result = addKeyNode.getAttribute(keyType)
		if not result == "added":
			raise Exception, "Error while adding key"
		return True

	def removeKey(self, methodName, keyId, keyType):
		""" Removes a key from the server """
		cmd = """<keychain><removekey method="%s" keyId="%s" flag="%s"/></keychain>""" % (methodName, keyId, keyType)
		result = self.__execute(data=cmd)
		doc = xml.dom.minidom.parseString(result)
		addKeyNode = doc.firstChild.firstChild.firstChild
		result = resultNode.getAttribute("keyType")
		if not result == "removed":
			raise Exception, "Error while removing key"
		else:
			return True

	def doXPath(self, xpathExpression, depth="0"):
		""" Perform an XPath search """
		cmd = """<requests><action type="xpath" depth="%s">%s</action></requests>""" % (depth, xpathExpression)
		result = self.__execute(data=cmd)
		print result
		doc = xml.dom.minidom.parseString(result)
		resultNode = doc.firstChild.firstChild.firstChild.firstChild
		return resultNode.childNodes

	def doXUpdate(self, xupdateQuery):
		""" Execute an XUpdate query """
		cmd = """<requests><action type="xupdate" version="1.0" xmlns:xupdate="http://www.xmldb.org/xupdate">%s</action></requests>""" % (xupdateQuery)
		result = self.__execute(data=cmd)
		doc = xml.dom.minidom.parseString(result)
		resultNode = doc.firstChild.firstChild.firstChild.firstChild
		exitCode = resultNode.getAttribute("exitCode")
		if not exitCode == "200":
			description = resultNode.firstChild.nodeValue
			detail = resultNode.lastChild.nodeValue
			raise Exception, "Error on XUpdate execution. %s : %s" %(description, detail)
		return resultNode.childNodes

	def encrypt(self, targetNode, methodName, keyId):
		""" Encrypts a node """
		cmd = """<requests><action type="encrypt" method="%s" keyId="%s">%s</action></requests>""" % (methodName, keyId, targetNode)
		result = self.__execute(data=cmd)
		doc = xml.dom.minidom.parseString(result)
		resultNode = doc.firstChild.firstChild.firstChild.firstChild
		exitCode = resultNode.getAttribute("exitCode")
		if not exitCode == "200":
			description = resultNode.firstChild.nodeValue
			detail = resultNode.lastChild.nodeValue
			raise Exception, "Error on node encryption. %s : %s" %(description, detail)
		return True

	def decrypt(self, targetNode):
		""" Decrypts a node """
		cmd = """<requests><action type="decrypt">%s</action></requests>""" % (targetNode)
		result = self.__execute(data=cmd)
		doc = xml.dom.minidom.parseString(result)
		resultNode = doc.firstChild.firstChild.firstChild.firstChild
		exitCode = resultNode.getAttribute("exitCode")
		if not exitCode == "200":
			description = resultNode.firstChild.nodeValue
			detail = resultNode.lastChild.nodeValue
			raise Exception, "Error on node decryption. %s : %s" %(description, detail)
		return True

	def bindNS(self, prefix, uri):
		""" Binds a namespace """
		cmd = """<requests><action type="nsbind" prefix="%s" uri="%s"/></requests>""" % (prefix, uri)
		result = self.__execute(data=cmd)
		doc = xml.dom.minidom.parseString(result)
		resultNode = doc.firstChild.firstChild.firstChild.firstChild
		exitCode = resultNode.getAttribute("exitCode")
		if not exitCode == "200":
			description = resultNode.firstChild.nodeValue
			detail = resultNode.lastChild.nodeValue
			raise Exception, "Error on namespace binding. %s : %s" %(description, detail)
		return True

	def unbindNS(self, prefix):
		""" Unbinds a namespace """
		cmd = """<requests><action type="nsunbind" prefix="%s"/></requests>""" % (prefix)
		result = self.__execute(data=cmd)
		doc = xml.dom.minidom.parseString(result)
		resultNode = doc.firstChild.firstChild.firstChild.firstChild
		exitCode = resultNode.getAttribute("exitCode")
		if not exitCode == "200":
			description = resultNode.firstChild.nodeValue
			detail = resultNode.lastChild.nodeValue
			raise Exception, "Error on namespace unbinding. %s : %s" %(description, detail)
		return True

	def unbindAllNS(self):
		""" Unbinds all namespaces """
		cmd = """<requests><action type="nsunbindall"/></requests>"""
		result = self.__execute(data=cmd)
		doc = xml.dom.minidom.parseString(result)
		resultNode = doc.firstChild.firstChild.firstChild.firstChild
		exitCode = resultNode.getAttribute("exitCode")
		if not exitCode == "200":
			description = resultNode.firstChild.nodeValue
			detail = resultNode.lastChild.nodeValue
			raise Exception, "Error on all namespaces unbinding. %s : %s" %(description, detail)
		return True



### TEST ROUTINE ###

if __name__ == '__main__':
	a = Pyrx()
	a.createSession(serverAddress="localhost", loginName="Amos", password="test")
	a.isConnected()
	symkey = a.generateKey("AES_128","asimplepassword")
	asymkey = a.generateKey("RSA_V1_5")
	a.addKey("AES_128","myaeskey","encryption",symkey[0])
	a.addKey("AES_128","myaeskey","decryption",symkey[0])
	a.addKey("RSA_V1_5","myrsakey","decryption",asymkey[0])
	a.addKey("RSA_V1_5","myrsakey","encryption",asymkey[1])
	a.bindNS("default","http://www.sample.com/tips/#2005ns")
	a.bindNS("test","http://www.sample.com/tips/#2004ns")
	a.unbindNS("default")
	a.unbindAllNS()
	a.encrypt("/spec/griffin","AES_128","myaeskey")
	a.doXUpdate("""<xupdate:append select="/spec/griffin"><examplenode/></xupdate:append>""")
	a.decrypt("/spec/griffin")
	a.closeSession()
	a.isConnected()
